 !-----------------------------------------------------------------------
 ! Program:   Kvazis
 ! Funkcija:  Transformacija poplavnog talasa 
 ! Opis:      Zadaje se ulazni hidrogram Qu(t) i kriva zapremine V(Qi),
 !            a racuna se izlazni protok Qi(t).
 !
 !            Racunski model : 
 !            Metoda kvazi-ustaljenog tecenja
 !            (iterativno resavanje jednacine kontinuiteta).
 !            Osnovna pretpostavka: zapremina deonice iskljucivo
 !            zavisi od nizvodnog protoka.
 !                            
 !            Racunska metoda: konacne razlike, polovljenje intervala
 !
 ! Autor:     M. Jovanovic, 2.10.1987, 19.12.1998, 30.05.1999.
 ! Okruzenje: Lahey Fortran90, LF90, V.2.00a, 1995.
 !            Microsoft Developer Studio, Visual Fortran V.6.6 2001.
 !----------------------------------------------------------------------



 !----------------------------------------------------------------------
  MODULE zapremina_deonice
 
     implicit none
     real, allocatable, dimension(:) :: QQ, VV
     integer :: nVQ
    
     contains

     SUBROUTINE ucitavanje_kriva_zapremine ( ir, igv )
        implicit none
        integer, intent(in) :: ir, igv
        integer, parameter  :: nVQmax = 100
        integer             :: i

        read( ir, * ) nVQ
        if( nVQ > nVQmax ) stop 'Premasen broj tacaka krive zapremine!'
        allocate( QQ(nVQ), VV(nVQ) )
        read( ir, * ) ( QQ(i), i = 1, nVQ ) ! izlazni protok
        read( ir, * ) ( VV(i), i = 1, nVQ ) ! zapremine deonice (mil. m3)
        do i = 1, nVQ
           write( igv, * ) QQ(i), VV(i)
        enddo
        VV = VV * 1.e+6                  
     END SUBROUTINE ucitavanje_kriva_zapremine

  END MODULE zapremina_deonice
 !---------------------------------------------------------------------- 



 !----------------------------------------------------------------------
  MODULE ulazni_hidrogram

     implicit none
     real, allocatable, dimension(:) :: tu, Qu
     real    :: Qmin, Qmax
     integer :: nQt

     contains


     SUBROUTINE ucitavanje_ulazni_hidrogram( ir )
        implicit none
        integer, intent(in) :: ir
        integer, parameter  :: nQtmax = 100
        integer             :: i

        read( ir, * ) nQt
        if( nQT > nQtmax ) stop 'Premasen broj tacaka krive Qu(t)!'
        allocate( tu(nQt), Qu(nQt) )
        read( ir, * ) ( tu(i), i = 1, nQt )  ! vreme u satima
        read( ir, * ) ( Qu(i), i = 1, nQt )  ! protok (m3/s)
     END SUBROUTINE ucitavanje_ulazni_hidrogram



     SUBROUTINE ekstremne_vrednosti
        implicit none
        Qmin = MINVAL (Qu)
        Qmax = MAXVAL (Qu)
     END SUBROUTINE ekstremne_vrednosti

  END MODULE ulazni_hidrogram
 !----------------------------------------------------------------------



 !----------------------------------------------------------------------
  MODULE registrovani_hidrogrami

     implicit none
     real, allocatable, dimension(:) :: tur, Qr1, Qr2
     integer                         :: nQtr

     contains


     SUBROUTINE ucitavanje_reg_hidrogrami( ir )
        implicit none
        integer, intent(in) :: ir
        integer, parameter  :: nQtrmax = 100
        integer             :: i
        read( ir, * ) nQtr
        if( nQtr > nQtrmax ) stop 'Premasen broj tacaka reg. hidrograma!'
        allocate( tur(nQtr), Qr1(nQtr), Qr2(nQtr) )
        read( ir, * ) ( tur(i), i = 1, nQtr )  ! vreme u satima
        read( ir, * ) ( Qr1(i), i = 1, nQtr )  ! ulazni  protok (m3/s)
        read( ir, * ) ( Qr2(i), i = 1, nQtr )  ! izlazni protok (m3/s)
     END SUBROUTINE ucitavanje_reg_hidrogrami



     SUBROUTINE zapremina( ir, iw, igv )
        implicit none
        real    :: Qusr, Qisr, delV, dt1, Vrpoz, Vrneg
        integer, intent(in) :: ir, iw, igv
        integer :: i
        read( ir, * ) dt1    ! korak ocitavanja (sati)
        dt1 = dt1 * 3600.
        Vrpoz= 0.;  Vrneg= 0.
        write(iw, 100)                                     
        100 format(/5x,'Vreme',5x,'Ulaz',5x,'Izlaz',6x,    &
        'V-punj.',3x,'V-praz.'/4x,'(sati)',4x,'(m3/s)',4x, &
        '(m3/s)',4x,'(mil. m3)',2x,'(mil. m3)'/)
        write(iw, "(5f10.2)" )                             &
        0*dt1/3600., Qr1(1), Qr2(1), Vrpoz, Vrneg
        write(igv, *) Qr2(1), Vrpoz
        do i = 2, nQtr
           Qusr = 0.5 * ( Qr1(i) + Qr1(i-1) )
           Qisr = 0.5 * ( Qr2(i) + Qr2(i-1) ) 
           delV = ( Qusr - Qisr ) * dt1
           if( delV > 0. )  then
              Vrpoz = Vrpoz + delV
              Vrneg = 0.
           else
              Vrpoz = 0.
              Vrneg = Vrneg + delV
           endif
           write(iw, "(5f10.2)") (i-1)*dt1/3600.,          &
           Qr1(i), Qr2(i), Vrpoz/1.e6, Vrneg/1.e6
           if( Vrpoz > 0. ) write(igv, *) Qr2(i), Vrpoz/ 1.e+6
        enddo
     END SUBROUTINE zapremina


  END MODULE registrovani_hidrogrami
 !----------------------------------------------------------------------  





 !----------------------------------------------------------------------  
  MODULE racunski_parametri

     implicit none
     integer, parameter :: ndtmax = 1000
     real               :: tpoc, tkraj, dt, tt   ! (sati)
     integer            :: ndt, npr

     contains


     SUBROUTINE ucitavanje_rac_parametri( ir )
        implicit none
        integer, intent(in) :: ir

        read( ir, * ) tpoc, tkraj, dt, npr
        if( tpoc >= tkraj ) stop 'Greska u vremenskim granicama!'
        tt = (tkraj - tpoc) * 3600.         ! trajanje
        dt   = dt   * 3600.                 ! vremenski korak
        ndt = INT( tt / dt )                ! broj koraka
        if( ndt > ndtmax ) stop 'Premasen maks. broj racunskih koraka!'
     END SUBROUTINE ucitavanje_rac_parametri

  END MODULE racunski_parametri
 !----------------------------------------------------------------------  




 !----------------------------------------------------------------------  
  MODULE racun
     use zapremina_deonice
     use racunski_parametri    

     implicit none
     real,    allocatable, dimension(:) :: Qul, Qiz, V, tw
     integer, allocatable, dimension(:) :: nitw
     real, parameter  :: eps = 0.01
     real             :: t, Qusr
     integer          :: j, nit, err

     contains



     FUNCTION fv( Qis )
        implicit none
        real, external :: flin1
        real :: Qis, Vs, fv
        Vs = flin1 ( nVQ, QQ, VV, Qis, err )
        if( err == 1 ) stop 'f: Ekstrapolacija krive zapremine!'
        fv = Qusr * dt - 0.5 * dt * ( Qiz(j-1) + Qis ) + V(j-1) - Vs 
     END FUNCTION fv



     SUBROUTINE neli(xd, xg, f, xo, eps, nit, ier)
        implicit none
        real,    external    :: f
        real,    intent(in)  :: xd, xg, eps
        real,    intent(out) :: xo
        integer, intent(out) :: nit, ier
        integer              :: it
        real                 :: xa, xb, xp, yd, yg, yo, p, po

        xa = xd;   xb = xg;   xp = xa
        if( xa >= xb ) then
           xa = xb;   xb = xp
        endif
        yd = f( xa );   yg = f( xb )
        p = yd * yg
        if( p == 0. ) then
           if( yd == 0. ) xo = xa
           if( yg == 0. ) xo = xb
        nit = 1; ier = 0;  return
        elseif( p < 0. ) then
           nit = INT( LOG( ( xb-xa )/eps )/LOG( 2. ) ) + 1
           do it = 1, nit
              xo = 0.5 * ( xa+xb ); yo = f( xo )
              po = yd * yo
              if( po == 0. ) then
                 nit = it; ier = 0;  return
              elseif( po < 0. ) then
                 xb = xo; yg = yo
              else
                 xa = xo; yd = yo
              endif
           enddo
           xo = 0.5 * ( xa + xb )
           nit = it; ier = 0; return 
        else
           ier = 1; return
        endif
     END SUBROUTINE neli               


  END MODULE racun
 !----------------------------------------------------------------------  





  FUNCTION flin1( nx, x, y, xx, err )
     implicit none
     integer,               intent( in )  :: nx
     real, dimension( nx ), intent( in )  :: x, y
     real                                 :: xx, flin1
     integer                              :: i, ip, ik, err

     if( xx < x(1)  .or.  xx > x(nx) ) then
        err = 1; return
     endif
     do i = 2, nx
        if( xx <= x (i) ) then
           ip = i-1;  ik = i;  exit
        endif
     enddo
     flin1 = y(ip) + ( y(ik)-y(ip) )/( x(ik)-x(ip) ) * (xx-x(ip) )
     err = 0
  END FUNCTION flin1   





  PROGRAM Kvazis
     use ulazni_hidrogram
     use registrovani_hidrogrami
     use racun

     implicit none
     integer        :: kod
     real, external :: flin1
     character(75)  :: text
     character(12)  :: datio, datu1, dati2, dati3


     ! Deklaracija I/O fajlova
     print "(5x,A)", "Ime fajla sa imenima I/O fajlova? "
     read "(A)", datio
     open( UNIT = 10, FILE = datio, STATUS = "old" )
     read( 10,"(A)" ) datu1         ! ulazni podaci
     read( 10,"(A)" ) dati2         ! rezultati
     read( 10,"(A)" ) dati3         ! kriva zapremine
     open( UNIT = 1, FILE = datu1, STATUS = "old" )
     open( UNIT = 2, FILE = dati2, STATUS = "replace" )
     open( UNIT = 3, FILE = dati3, STATUS = "replace" )
   
     
	 ! Ucitavanje ulaznih podataka
     read ( 1, "(A)" ) text
     write( 2, "(A)" ) text
     read ( 1, * ) kod  ! proracun: 0-zapremine; 1-transformacije
     if( kod /= 0   .and.  kod /= 1 ) stop ' kod?'

     SELECT CASE (kod == 0)
        CASE (.true.)
           call ucitavanje_reg_hidrogrami( 1 )
           call zapremina( 1, 2, 3 )
        CASE (.false.)
           call ucitavanje_kriva_zapremine( 1, 3 )
           call ucitavanje_ulazni_hidrogram( 1 )
           call ucitavanje_rac_parametri( 1 )
           call ekstremne_vrednosti

           allocate ( Qul(ndt), Qiz(ndt), V(ndt), tw(ndt), nitw(ndt) )


           ! Pocetni uslovi
           t = tpoc; j = 1
           Qul(1) = flin1 ( nQt, tu, Qu, t, err )
           if( err == 1 ) stop 'Estrapolacija ulaznog hidrograma!'
           Qiz( 1 ) = Qul( 1 )
           V( 1 ) = flin1 ( nVQ, QQ, VV, Qiz( 1 ), err )
           if( err == 1 ) stop 'Ekstrapolacija krive zapremine!'
		   tw  ( 1 ) = t
           nitw( 1 ) = 0


           ! Ciklus po vremenu
           do j = 2, ndt
              t = ( j-1 ) * dt / 3600.
              Qul( j ) = flin1 ( nQt, tu, Qu, t, err )
              if( err == 1 ) stop 'Eksptrapolacija krive zapremine!'
              Qusr = 0.5 * ( Qul( j-1 ) + Qul( j ) ) 
              call neli( Qmin, Qmax, fv, Qiz( j ), eps, nit, err )
              if( err == 1 ) stop 'Resenje ne konvergira!'           
              V( j ) = flin1 ( nVQ, QQ, VV, Qiz(j), err )
              if( err == 1 ) stop 'Ekstrapolacija krive zapremine!'
              tw( j ) = t
              nitw( j ) = nit
           enddo


           ! Stampanje rezultata
           print 100
           write( 2, 100 )
           100 format(/8x,'Br.',7x,'t (h)',7x,'Qu (m3/s)', &
           5x,'Qi (m3/s)',4x,'V (mil.m3)',3x,'nit')
           200 format(5x, i4, 4f14.2, i8)

           do j = 1, ndt
              if( j / npr * npr == j ) then
                 print 200,      j, tw(j), Qul(j), Qiz(j), V(j)/1.e+6, nitw(j)
                 write( 2, 200 ) j, tw(j), Qul(j), Qiz(j), V(j)/1.e+6, nitw(j)
              endif
           enddo
     END SELECT

     close(1); close(2); close(3)     
  END PROGRAM Kvazis





