 !-----------------------------------------------------------------------
 ! Program:   Muski
 ! Funkcija:  Transformacija poplavnog talasa 
 ! Opis:      Zadaje se ulazni hidrogram Q1(t) i parametri krive
 !            zapremine, a racuna se izlazni protok Q2(t).
 !
 !            Racunski model: Metoda Muskingum            
 !            Osnovna pretpostavka: zapremina deonice zavisi od 
 !            uzvodni i nizvodnog protoka: V = K [ X Q1 + (1-X) Q2 ]
 !                            
 !            Racunska metoda: Optimizacija parametara X i K pomocu 
 !            statisticke analize registrovanih hidrograma, a resavanje 
 !            jednacine kontinuiteta Metodom konacnih razlika.
 !
 ! Autor:     M. Jovanovic, 10.05.1987, 19.01.1991. 20.09.1999.
 ! Okruzenje: Lahey Fortran90, LF90, V.2.00a, 1995.
 !            Microsoft Developer Studio, Visual Fortran V.5, 1997.
 !----------------------------------------------------------------------


 !----------------------------------------------------------------------
  MODULE hidrogrami

     implicit none
     real, allocatable, dimension(:) :: Q1, Q2r
     integer :: n1, n2

     contains


     SUBROUTINE ucitavanje_ulazni_hidrogram
        implicit none
        integer, parameter  :: n1max = 200
        integer             :: i
        read( 1, * ) n1
        if( n1 > n1max ) stop 'Previse tacaka ulaznog hidrograma!'
        allocate( Q1( n1 ) )
        read( 1, * ) ( Q1( i ), i = 1, n1 )  ! protok (m3/s)
     END SUBROUTINE ucitavanje_ulazni_hidrogram



     SUBROUTINE ucitavanje_reg_izl_hidrogram
        implicit none
        integer, parameter  :: n2max=200
        integer             :: i
        read( 1, * ) n2
        if( n2 > n2max ) stop 'Previse tacaka reg. izl. hidrograma!'
        allocate( Q2r( n2 ) )
        read( 1, * ) ( Q2r( i ), i = 1, n2 )  ! reg. izlazni protok (m3/s)
        if( n1 < n2 ) then
           do i = n1+1, n2
              Q1( i ) = Q1( 1 )
              n1 = n2
           enddo
        endif
     END SUBROUTINE ucitavanje_reg_izl_hidrogram


  END MODULE hidrogrami
 !----------------------------------------------------------------------





 !----------------------------------------------------------------------  
  MODULE parametri

  !....................................................................
  ! U nedostatku podataka, iskustveno se preporucuje X = 0.2.
  ! Parametar K ima dimenziju vremena (predstavlja vreme putovanja 
  ! talasa na datoj deonici i zadaje se u ISTIM JEDINICAMA kao dt (h)
  ! (ta vremenska jedinica, ne mora biti ista kao ona za protok).
  !....................................................................

     implicit none
     real    :: e1, e2 ! greska zaokruzenja Q1 i Q2
     real    :: delta  ! prihvatljivo odstupanje (0.20)
     real    :: Xo, Ko ! racunske vrednosti parametara X i K               
     real    :: dt     ! racunski (vremenski) korak
     integer :: nX     ! broj vrednosti X koje se ispituju
     integer, parameter :: nXmax = 10  ! maksimalni broj nX
     real, allocatable, dimension(:) :: X 

     contains



     SUBROUTINE ucitavanje_parametri ( kod )
        implicit none
        integer :: i, kod

        read( 1, * ) dt    
        SELECT CASE ( kod )
           CASE ( 0 )  ! zadate vrednosti parametara
              read( 1, * ) Xo, Ko
           CASE ( 1 )  ! optimizacija vrednosti parametara
              read( 1, * ) e1, e2, delta
              read( 1, * ) nX
              if( nX > nXmax ) stop 'Premasen broj nX!'
              allocate ( X(nX) )
              read( 1, * ) ( X( i ), i = 1, nX )
        END SELECT
     END SUBROUTINE ucitavanje_parametri


  END MODULE parametri
 !----------------------------------------------------------------------  



 !----------------------------------------------------------------------  
  MODULE statistike

  !......................................................................
  ! Parametri X i K se kalibrisu na osnovu registrovanih hidrograma
  ! iz proslosti; za niz pretpostavljenih vrednosti X (0,0.5) racunaju
  ! se vrednosti parametra K:
  !
  !     ( Q1(i) + Q1(i+1) ) - ( Q2(i) + Q2(i+1) )                 BR
  ! K = ------------------------------------------------------ = ----,
  !     2*( X*( Q1(i+1)-Q1(i) ) + (1-X)*( Q2(i+1)-Q2(i) ) )/dt    IM
  !
  ! gde je i -- vremenski indeks. Vrednosti brojioca BR i imenioca IM
  ! se izborom vrednosti X podesavaju sve dok se (sto je vise moguce) ne
  ! uspostavi njihova LINEARNA zavisnost, pri cemu je onda vrednost K,
  ! nagib prave.
  ! 
  ! Optimizacija parametara X i K se moze obaviti racunski iz uslova da je
  ! njihova optimalna kombinacija ona, za koju se dobija najmanja vrednost
  ! standardne devijacije parametra K Objasnjenje ovog postupka se moze naci 
  ! u knjizi: Croley, T.E. "Hydrologic and Hydraulic Calculation in BASIC 
  ! for small computers", University of Iowa, Iowa City, 1980.
  !.......................................................................

     use hidrogrami
     use parametri    

     contains


     SUBROUTINE optimizacija
        implicit none
        integer :: i, j, n
        integer, dimension(nX) :: nn
        real :: pragBR, BR, IM, K, sumaK, sumaK2, sigmaKmin
        real, dimension(nX) :: pragIM, Ksr, SigmaK

        pragBR = SQRT( 2.*( e1**2 + e2**2 ) ) / delta

        write( 2, fmt="(/5x,A/5x,A,f12.4)" )       &
        "Kalibracija parametara:",                 &
        "Prag tolerancije brojioca:", pragBR
        write( 2, fmt="(/5x,A/5x,A/)" )            &
        "Parametar  Broj    Prag toler.  Parametar   St.devijacija",&
        "    X      tacaka   imenioca      Ksr          SigmaK    "

        do j = 1, nX
           pragIM( j ) = SQRT( 2.*( (X( j )   * e1 )**2 +             &
		                     ( (1. - X( j ) ) * e2 )**2 ) )/ delta
           sumaK   = 0.
           sumaK2  = 0.
           n = 0
           do i = 1, n2 - 1
              BR = Q1( i ) + Q1( i+1 ) - Q2r( i ) - Q2r( i+1 )
              if( ABS( BR ) > pragBR ) then
                 IM = 2./dt * (      X( j )  * ( Q1(  i+1 ) - Q1(  i ) ) + &
                              ( 1. - X( j )) * ( Q2r( i+1 ) - Q2r( i ) )   )
                 if( ABS( IM ) > pragIM( j ) ) then
                    K = BR / IM
                    sumaK  = sumaK  + K
                    sumaK2 = sumaK2 + K*K
                    n = n + 1
                 endif
              endif
           enddo  
           Ksr( j )    = sumaK / n
           SigmaK( j ) = SQRT( sumaK2/n - Ksr( j )**2 )
           nn( j )     = n 
           write( 2, fmt="(5x,f7.3,i7,3f13.4)"  ) &
           X( j ), nn( j ), pragIM( j ), Ksr( j ), SigmaK( j )
        enddo
        SigmaKmin = MINVAL( SigmaK )
        do j = 1, nX
           if( ( SigmaK( j ) - SigmaKmin ) ==  0. ) then
              Xo = X( j )
              Ko = Ksr( j )
           endif
        enddo 
     END SUBROUTINE optimizacija

  END MODULE statistike
 !----------------------------------------------------------------------  




 PROGRAM Muski
    use hidrogrami
    use parametri
    use statistike

    implicit none
    character(75) :: text
    character(12) :: datio, datu, dati
    integer       :: i, kod
    real          :: t, C1, C2, C3, C4   
    real, allocatable, dimension(:) :: Q2


    ! Ucitavanje ulaznih podataka
    print "(5x,A)", "Ime fajla sa imenima I/O fajlova? "
    read "(A)", datio
    open( UNIT = 10, FILE = datio, STATUS = "old" )
    read( 10, "(A)" ) datu    ! ulazni podaci
    read( 10, "(A)" ) dati    ! rezultati
    open( UNIT = 1, FILE = datu, STATUS = "old" )
    open( UNIT = 2, FILE = dati, STATUS = "replace" )

    read ( 1, "(A)" ) text
    write( 2, "(A)" ) text
    read ( 1, * ) kod 
    if( kod /= 0 .and. kod /= 1 ) stop 'kod? '

    !................................................................
    ! kod = 0: proracun transformacije sa zadatim vrednostima Xo, Ko;
    !     = 1: optimizacija X i K na osnovu registrovanih hidrograma 
    !................................................................
 
    call ucitavanje_ulazni_hidrogram
    call ucitavanje_parametri ( kod )
    n2 = n1;   allocate (  Q2( n2 ) );   Q2( 1 ) = Q1( 1 )         


    ! Optimizacija parametara    
    if( kod == 1 ) then
       call ucitavanje_reg_izl_hidrogram
       call optimizacija
       Q2( 1 ) = Q2r( 1 )
    endif


    ! Stampanje racunskih parametara
    write( 2, fmt="(/5x,A,3( /5x,A,f6.2 ) )" ) &
    "Racunski parametri:",                     &
    "- tezinski faktor  X  = ", Xo,            &
    "- vreme putovanja  K  = ", Ko,            &
    "- vremenski korak  dt = ", dt 


    ! Proracun
    call koeficijenti_Muskingum

    t = 0.
    if( kod == 0 ) then    
       write( 2, fmt="(5x,A)" )    &
       "Vreme (h)    Qul (m3/s)    Qiz (m3/s)"
    else
       write( 2, fmt="(5x,A)" )    &
       "Vreme (h)    Qul (m3/s)    Qiz (m3/s)    Qiz-r (m3/s)"
    endif

    do i = 1, n2-1
       if( kod == 0 ) then
          write( 2, fmt="(f11.2,2f14.2)" ) t, Q1( i ), Q2( i )             
       else
          write( 2, fmt="(f11.2,3f14.2)" ) t, Q1( i ), Q2( i ), Q2r( i )
       endif
       t = t + dt
       Q2( i+1 ) = C1 * Q1( i+1 ) + C2 * Q1( i ) + C3 * Q2( i )       
    enddo
    close( 1 ); close( 2 )


    contains


    SUBROUTINE koeficijenti_Muskingum
       C4 = 2. * Ko * ( 1.-Xo ) + dt
       C1 = ( dt - 2. * Ko * Xo ) / C4
       C2 = ( dt + 2. * Ko * Xo ) / C4
       C3 = ( 2. * Ko * (1.-Xo) - dt ) / C4 
       write( 2, fmt="(/5x,A)" ) "Koeficijenti:"
       write( 2, fmt="(5x,3(A,f8.4)/)" ) &
       "C1 = ",C1,"   C2 = ",C2,"   C3 = ",C3
    END SUBROUTINE koeficijenti_Muskingum


 END PROGRAM Muski
