 !----------------------------------------------------------------------------------------
 ! Program:   Nanos
 ! Funkcija:  Proracun transportnog kapaciteta vodotoka za nanos
 !            i opste deformacije recnog korita
 ! Opis:      Hidraulicki elementi posmatrane deonice preuzimaju se iz programa 
 !            Reke1 (izlazni fajl "__ime__.out").
 !            Za proracun pronosa nanosa na raspolaganju su metode:
 !            
 !               o Meyer-Peter i Muller;
 !               o Van Rijn;
 !               o Engelund-Hansen. 
 !            
 !            Proracun se obavlja po frakcijama ili na osnovu srednjeg precnika.
 !              
 !            Opsta deformacija korita racuna se pomocu bilansne jednacine
 !            za nanos (Exner-ove jednacine):
 !
 !               ( 1 - por ) * dAs /dt + dQv / dx = 0,
 !            
 !            gde je: por - poroznost nanosa, As - povrsina izmedju konture
 !            dna i proizvoljne referentne ravni, a Qv - zapreminski pronos
 !            nanosa.
 !
 !            Primenom konacnih razlika, promena povrsine dna u profilu "i" je:
 !                           
 !               DeltaAs_i = c * ( Qvm_i-1 - Qvm_i ) / Deltax_i-1, 
 !
 !            gde je: c = dt / (ros * ( 1 - por ) ), a Qvm - maseni pronos 
 !            nanosa u profilima "i" i "i-1".
 !
 !            Korekcija vertikalnih koordinata tacaka koje definisu geometriju 
 !            proila, odredjuje se na osnovu vrednosti DeltaAs, proporcionalno 
 !            lokalnoj dubini:         
 !
 !               Deltazp_j = ( Z - zp_j ) * ( DeltaAs / A ),
 !
 !            gde je: A - vodena povrsina poprecnog profila, Z - kota nivoa,
 !            a zp - kota dna u vertikali "j".
 !            
 ! Autor:     M. Jovanovic, avg. 1978, jun 1986, okt. 1993, okt. 1999., okt. 2002.
 ! Okruzenje: Lahey Fortran90, LF90, V.2.00a, 1995.
 !            Microsoft Developer Studio, Visual Fortran, V.6,6.0, 2001.
 !----------------------------------------------------------------------------------------





 !----------------------------------------------------------------------------------------
  MODULE Profili

     implicit none
     character(75)             :: text
     integer                   :: nprof, ip, j, irun, eof
     integer, parameter        :: maxt = 50
     real,    parameter        :: g = 9.81, ro = 1.

     TYPE :: Profil
        real                   :: xs   ! stacionaza (m)
        character(12)          :: oz   ! oznaka profila
        integer                :: nt   ! broj koordinatnih tacaka
        real, dimension (maxt) :: zp   ! kote dna
        real, dimension (maxt) :: yp   ! odstojanje
        real, dimension (maxt) :: dzp  ! vertikalna korekcija kote dna
        real, dimension (maxt) :: zpn  ! kote dna nakon deformacije
        real                   :: Ie   ! lokalni nagib linije energije
		real                   :: CMan ! Manning-ov koeficijent
        real                   :: dx   ! pripadajuce odstojanje
        real                   :: Q    ! protok
        real                   :: Z    ! racunska kota nivoa
        real                   :: A    ! povrsina preseka
        real                   :: B    ! sirina vodenog ogledala
        real                   :: O    ! okvaseni obim
        real                   :: R    ! hidraulicki radijus
        real                   :: V    ! srednja brzina
        real                   :: E    ! kota energije
	    real                   :: Tauo ! tangencijalni napon na dnu
        real                   :: dAd  ! ukupna deformacija u profilu
     END TYPE Profil

     TYPE( Profil ), allocatable, dimension (:) :: pp


     contains


     SUBROUTINE UcitavanjeProfila ( ir1 )
        integer, intent( in ) :: ir1

        read( ir1, "(A)") text
        read( ir1, *    ) nprof
        allocate( pp( nprof ) )
        do ip = 1, nprof
           read( ir1, * )  j, pp( ip )%xs, pp( ip )%nt, pp( ip )%oz
           if( pp( ip )%nt > maxt ) then
              print *, ' Profil broj :', ip, pp( ip )%oz
	          stop     ' Premasen maksimalni broj koordinata !'
           endif
           read( ir1, * ) ( pp( ip )%zp( j ), j = 1, pp( ip )%nt )
           read( ir1, * ) ( pp( ip )%yp( j ), j = 1, pp( ip )%nt )
           do j = 1, pp( ip )%nt - 1             ! provera podataka
              if(  ( pp( ip )%yp( j+1 ) - pp( ip )%yp( j ) )  < 0.  ) then
                 print *, ' Profil broj :', ip, pp( ip )%oz
	             stop     ' Greska u podacima! <y> ??'
              endif
           enddo
        enddo
     END SUBROUTINE UcitavanjeProfila





     SUBROUTINE UcitavanjeHidraulickihElemenata( ir11, irun )
        integer, intent( in ) :: ir11, irun
        if( irun == 1 ) read( ir11, "(A)") text
        read( ir11, fmt="(///)", iostat = eof )
 	    do ip = 1, nprof
           read( ir11, 800, iostat = eof ) j,                             &
	       pp( j )%oz, pp( j )%xs,   pp( j )%Q, pp( j )%Z, pp( j )%E,     &
           pp( j )%A,  pp( j )%B,    pp( j )%O, pp( j )%R, pp( j )%V,     &
           pp( j )%Ie, pp( j )%CMan, pp( j )%Tauo
        end do
        800 format( i4,A,f10.3,f9.2,2f9.3,3f8.2,2f6.2,f10.6,f7.3,f8.4 )
     END SUBROUTINE UcitavanjeHidraulickihElemenata





     SUBROUTINE RazmakProfila
        do ip = nprof, 2, -1
           pp( ip )%dx = pp( ip-1 )%xs - pp( ip )%xs 
        end do
        pp( 1 )%dx = pp( 2 )%dx
     END SUBROUTINE RazmakProfila



   END MODULE Profili
 !----------------------------------------------------------------------------------------







 !----------------------------------------------------------------------------------------
  MODULE TransportniKapacitetNanos
     use Profili


     character(75)                      :: text2
     integer, parameter                 :: maxnp_grf = 10
     integer                            :: kodm, kodd, nfrak, np_grf
     real                               :: ros, roz, Delta, por
     real                               :: d50, d90, d50v, d90v, d50s, pSum
	 real                               :: dzvv, dzvs, W, Visk
     real                               :: Temp, Kappa, Tauoc
     real                               :: Qdef, Q1nanos, Q1dn, dt
     real                               :: Qvm, QvmDan, Qsm, Qum, QumDan
	 real                               :: nr, qvm_frak 
	 real,    allocatable, dimension(:) :: QrQ, BvB 
     real,    allocatable, dimension(:) :: d, p
     real,    allocatable, dimension(:) :: NagibPrave, Theta, cMP, Tauc
     real,    allocatable, dimension(:) :: Qnanos
     integer, allocatable, dimension(:) :: ip_grf



     contains


     SUBROUTINE AlociranjeNizaNanos
        allocate(   Qnanos( nprof ), QrQ( nprof), BvB( nprof), ip_grf( maxnp_grf ) )
     END SUBROUTINE



     SUBROUTINE UcitavanjePodatakaNanos( ir12, iw13, iw14 )
        integer, intent( in ) ::  ir12, iw13, iw14
        read ( ir12, "(A)"  ) text2              ! komentar 
        write( iw13, "(A/)" ) text2
		write( iw14, "(A/)" ) text2

        read( ir12, * ) nfrak                    ! broj frakcija
        allocate( d( nfrak ), p( nfrak ) )       ! dimenzionisanje nizova
	    read( ir12, * ) ( d( j ), j = 1, nfrak ) ! precnik [mm]
        read( ir12, * ) ( p( j ), j = 1, nfrak ) ! zastupljenost [%]
                     
        pSum = 0.                               
        do j = 1, nfrak
	       d( j ) = d( j ) / 1000.               ! precnik [m]
	       p( j ) = p( j ) / 100.                ! zastupljenost [-]
	       pSum = pSum + p( j )
        end do
        if( pSum /= 1.) stop 'Zastupljenost svih frakcija nije 100%  ?'

        read( ir12, * ) ros, por                 ! gustina [t/m3], poroznost [%]
		por   = por / 100                        ! poroznost [-]
        roz   = ros * ( 1. - por )               ! zapreminska masa u suvom [t/m3]
        Delta = ( ros - ro ) / ro                ! relativna gustina [-]
		                     
        read( ir12, * ) kodm                     ! metoda: 1-MPM, 2-vanRijn, 3-EH                                                                  
        if( kodm /= 1 .and. kodm /= 2 .and. kodm /= 3 ) stop 'Proveri kodm !'
		if( kodm == 2 .and. nfrak > 1 ) stop 'Metoda van Rijn-a iziskuje nfrak = 1 !'


        SELECT CASE( kodm )

           case( 1 ) ! Mayer-Peter & Muller		      
              allocate( NagibPrave( nfrak ), Theta( nfrak ), cMP( nfrak ), Tauc( nfrak ) )
              read( ir12, * ) ( NagibPrave( j ), j = 1, nfrak ) ! parametar MPM
              read( ir12, * ) ( Theta( j ),      j = 1, nfrak ) ! parametar MMM

		      write( iw14, 102 )
	          102 format(/1x,'Podaci po frakcijama'//                                    &
	          1x,'Frak.',3x,'d',6x,'p',4x,'NagibPrave',3x,'Theta',6x,'Tauc',6x,'cMP'/    &
		      8x,'(mm)',3x,'(%)',6x,'(-)',8x,'(-)',7x,'(Pa)',5x,'(...)'/ )          

              do j = 1, nfrak
                 cMP ( j ) = NagibPrave( j )**(-1.5) * ro**(-0.5) * ( ros/(ros-ro) ) / g
                 Tauc( j ) = Theta( j ) * g * ( ros - ro ) * d( j )  
                 write( iw14, 201 )  j, d( j ) * 1000., p( j ) * 100,    &
			     NagibPrave( j ), Theta( j ), Tauc( j ) * 1000., cMP( j )
                 201 format(i4,3f8.2,f11.4,2e12.4)
              enddo             
              write( iw14, "(//1x,A)" ) "Transportni kapacitet po frakcijama kg/(sm):"

              read( ir12, * ) d90                           ! krupnoca najveceg zrna [mm]
              d90 = d90 / 1000.                             ! krupnoca njeveceg zrna [m]
              read( ir12, * ) ( QrQ( i ), i = 1, nprof)     ! korek. faktori za protok
			  read( ir12, * ) ( BvB( i ), i = 1, nprof)     ! korek. faktori za pojas vucenja

           case( 2 ) ! VanRijn
		      d50v = d( nfrak )                             ! srednja krupnoca vuc.nanosa [m]
              read( ir12, * ) d90v                          ! najveca krupnoca vuc.nanosa [mm]
			  read( ir12, * ) d50s                          ! srednja krupnoca sus.nanosa [mm]
              d90v = d90v / 1000.                           ! najveca krupnoca vuc.nanosa [m]
              d50s = d50s / 1000.		                    ! srednja krupnoca sus.nanosa [m]
              read( ir12, * ) Temp, Kappa                   ! temper,vode, Karman-ova konst.
			  read( ir12, * ) ( BvB( i ), i = 1, nprof)     ! korekcioni faktori ( <= 1 )
			  Visk = Viskoznost( Temp )                     ! kinematicka viskoznost [m2/s] 
              dzvv = d50v * ( g * Delta / Visk**2 )**0.3333 ! parametar zrna vuc.nanosa [-]
              dzvs = d50s * ( g * Delta / Visk**2 )**0.3333 ! parametar zrna sus.nanosa [-]
              Tauoc = g * ( ros-ro ) * d50v * Thetac( dzvv )! kriticni tau napon [kPa]
              W = fw2( d50s )                               ! brzina tonjenja [m/s]

			  write(iw14, 203) d50v * 1000., dzvv, d90v * 1000., Tauoc
              203 format( /1x,'Podaci o vucenom nanosu:  d50 =',f6.2,' mm    d* =',f6.2,&
			  '   d90 =',f6.2,' mm','   Tauoc =',f8.4,' kPa' // )
			  write(iw13, 204) d50s * 1000., dzvs, Visk, W
              204 format( /1x,'Podaci o suspendovanom nanosu: d50 =',f6.2,' mm    d* =',&
              f6.2,'   Visk =',e10.3,'   W =',f8.3,' m/s' // )

           case( 3 ) ! Engelund-Hansen
		      write( iw14, 302 )
	          302 format(/1x,'Podaci po frakcijama'//      &
			  1x,'Frak.',3x,'d',6x,'p'/ 8x,'(mm)',3x,'(%)'/ )          
              do j = 1, nfrak
			     write( iw14, 201 )  j, d( j ) * 1000., p( j ) * 100 
                 301 format(i4,2f8.2)
              enddo             
              write( iw14, "(//1x,A)" ) "Transportni kapacitet po frakcijama kg/(sm):"

        END SELECT

        read( ir12, * ) kodd                     ! deformacija: 0-ne 1-da               
        if( kodd /= 0 .and. kodd /= 1 ) stop 'Proveri kodd !'
        if( kodd == 1 ) then
		   read( ir12, * ) Qdef                  ! protok za proracun opste deformacije
           read( ir12, * ) Q1nanos               ! ulazni tovar [kg/s] 
		   Q1dn = Q1nanos * 86.400               ! ulazni tovar [t/dan]
		   read( ir12, * ) dt                    ! vremenski korak [dani]
           read( ir12, * ) np_grf                ! ukupno profila za graf. prikaz deform,
           if( np_grf > maxnp_grf ) stop 'Broj profila za graficki prikaz deformacije ?'
           read( ir12, * ) ( ip_grf( i ), i = 1, np_grf ) ! brojevi profila
        endif
     END SUBROUTINE UcitavanjePodatakaNanos




  
     SUBROUTINE MeyerPeterMuller( iw13, iw14 )
        integer, intent( in )  :: iw13, iw14
        real :: ko, Vsm, CMan, Tauo, Bv, Sumqvm
          
        ! Konstante
		pp( ip )%Tauo = pp( ip )%Tauo / 1000.    ! tang. napon na dnu [kPa]
        Vsm = SQRT( pp( ip )%Tauo / ro )         ! profilska smicuca brzina        
		ko  = 1.                                 ! korekcioni faktor nabora
		if( d90 * 1000. < 0.6 ) then             ! kriterijum pojave nabora
		   nr = d90 **0.1667 / 26.               ! Manning za najvece zrno
           ko = ( nr / pp( ip )%CMan )**1.5      ! korekcioni faktor nabora                            
        endif
		Tauo = pp( ip )%Tauo * ko * QrQ( ip )    ! korigovani Tau napon [kPa]

        ! Pronos po frakcijama
		if( ip == 1 ) write( iw14, "(//A,f8.2)" ) " Q (m3/s): ", pp( ip )%Q
		write( iw14, "(A,i4)" ) " Prof.", ip
        Sumqvm = 0.
        do j = 1, nfrak                                        
           if( Tauo > Tauc( j ) ) then                               
              qvm_frak  = cMP( j ) * p( j ) * ( Tauo - Tauc( j ) )**1.5
           else
              qvm_frak  = 0.
           endif
		   write( iw14, "(i3,2x,f8.6)", advance="no" ) j, qvm_frak * 1000.
           Sumqvm = Sumqvm + qvm_frak
        end do
		write( iw14,"(2x,A,f8.6)" ) " Ukupno:",  Sumqvm * 1000.
        Bv  = pp( ip )%B * BvB( ip )             ! pojas vucenja
        Qvm = Sumqvm * Bv                        ! maseni pronos u profilu (t/s)
		QvmDan = Qvm * 86400.                    ! maseni pronos (t/dan)
        
        ! Stampanje ukupnog pronosa u profilu
		if( ip == 1 ) then
           write( iw13, 101 ) 
	       101 format(//1x,'Prof. Oznaka',9x,'x',7x,'Q',7x,'Z',7x,'R',4x,'Bv',7x,'Ie',   &
	       7x,'V',5x,'V*',6x,'n',5x,'ko',4x,'QrQ',4x,'Tauo',4x,'Qvm',5x,'Qvm'/           &
		   21x,'(km)',3x,'(m3/s)',2x'(mnm)',4x,'(m)',3x,'(m)',6x,'(-)',4x,'(m/s)',       &
	       2x,'(m/s)',3x,'(...)',2x,'(-)',3x,'(-)',4x,'(Pa)',3x,'(kg/s)'1x,'(t/dan)'/ )
        endif
        write( iw13, 102 ) ip, pp( ip )%oz, pp( ip )%xs, pp( ip )%Q,           &
		                       pp( ip )%Z, pp( ip )%R, Bv, pp( ip )%Ie,        &
							   pp( ip )%V, Vsm, pp( ip )%CMan, ko,  QrQ( ip ), &
							   Tauo  * 1000., Qvm * 1000., QvmDan
        102 format( i4,A,f10.3,f8.2,f8.3,f6.2,f7.2,e11.3,f6.2,f7.2,f8.4,2f6.2, &
		f8.4,f8.3,f8.0)
     END SUBROUTINE MeyerPeterMuller


  



     SUBROUTINE VanRijn( iw13, iw14 )
        integer, intent( in )  :: iw13, iw14
        real :: Vsm, CPrim, VzvPrim, TauoPrim, TT 
		real :: Co, uo, deltao, qvm, ah, FVR 

        ! Osnovne velicine
		pp( ip )%Tauo = pp( ip )%Tauo / 1000.               ! tang. napon na dnu [kPa]
        Vsm   = SQRT( pp( ip )%Tauo / ro )                  ! smicuca brzina [m/s]

        ! Vuceni nanos
        CPrim    = 18.*ALOG10( 12. * pp( ip )%R / 3./d90v ) ! Chezy-jev koeficijent [m1/2 /s]
        VsmPrim  = SQRT( g ) * pp( ip )%V / CPrim           ! deo smicuce brzine [m/s]
		if( VsmPrim > Vsm ) VsmPrim = Vsm                   ! kontrolni uslov
        TauoPrim = VsmPrim * VsmPrim * ro                   ! deo tangecijanog napona [kPa]
        TT       = ( TauoPrim - Tauoc ) / Tauoc             ! bezdimenzioni faktor pokretanja
		if( TT <= 0. ) TT = 0.                              ! kontrolni uslov
        Co       = 0.117 * TT / dzvv                        ! sr.koncentracija pokr.sloja [-]
        deltao   = 0.3 * d50v * dzvv**0.7 * TT**0.5         ! debljina pokretnog sloja [m]
        uo       = 1.5 * TT**0.6 * SQRT( g * Delta * d50v ) ! brzina nanosa u pok.sloju [m/s]
        qvm      = Co * deltao * uo * ros                   ! jedinicni maseni pronos [t/(sm)]
        Bv       = pp( ip )%B * BvB( ip )                   ! aktivni pojas vucenja [m]
        Qvm      = qvm * Bv                                 ! maseni pronos u profilu [t/s]

        ! Suspendovani nanos
        WVsm   = W / Vsm                                    ! faktor formiranja suspenzije
        Beta   = 1. + 2. * WVsm * WVsm                      ! odnos koeficijenata mesanja
		if( Beta > 2. ) Beta = 2.                           ! kontrolni uslov (ogranicenje)
        aref   = 10. * d50v                                 ! referentno odstojanje [m]
        Cref   = 0.015 * d50v * TT**1.5 / aref / dzvv**0.3  ! referentna koncentracija [-]
        ZZ     = W / Beta / Kappa / Vsm                     ! suspenzioni broj [-]
        Psi    = 2.5 * WVsm**0.8 * ( Cref / 0.65 )**0.4     ! korekcioni faktor [-]
        ZZPrim = ZZ + Psi                                   ! korigovani suspenzioni broj [-]
        if( ZZPrim == 1.2 ) ZZPrim = 1.1                    ! ogranicenje br. 1
		if( ZZPrim >  3.  ) ZZPrim = 3.                     ! ogranicenje br. 2
        ah     = aref / pp( ip )%R                          ! pomocna promenljiva
        FVR    = ( ah**ZZPrim - ah**1.2 ) /      &          ! bezdimenziona pomocna funkcija
		         ( ( 1. - ah )**ZZPrim * (1.2 - ZZPrim ) )
        Qsm    = FVR * pp( ip )%R * pp( ip )%V * Cref * ros ! maseni pronos [t/s]
  
        ! Ukupni pronos
        Qum = Qvm + Qsm                                     ! maseni pronos [t/s]
		QumDan = Qum * 86400.                               ! maseni pronos [t/dan]


        ! Stampanje pronosa vucenog nanosa
		if( ip == 1 ) then
           write( iw14, 201 ) 
	       201 format(//1x,'Prof. Oznaka',9x,'x',7x,'Q',7x,'Z',7x,'V*',6x,'C`',4x,       &
		   'Tauo`',7x,'T',5x,'Co',4x,'deltao',3x,'Uo',7x,'qvm',5x,'Bv',6x,'Qvm'/         &        
		   21x,'(km)',3x,'(m3/s)',2x'(mnm)',4x,'(m/s)',2x,'(...)',3x,'(kPa)',6x,'(-)',   &
	       4x,'(-)',5x,'(m)',3x,'(m/s)',3x,'(kg/sm)'3x,'(m)',4x,'(kg/s)'/ )
        endif
        write( iw14, 202 ) ip, pp( ip )%oz, pp( ip )%xs, pp( ip )%Q,pp( ip )%Z,Vsm,CPrim,&
	    TauoPrim, TT, Co, deltao, uo, qvm * 1000., Bv, Qvm * 1000.
        202 format( i4,A,f10.3,f8.2,f8.3,f8.4,f7.2,e11.3,3f7.3,f8.3,f9.3,2f8.2)


        ! Stampanje pronosa suspendovanog i ukupnog nanosa 
		if( ip == 1 ) then
           write( iw13, 203 ) 
	       203 format(//1x,'Prof. Oznaka',9x,'x',7x,'Q',7x,'Z',7x,'V*',6x,'W/V*',4x,     &
		   'Beta',5x,'a',7x,'Ca',6x,'Z*',5x,'Psi',4x,'Z*`',5x,'F',6x,'Qsm'5x,'Qum',4x,   &
		   'Qum'/ 21x,                                                                   & 
		   '(km)',3x,'(m3/s)',2x'(mnm)',4x,'(m/s)',4x,'(-)',5x,'(-)',5x,'(m)',6x,'(-)',  &
	        4x,'(-)',5x,'(-)',2(4x,'(-)'),2(3x,'(kg/s)'),1x,'(t/dan)'/ ) 
        endif
        write( iw13, 204 ) ip, pp( ip )%oz, pp( ip )%xs, pp( ip )%Q,pp( ip )%Z,Vsm,WVsm, &
		Beta, aref, Cref, ZZ, Psi, ZZPrim, FVR, Qsm * 1000., Qum * 1000., QumDan
        204 format( i4,A,f10.3,f8.2,f8.3,f8.4,2f8.3,f9.4,f8.4,3f7.3,f8.4,2f8.2,f8.0)
    END SUBROUTINE VanRijn




     real FUNCTION Viskoznost( Temp ) ! kinematicki koeficijent viskoznosti
        real, intent( in ) :: Temp
        Viskoznost = ( 1.14 - 0.03100 * ( Temp - 15. ) +   &
		                      0.00068 * ( Temp - 15. )**2 ) * 1.e-6
     END FUNCTION Viskoznost




     real FUNCTION Thetac( dzv ) ! Shields-ova zavisnost
        real, intent( in ) :: dzv
        if( dzv >   1. .and. dzv <=   4. ) Thetac = 0.24  * dzv**( -1.00 )
        if( dzv >   4. .and. dzv <=  10. ) Thetac = 0.14  * dzv**( -0.64 )
        if( dzv >  10. .and. dzv <=  20. ) Thetac = 0.04  * dzv**( -0.10 )
        if( dzv >  20. .and. dzv <= 150. ) Thetac = 0.013 * dzv**(  0.29 )
        if( dzv > 150.                   ) Thetac = 0.056    
     END FUNCTION Thetac




     real FUNCTION fw2( ds ) ! Brzina talozenja 
        real :: ds, s, s1
        s  = ros / ro;  s1 = ( s - 1.) * 9.81
        if(     ds >= 1.e-06 .and. ds < 100.e-06    ) then
           fw2 = s1 * ds * ds / 18. / ro / Visk
        elseif( ds >= 100.e-06 .and. ds < 1000.e-06 ) then
           fw2 = 10.* Visk * ( SQRT( 1. + 0.01 * s1 * ds**3 / Visk**2 ) - 1. ) / ds
        elseif( ds >= 1000.e-06 ) then
           fw2 = 1.1 * SQRT( s1 * ds )
        else
           stop 'fw2: d van datih opsega!'
        endif
     END FUNCTION fw2




     SUBROUTINE EngelundHansen( iw13, iw14 )
        integer, intent( in )  :: iw13, iw14
        real                   :: cEH, Cm, SumCm

        ! Konstanta
        cEH = 0.05 * ros / SQRT( g ) / Delta**2
		
        ! Pronos po frakcijama
		if( ip == 1 ) write( iw14, "(//A,f8.2)" ) " Q (m3/s): ", pp( ip )%Q
		write( iw14, "(A,i4)" ) " Prof.", ip
        SumCm = 0.
        do j = 1, nfrak                                       
           Cm = cEH * pp( ip )%CMan**3 * pp( ip )%V**4 * p( j ) / d( j ) / pp( ip )%R**1.5
		   write( iw14, "(i3,2x,f8.6)", advance="no" ) j, Cm * 1000.
           SumCm = SumCm + Cm
        end do
		write( iw14,"(2x,A,f8.6)" ) " Ukupno:",  SumCm * 1000.
        Qum = SumCm * pp( ip )%Q    ! pronos ukupnog nanosa [t/s]
		QumDan = Qum * 86400.       ! pronos ukupnog nanosa [t/dan]           
           

        ! Stampanje ukupnog pronosa u profilu
		if( ip == 1 ) then
           write( iw13, 303 ) 
	       303 format(//1x,'Prof. Oznaka',9x,'x',7x,'Q',7x,'Z',7x,'R',5x,'B',       &
		   7x,'Ie', 7x,'V',6x,'V*',6x,'n',8x,'Cm',7x,'Qum',5x,'Qum'/                &                   
		   21x,'(km)',3x,'(m3/s)',2x'(mnm)',4x,'(m)',3x,'(m)',6x,'(-)',             &
		   4x,'(m/s)',2x,'(m/s)',3x,'(...)',3x,'(kg/m3)',3x,'(kg/s)',2x,'(t/dan)'/ )
        endif
        write( iw13, 304 ) ip, pp( ip )%oz, pp( ip )%xs, pp( ip )%Q,  pp( ip )%Z,   &
		                       pp( ip )%R,  pp( ip )%B,  pp( ip )%Ie, pp( ip )%V,   &
			                   SQRT( g * pp( ip )%R * pp( ip )%Ie ),                &
							   pp( ip )%CMan, SumCm * 1000., Qum * 1000., QumDan
        304 format( i4,A,f10.3,f8.2,f8.3,f6.2,f7.2,e11.3,f6.2,f7.3,f8.4,2f10.4,f8.0 )
     END SUBROUTINE EngelundHansen



  END MODULE TransportniKapacitetNanos
 !----------------------------------------------------------------------------------------
  
  
  
  


 !---------------------------------------------------------------------------------------- 
  MODULE OpstaDeformacija
     use TransportniKapacitetNanos

     real :: Qnanos_ul, Qnanos_iz, dQnanos, Dubina


     contains


     SUBROUTINE JednacinaKontinuitetaNanos( iw14 )
        integer, intent( in ) :: iw14
        write( iw14, "(A/)" ) "  Proracun opste deformacije"
		write( iw14, "(A,f12.2)" ) "  Q (m3/s):     ", Qdef
		write( iw14, "(A,f12.2)" ) "  dt (dani):    ", dt
		write( iw14, "(A,f12.2)" ) "  Ulaz (t/dan): ", Q1dn
        write( iw14, 101 )

        Qnanos( 1 ) = Q1dn ! uzvodni granicni uslov (ulazni tovar, t/dan)

        do ip = 2, nprof
           Qnanos_ul   = Qnanos( ip - 1 )
           Qnanos_iz   = Qnanos( ip )
           dQnanos     = Qnanos_ul - Qnanos_iz
           pp( ip )%dAd = dQnanos * dt / roz / pp( ip )%dx / 1000.
           write( iw14, 102 ) ip, pp( ip )%xs, Qnanos_ul, Qnanos_iz, dQnanos, pp( ip )%dAd
        enddo

        101 format(//5x,'Prof.   Stac.',6x,'Ulaz',6x,'Izlaz',6x,'Razlika',5x,'DeltaAd'/    &
                     5x,'(-)     (km) ',5x,'(t/dan)',3x,'(t/dan)',5x,'(t/dan)',7x,'(m2)'/)     
        102 format(  4x, i4, f10.3, 3e12.4, f10.2 )
     END SUBROUTINE JednacinaKontinuitetaNanos







     SUBROUTINE KorekcijaKoordinata
        do ip = 1, nprof
           do j = 1, pp( ip )%nt
              Dubina = pp( ip )%Z - pp( ip )%zp( j )
              if( Dubina >  0. ) pp( ip )%dzp( j ) = Dubina * pp( ip )%dAd / pp( ip )%A
              if( Dubina <= 0. ) pp( ip )%dzp( j ) = 0.
              if( ip == 1 ) then
                 pp( ip )%zpn( j ) = pp( ip )%zp( j )
              else
                 pp( ip )%zpn( j ) = pp( ip )%zp( j ) + pp( ip )%dzp( j )
              endif
           enddo
        enddo
     END SUBROUTINE KorekcijaKoordinata








    SUBROUTINE StampaKorigovaneKoordinate( iw16, iw17, iw21 )
       integer, intent( in ) :: iw16, iw17, iw21
       write( iw16, "(A/)" ) "  Korekcije vertikalnih koordinata"
       write( iw17, "(A/)" ) "  Graficki prikaz opste deformacije u izabranim profilima"
       write( iw21, "(A/)" ) "  Geometrija profila nakon deformacije"
       do ip = 2, nprof
          write( iw16, "(/2x,i4,f12.3)" ) ip, pp( ip )%xs
          write( iw16, "(2x, 5f10.3)" ) ( pp( ip )%dzp( j ), j = 1, pp( ip )%nt )
       enddo
       i = 1
       do ip = 1, nprof
          if( ip == ip_grf( i ) ) then
		     write( iw17, "(5x,i5)" ) ip
             do j = 1, pp( ip )%nt
                write( iw17, "(5x,3f12.3)" )     &
				pp( ip )%yp( j ), pp( ip )%zp( j ), pp( ip )%zpn( j )
             enddo
             i = i + 1
          endif
       end do
       do ip = 1, nprof
          write( iw21, "(/2x,i4,f12.3,i5,A)" ) ip, pp( ip )%xs, pp( ip )%nt, pp( ip )%oz
          write( iw21, "(2x,10f8.3)" ) ( pp( ip )%zpn( j ), j = 1, pp( ip )%nt )
          write( iw21, "(2x,10f8.3)" ) ( pp( ip )%yp ( j ), j = 1, pp( ip )%nt )
       end do
    END SUBROUTINE StampaKorigovaneKoordinate



    END MODULE OpstaDeformacija
 !----------------------------------------------------------------------------------------





     PROGRAM Nanos
        use OpstaDeformacija

        character(12) :: datio, datu1, datu11, datu12, dati13, dati14,    &
                                       dati15, dati16, dati17, dati21

        print "(5x,A)", "Ime fajla sa imenima I/O fajlova? "
        read "(A)", datio
        open( UNIT = 20, FILE = datio, STATUS = "old" )
        read( 20, "(A)" ) datu1   ! ulazni podaci: profili
        read( 20, "(A)" ) datu11  ! ulazni podaci: hidraul. elementi iz Reke1
        read( 20, "(A)" ) datu12  ! ulazni podaci: nanos
        read( 20, "(A)" ) dati13  ! rezultati: transp. kapacitet 
        read( 20, "(A)" ) dati14  ! rezultati: transp. kapacitet po frakcijama
        read( 20, "(A)" ) dati15  ! rezultati: deformacija po profilima
        read( 20, "(A)" ) dati16  ! rezultati: deformacija po koordinatama
		read( 20, "(A)" ) dati17  ! rezultati: graficki prikaz deformacije
        read( 20, "(A)" ) dati21  ! rezultati: novi profili za Reke1
        open( UNIT =   1, FILE =  datu1,  STATUS = "old"     )
        open( UNIT =  11, FILE =  datu11, STATUS = "old"     )
        open( UNIT =  12, FILE =  datu12, STATUS = "old"     )
        open( UNIT =  13, FILE =  dati13, STATUS = "replace" )
        open( UNIT =  14, FILE =  dati14, STATUS = "replace" )

        ! Ucitavanje ulaznih podataka
        call UcitavanjeProfila( 1 )
        call RazmakProfila
        call AlociranjeNizaNanos
        call UcitavanjePodatakaNanos( 12, 13, 14 )
  
        ! Proracun tranportnog kapaciteta
		irun = 1
		eof  = 0
1       call UcitavanjeHidraulickihElemenata( 11, irun)
        if( eof >= 0 ) then
           do ip = 1, nprof
              SELECT CASE ( kodm )
                  case( 1 ) ! Meyer-Peter & Muller
                     call MeyerPeterMuller( 13, 14 )
                     Qnanos( ip ) = QvmDan
                  case( 2 ) ! VanRijn
                     call VanRijn( 13, 14 )
                     Qnanos( ip ) = QumDan
                  case( 3 ) ! EngelundHansen
                     call EngelundHansen( 13, 14 )
                     Qnanos( ip ) = QumDan
              END SELECT
           end do
           irun = irun + 1
           goto 1
		endif

        ! Proracun opste deformacije korita
        if( kodd == 1 ) then	   
           open( UNIT =  15, FILE =  dati15, STATUS = "replace" )
           open( UNIT =  16, FILE =  dati16, STATUS = "replace" )
           open( UNIT =  17, FILE =  dati17, STATUS = "replace" )
           open( UNIT =  21, FILE =  dati21, STATUS = "replace" )
     
           call JednacinaKontinuitetaNanos( 15 )
           call KorekcijaKoordinata
           call StampaKorigovaneKoordinate( 16, 17, 21 )
        endif


       close( 1 );     close( 11 ); close( 12 ); close( 13 ); close( 14 )
	   if( kodd == 1 ) then
          close( 15 ); close( 16 ); close( 17 ); close( 21 )
       endif
     END PROGRAM Nanos
