 !-----------------------------------------------------------------------
 ! Program:   Reke1
 ! Funkcija:  Proracun ustaljenog tecenja u neprizmaticnom koritu 
 ! Opis:      Iterativno resavanje Bernoullijeve jednacine       
 !            u mirnom ili burnom rezimu; mesoviti rezim se ne         
 !            moze automatski racunati, vec se u jednom racunskom 
 !            prolazu hidraulicki skok "premoscava" kriticnom dubinom. 
 !                                                                           
 !            Racunske opcije:    
 !            (1) normalna/kriticna dubina po profilima;              
 !            (2) normalna/kriticna dubina+linije nivoa;
 !            (3) kalibracija koeficijenta otpora.
 !            
 !            Racunska metoda: polovljenje intervala
 !
 ! Autor:     M. Jovanovic, avg. 1978, jun 1986, okt. 1993, okt. 1999.
 ! Okruzenje: Lahey Fortran90, LF90, V.2.00a, 1995.
 !            Microsoft Developer Studio, Visual Fortran, V.6,6 2001.
 !----------------------------------------------------------------------




 !----------------------------------------------------------------------
  MODULE PoprecniProfili
 !......................................................................
 !      Napomene:   
 !
 !      1. Smer ucitavanja profila je u NIZVODNI: profil br. "1" je onaj
 !         na uzvodnom kraju racunske deonice, a profil br. "nprof" je
 !         na nizvodnom kraju. (Ako izvorni fajl sa prifilima nije formiran
 !         na taj nacin, moze se modifikovati pomocu programa Reke1p).
 !         
 !      2. Nacin osrednjavanja linije energije definisan je parametrom:
 !         kodsr = 1 aritmeticka  sredina;
 !               = 2 geometrijska sredina;
 !               = 3 harmonijska  sredina.     
 !.......................................................................    
     implicit none
     character(75)             :: text1
     integer                   :: nprof, smerr, nit, ier, nred, ip
     integer, parameter        :: maxp = 200, maxt = 50, kodsr = 1
     real,    parameter        :: g = 9.81, eps = .001
     real                      :: zdmin

     TYPE :: profil
        integer                :: nt   ! broj koordinatnih tacaka
        real                   :: xs   ! stacionaza (m)
        character(12)          :: oz   ! oznaka profila
        real, dimension (maxt) :: zp   ! kota dna
        real, dimension (maxt) :: yp   ! odstojanje
        real                   :: zd   ! minimalna kota dna
        real                   :: zo   ! minimalna kota obale
        real                   :: Id   ! lokalni nagib dna (uzvodni)
        real                   :: Ie   ! lokalni nagib linije energije
        real                   :: dx   ! pripadajuce odstojanje
        real                   :: Q    ! protok
        real                   :: Z    ! racunska kota nivoa
        real                   :: Zreg ! registrovana kota nivoa
        real                   :: Zk   ! kota kriticne dubine
        real                   :: Zn   ! kota normalne dubine
        real                   :: A    ! povrsina preseka
        real                   :: B    ! sirina vodenog ogledala
        real                   :: O    ! okvaseni obim
        real                   :: R    ! hidraulicki radijus
        real                   :: K    ! modul protoka 
        real                   :: CMan ! Manningov koef. rapavosti
        real                   :: Clok ! koef. lokalnog gubitka
        real                   :: V    ! srednja brzina
        real                   :: V2g  ! brzinska visina
        real                   :: E    ! kota energije
        real                   :: dEt  ! gubitak na trenje
        real                   :: dEl  ! lokalni gubitak
        real                   :: Fr   ! Froudeov broj
     END TYPE profil

     TYPE( profil ), allocatable, dimension (:) :: pp


     contains


     SUBROUTINE UcitavanjeProfila ( ir, ic )
        integer :: ir, ic, i, ii, ii1, ip1, j, ndx, ip
        real    :: dy, dd

        read( ir, "(A)") text1
        read( ir, *    ) nprof
        if( nprof > maxp ) stop 'Premasen broj profila   !!'

        allocate( pp( nprof ) )

        ii = 0; ip = 0
        do while( ip < nprof )
           ii = ii + 1; ip = ip + 1
           read( ir, * )  ip, pp( ip )%xs, pp( ip )%nt, pp( ip )%oz 
           if( pp( ip )%nt > maxt ) then
	          print *, ' Profil broj :', ip, pp( ip )%oz
	          stop     ' Premasen maksimalni broj tacaka !'
	       else                                  ! ucitavanje koordinata
              read( ir, * ) ( pp( ip )%zp(j), j = 1, pp( ip )%nt )
              read( ir, * ) ( pp( ip )%yp(j), j = 1, pp( ip )%nt )
	       endif

           do j = 1, pp( ip )%nt - 1             ! provera podataka
              dy = pp( ip )%yp(j+1) - pp( ip )%yp(j)
	          if( dy < 0. ) then
                 print *, ' Profil broj :', ip, pp( ip )%oz
	             stop     ' Greska u podacima! <y> ??'
              endif
           enddo

           if( ip /= ii ) then                   !  interpolacija 
              if( pp( ii-1 )%nt  /=  pp( ip )%nt  ) then
	             print *, ' Profili broj :', ii-1, ip   
	             stop     ' Nemoguca interpolacija! '
              endif
              ndx = ( ip - ii ) + 1;    ii1 = ii - 1;    ip1 = ip - 1
	          do  i = ii, ip1
                 dd = FLOAT( ( i - ii1 ) ) / ndx
	             pp( i )%nt = pp( ip )%nt
	             pp( i )%xs = pp( ii1 )%xs + ( pp( ip )%xs - pp( ii1 )%xs )*dd
	             do j = 1, pp( ip )%nt
	                pp( i )%zp(j) = pp( ii1 )%zp(j) +                     &
                      ( pp( ip )%zp(j) - pp( ii1 )%zp(j) ) * dd         
                    pp( i )%yp(j) = pp( ii1 )%yp(j) +                     &
                      ( pp( ip )%yp(j) - pp( ii1 )%yp(j) ) * dd
                 enddo
              enddo		 
              ii = ip
           endif
        enddo

        write(ic, "(A)") text1                   ! kontrolno stampanje
	    do i = 1, nprof
           write(ic, 100)  i, pp( i )%xs, pp( i )%oz                    
           100 format (//'  Profil', i3, '  Stacionaza (km):',            &
           f8.3,'  Oznaka: ', A/ 3( 9x, 'y', 9x, 'zd', 3x ) )
           write(ic,"(6f12.2)") (pp( i )%yp(j), pp( i )%zp(j),j=1, pp( i )%nt)
        enddo
     END SUBROUTINE UcitavanjeProfila





     SUBROUTINE UzduzniNagibDna ( ic )
        integer :: i, ic;  real :: sumx = 0., sums = 0., Id_sr

        do i = 1, nprof                ! minimalne kote: dna, obale
           pp( i )%zd = MINVAL ( RESHAPE( pp( i )%zp, (/pp( i )%nt/) ) )    
           pp( i )%zo = MIN    ( pp( i )%zp( 1 ), pp( i )%zp( pp( i )%nt ) )  
        enddo

        zdmin = MINVAL( pp%zd )        ! aps. minimum deonice

        do i = 2, nprof                ! lokalni nagib dna
           pp( i )%dx = ABS( pp( i )%xs - pp( i-1 )%xs ) * 1000.
           pp( i )%Id = ( pp( i-1 )%zd - pp( i )%zd ) / pp( i )%dx
           sums = sums + pp( i )%Id * pp( i )%dx
           sumx = sumx + pp( i )%dx
        enddo

        pp( 1 )%dx = pp( 2 )%dx;  pp( 1 )%Id = pp( 2 )%Id
        Id_sr = sums / sumx            ! poresecni nagib dna 

        write( ic,"(//5x,'Deonica:', 4x, 'dx (m)', 3x, 'zdmin', 7x, 'Id')"  )
        do i = 2, nprof
           write( ic, "(i7, ' -', i3, f10.1, f10.2, f12.6)"  )            &
           i-1, i, pp( i )%dx, pp( i )%zd, pp( i )%Id
        enddo

        write( ic, "( //5x,'Prosecni nagib dna:  ', f8.6 )" ) Id_sr
        write( ic, "(  /5x,'Najniza kota dna deonice (mnm): ', f8.2)" ) zdmin
     END SUBROUTINE UzduzniNagibDna





     SUBROUTINE FunkcijeProfila( ip, ZZ )
        integer :: i, k, ip
        real    :: ZZ, Zp, Zpp, dy, dz, ai, bi

        if( ZZ > (MIN( pp( ip )%zp( 1 ), pp( ip )%zp(pp( ip )%nt ) ) ) ) then
	       print *, ' Profil broj :', ip, pp( ip )%oz
	       stop     ' Izlivanje !! '
        else
	       pp( ip )%A = 0.;   pp( ip )%B = 0.;   pp( ip )%O = 0.
	    endif

        do i = 1, pp( ip )%nt - 1

           dy = pp( ip )%yp( i+1 ) - pp( ip )%yp( i )
           dz = pp( ip )%zp( i+1 ) - pp( ip )%zp( i )
           Zp = ( ZZ - pp( ip)%zp(i) ) * ( ZZ - pp( ip )%zp(i+1) )

           if( Zp >  0. .and. ZZ < pp( ip )%zp( i ) ) k = 1 ! seg. iznad nivoa
           if( Zp >  0. .and. ZZ > pp( ip )%zp( i ) ) k = 2 ! seg. ispod nivoa
           if( Zp <= 0. .and. dz /= 0. )              k = 3 ! nivo sece segment
           if( Zp == 0. .and. dz == 0. )              k = 4 ! nivo = horiz. s.

           SELECT CASE (k)
              case (1)
              case (2)
                 bi = dy
                 pp( ip )%B = pp( ip )%B + bi
                 pp( ip )%A = pp( ip )%A + bi * ( ZZ - 0.5 *         &
                            ( pp( ip )%zp( i+1 ) + pp( ip )%zp( i ) ) )
                 pp( ip )%O = pp( ip )%O + SQRT( bi**2 + dz**2 )
              case (3)
                 if( ZZ > pp( ip )%zp( i ) )   Zpp = pp( ip )%zp( i )
                 if( ZZ > pp( ip )%zp( i+1 ) ) Zpp = pp( ip )%zp( i+1 )
                 bi = dy * ( ZZ - Zpp ) / ABS( dz )
                 ai = bi * 0.5 * ( ZZ - Zpp )
                 pp( ip )%B = pp( ip )%B + bi
                 pp( ip )%A = pp( ip )%A + ai
                 pp( ip )%O = pp( ip )%O + SQRT( bi**2 + ( ZZ - Zpp )**2 )
              case (4)
                 pp( ip )%B = pp( ip )%B + dy
                 pp( ip )%O = pp( ip )%O + dy
           END SELECT
        enddo

        if( pp( ip )%A == 0. ) then
           print *, ' Profil broj :', ip, pp( ip )%oz
           stop     ' Nivo ispod najnize kote dna !!'
        endif      

        pp( ip )%R   = pp( ip )%A / pp( ip )%O
        pp( ip )%K   = pp( ip )%A * pp( ip )%R**.666667 / pp( ip )%CMan 
        pp( ip )%V   = pp( ip )%Q / pp( ip )%A
        pp( ip )%Fr  = pp( ip )%V**2 * pp( ip )%B / g / pp( ip )%A
        pp( ip )%V2g = pp( ip )%V**2 / ( 2. * g )    
        pp( ip )%E   = ZZ + pp( ip )%V2g  
        pp( ip )%Ie  = ( pp( ip )%Q / pp( ip )%K )**2
     END SUBROUTINE FunkcijeProfila




     real FUNCTION fnorm ( ZZ )
        real :: ZZ
        call FunkcijeProfila ( ip, ZZ )
        fnorm = ( pp( ip )%Q / pp( ip )%K )**2 - pp( ip )%Id
     END FUNCTION fnorm




     real FUNCTION fkrit ( ZZ )
        real :: ZZ
        call FunkcijeProfila ( ip, ZZ )
        fkrit = 1. -  pp( ip )%Fr
     END FUNCTION fkrit




     real FUNCTION fener ( ZZ )
        integer :: ip1;  real :: ZZ, Ie_sr
        call FunkcijeProfila ( ip, ZZ )
        ip1 = ip - smerr

        ! lokalni gubici
        pp( ip )%dEl = pp( ip )%Clok * ABS( pp( ip )%V2g - pp( ip1 )%V2g )

        ! osrednjavanje nagiba linije energije
        if( kodsr == 1 ) Ie_sr = 0.5 * ( pp( ip )%Ie + pp( ip1 )%Ie ) 
        if( kodsr == 2 ) Ie_sr = SQRT( pp( ip )%Ie * pp( ip1 )%Ie )   
        if( kodsr == 3 ) Ie_sr = pp( ip )%Ie * pp( ip1 )%Ie /             &
                               ( pp( ip )%Ie + pp( ip1 )%Ie )
        ! gubici na trenje i funkcija energetske kote
        if( smerr == -1 ) then                   ! miran rezim
           pp( ip )%dEt = Ie_sr * pp( ip1 )%dx
           fener = pp( ip1 )%E - pp( ip )%E + pp( ip )%dEt + pp( ip )%dEl
        else                                     ! buran rezim
           pp( ip )%dEt = Ie_sr * pp( ip  )%dx
           fener = pp( ip1 )%E - pp( ip )%E - pp( ip )%dEt - pp( ip )%dEl
        endif         
     END FUNCTION fener


  END MODULE PoprecniProfili
 !----------------------------------------------------------------------

 

    


 !----------------------------------------------------------------------
  MODULE RacunskiParametri
     use PoprecniProfili

     implicit none
     integer, parameter :: nredmax = 45      ! duzina strane za stampu
     integer            :: eof               ! indikator greske ucitavanja
     character(75)      :: text2, text3      ! identifikacija proracuna
     real               :: Zini              ! pocetna  kota nivoa


     contains



     SUBROUTINE IdentifikacijaProracuna( i2, i4, i7, i8, i9, i11, kodr )
        integer :: i2, i4, i7, i8, i9, i11, kodr
        read ( i2, "(A)" ) text2
        read ( i4, "(A)" ) text3
        write( i7, "(A)" ) text2
        if( kodr /= 3 )  write( i8, "(A)" ) text2
        if( kodr == 3 )  write( i8, "(A)" ) text3
        write( i9, "(A)" ) text2
        write(i11, "(A)" ) text2
     END SUBROUTINE IdentifikacijaProracuna





     SUBROUTINE UcitavanjeParametara ( i2, i4, kodr )
        integer :: i2, i4, i, kodr
        if( kodr /= 3 ) then
           read( i2, *, iostat=eof )  ( pp( i )%Q,    i = 1, nprof )    
           read( i2, *, iostat=eof )  ( pp( i )%CMan, i = 1, nprof )    
           read( i2, *, iostat=eof )  ( pp( i )%Clok, i = 1, nprof )    
           read( i2, *, iostat=eof )  Zini
           read( i2, *, iostat=eof )  smerr    ! mirno: -1; burno: + 1   
           if( smerr /= -1  .and.  smerr /= 1 ) stop 'Proveri smerr !! '
        elseif( kodr == 3 ) then
           read( i4, *, iostat=eof )  ( pp( i )%Q,    i = 1, nprof )    
           read( i4, *, iostat=eof )  ( pp( i )%Clok, i = 1, nprof )    
           read( i4, *, iostat=eof )  ( pp( i )%Zreg, i = 1, nprof )
        endif
     END SUBROUTINE UcitavanjeParametara





     SUBROUTINE GranicniUslov( ip )
        integer :: ip
        if( smerr == -1 .and. Zini < pp( nprof )%Zk ) stop ' Granicni uslov? Smerr?'
        if( smerr ==  1 .and. Zini > pp( 1 )%Zk     ) stop ' Granicni uslov? Smerr?'
        if( smerr == -1 ) ip = nprof   ! miran rezim
        if( smerr ==  1 ) ip = 1;      ! buran rezim 
        pp( ip )%Z = Zini              ! pocetna kota nivoa
     END SUBROUTINE GranicniUslov


  END MODULE RacunskiParametri
 !----------------------------------------------------------------------




 
 !----------------------------------------------------------------------
  MODULE Stampa
     use RacunskiParametri

     integer, parameter :: irunmax = 10 ! maksimalni broj linija nivoa
     real, allocatable, dimension(:,:) :: Zgr ! kote za graficki prikaz


     contains


     
     SUBROUTINE NormalnaKriticnaZaglavlje ( i7 )
        integer :: i7
        write( i7, 201)     
        201 format( //1x,'Pr.',3x,'Stac.',7x,'Q',6x,'zd',8x,'Id',         &
        7x,'Zn',6x,'hn',6x,'Zk',6x,'hk',3x,'nit'/1x,'(-)',3x,'(Km)',      &
        5x,'(m3/s)',3x,'(mnm)',6x,'(-)',5x,'(mnm)',4x,'(m)',              &
        4x,'(mnm)',4x,'(m)',2x,'(-)' / )
     END SUBROUTINE NormalnaKriticnaZaglavlje





     SUBROUTINE NormalnaKriticnaRezultati ( i7, ip )
        integer :: i7, ip;   real :: hn 
        hn = pp( ip )%Zn - pp( ip )%zd
        if( pp( ip )%Id <= 0. ) hn = 99.999
        write(i7, "(i4, f9.3, f9.2, f9.3, f10.6, 2(f9.3,f7.3),i4 )" )     &
        ip, pp( ip )%xs, pp( ip )%Q,  pp( ip )%zd, pp( ip )%Id,           &
        pp( ip )%Zn, hn, pp( ip )%Zk, pp( ip )%Zk - pp( ip )%zd, nit
     END SUBROUTINE NormalnaKriticnaRezultati




    
     SUBROUTINE  LinijaNivoaZaglavlje ( i8, i11 )
        integer, intent( in ) :: i8, i11
        write( i8, 300 )
        300 format( //1x, 79('-')/                                        &
        1x,'Profil',4x,'Z(mnm)',5x,'A(m2)',3x,'B(m)',2x,'R(m)',           &
        2x,'h(m) ',4x,'Ie',5x,'V(m/s)',2x,'dEt(m)',2x,'E(mnm)'/,          &
        1x,'St.(km)',2x,'Q(m3/s)',3x,'K(m3/s)',3x,'O(m)',3x,'n',          &
        4x,'hk(m)',4x,'Id',5x,'V2/2g',3x,'dEl(m)',4x,'Fr',/               &
        1x,79('=')  )  
        write( i11, 301 )
		301 format( //1x,'Prof.',1x,'Oznaka',6x,'St.(km)',2x,'Q(m3/s)',2x,&
		'Z(mnm)',3x,'E(mnm)',4x,'A(m2)',3x,'B(m)',4x,'O(m)',3x,'R(m)',1x, &
		'V(m/s)', 3x,'Ie(-)',5x,'n',2x,'Tauo(Pa)'/ )
     END SUBROUTINE LinijaNivoaZaglavlje





     SUBROUTINE LinijaNivoaRezultati ( k, i8, i11, ier )
        integer :: k, i8, i11
        if(ier == 0 ) then
           print 400,  k, pp( k )%Q, pp( k )%Z, nit, ier 
           400 format( 5x, i3, 3x,'Q =', f8.2, 3x,'Z =', f8.3, 3x,'nit =',&
                      i4, '   ier =', i3 )
           write( i8, 500 )  k,                                           &
           pp( k )%Z,   pp( k )%A,    pp( k )%B,    pp( k )%R,            &
           pp( k )%Z  - pp( k )%zd,   pp( k )%Ie,   pp( k )%V,            &
           pp( k )%dEt, pp( k )%E,    pp( k )%xs,   pp( k )%Q,            &
           pp( k )%K,   pp( k )%O,    pp( k )%CMan, pp( k )%Zk-pp( k )%zd,&
           pp( k )%Id,  pp( k )%V2g,  pp( k )%dEl,  pp( k )%Fr
           500 format( i4,5x,f8.3,f10.2,f7.2,2f6.2,e11.3,f7.3,f8.4,       &
           f8.3/f8.3,f9.2,e10.3,f7.2,f6.3,f6.2,e11.3,f7.3,2f8.4/1x,79('-') )
        else
           print 600,  k, pp( k )%Q, pp( k )%Z, nit, ier
           600 format(5x, i3, 3x,'Q =', f8.2,' **Z =', f8.3, 3x,'nit =',  &
                      i4, '   ier = ', i3 )
           write( i8, 700 )  k,                                           &
           pp( k )%Z,   pp( k )%A,    pp( k )%B,    pp( k )%R,            &
           pp( k )%Z  - pp( k )%zd,   pp( k )%Ie,   pp( k )%V,            &
           pp( k )%dEt, pp( k )%E,    pp( k )%xs,   pp( k )%Q,            &
           pp( k )%K,   pp( k )%O,    pp( k )%CMan, pp( k )%Zk-pp( k )%zd,&
           pp( k )%Id,  pp( k )%V2g,  pp( k )%dEl,  pp( k )%Fr
           700 format( i4,1x,'  **',f8.3,f10.2,f7.2,f6.2,e11.3,f7.3,f8.4, &
           f8.3/f8.3,f9.2,e10.3,f7.2,f6.3,f6.2,e11.3,F7.3,2F8.4/1X,79('-') )
        endif 
        	
        write( i11, 800 ) k,                                              &
		pp( k )%oz, pp( k )%xs, pp( k )%Q, pp( k )%Z, pp( k )%E,          &
        pp( k )%A,  pp( k )%B,  pp( k )%O, pp( k )%R, pp( k )%V,          &
        pp( k )%Ie, pp( k )%Cman, 1000. * g * pp( k )%R * pp( k )%Ie                     
        800 format( i4,A,f10.3,f9.2,2f9.3,3f8.2,2f6.2,f10.6,f7.3,f8.4 )
     END SUBROUTINE LinijaNivoaRezultati





     SUBROUTINE LinijaNivoaGrafika ( i9, irun )
        integer :: j, i, i9, irun
        do i = 1, nprof
           write( i9, "(12f10.3)" ) pp( i )%xs, pp( i )%zd,               &
           ( Zgr( j, i ), j = 1, irun )
        enddo
     END SUBROUTINE LinijaNivoaGrafika



  END MODULE Stampa
 !----------------------------------------------------------------------






 !----------------------------------------------------------------------
  MODULE KalibracijaOtpora
     use RacunskiParametri


     contains 



     SUBROUTINE ManningovKoefDeonice ( i8 )
        integer :: i, i1, i8
        real :: suml = 0., sumt = 0., ti1, ti, ss, Vsm, VR6, Manning_deonica
      
        write( i8, 900 )
        900 format(// 1x,'Pr.  Stac.',6x,'Q',7x,'Z',8x,'A',7x,'B',        &
        7x,'O',5x,'R',6x,'V*',3x,'V/R**(1/6)'/1x,'(-)',2x,'(Km)',         &
        4x,'(m3/s)',3x,'(mnm)',4x,'(m2)',5x,'(m)',5x,'(m)',3x,'(m)',      &
        3x,'(m/s)',2x,'(m^(5/6)/s)'/ )

        do i = 1, nprof - 1
           i1 = i + 1
           call FunkcijeProfila( i,  pp( i  )%Zreg )
           call FunkcijeProfila( i1, pp( i1 )%Zreg )

           ! Regresiona metoda
           ss  = ( pp( i )%E - pp( i1 )%E ) / pp( i1 )%dx
           Vsm = SQRT( g * pp( i )%R * ss)
           VR6 = pp( i )%V / pp( i )%R ** .166667
           write( i8,"(i4, f8.3, f9.2, 4f8.2, f6.2, f8.4, f9.4)" )        &
           i, pp( i )%xs, pp( i )%Q, pp( i )%Zreg, pp( i )%A,             &
              pp( i )%B,  pp( i )%O, pp( i )%R,    Vsm,    VR6

           ! Direktna metoda
           suml = suml + pp( i )%Clok * ABS( pp( i )%V2g - pp( i1 )%V2g )
           ti  = ( pp( i  )%Q / ( pp( i  )%A * pp( i  )%R **.6667 ) )**2
           ti1 = ( pp( i1 )%Q / ( pp( i1 )%A * pp( i1 )%R **.6667 ) )**2   
           if( kodsr == 1 ) sumt = sumt + 0.5 * ( ti + ti1 ) * pp( i )%dx
           if( kodsr == 2 ) sumt = sumt + SQRT( ti * ti1 )   * pp( i )%dx
           if( kodsr == 3 ) sumt = sumt + 2.*ti*ti1/(ti+ti1) * pp( i )%dx
        enddo

        Manning_deonica = SQRT( (pp( 1 )%E - pp( nprof )%E - suml) / sumt )
        write( i8, 950 ) Manning_deonica
        950 format(/3x,'Direktni postupak: kalibrisana vrednost n = ',    &
        f6.4,'  m^(1/3) s')
     END SUBROUTINE ManningovKoefDeonice


  END MODULE KalibracijaOtpora
 !---------------------------------------------------------------------- 






     SUBROUTINE neli( xd, xg, f, xo, eps, nit, ier )
        implicit none
        real,    external      :: f
        real,    intent( in  ) :: xd, xg, eps
        real,    intent( out ) :: xo
        integer, intent( out ) :: nit, ier
        integer                :: it
        real                   :: xa, xb, xp, yd, yg, yo, p, po

        xa = xd;   xb = xg;   xp = xa
        if( xa >= xb ) then
           xa = xb;   xb = xp
        endif
        yd = f( xa );   yg = f( xb )
        p = yd * yg
        if( p == 0. ) then
           if( yd == 0. ) xo = xa
           if( yg == 0. ) xo = xb
        nit = 1; ier = 0;  return
        elseif( p < 0. ) then
           nit = INT( LOG( ( xb - xa ) / eps ) / LOG( 2. ) ) + 1
           do it = 1, nit
              xo = 0.5 * ( xa + xb ); yo = f( xo )
              po = yd * yo
              if( po == 0. ) then
                 nit = it; ier = 0;  return
              elseif( po < 0. ) then
                 xb = xo; yg = yo
              else
                 xa = xo; yd = yo
              endif
           enddo
           xo = 0.5 * ( xa + xb )
           nit = it; ier = 0; return 
        else
           ier = 1; return
        endif
     END SUBROUTINE neli               





     PROGRAM Reke1
        use KalibracijaOtpora
        use Stampa

        implicit none
        integer :: kodr, izlaz, ii, irun
        character(12) :: datio, datu1, datu2, datu4,                      &
                         dati3, dati7, dati8, dati9, dati11

        ! Definicija I/O fajlova
        print "(5x,A)", "Ime fajla sa imenima I/O fajlova? "
        read "(A)", datio
        open( UNIT = 10, FILE = datio, STATUS = "old" )
        read( 10, "(A)" ) datu1   ! ulazni podaci - profili
        read( 10, "(A)" ) datu2   ! ulazni podaci - rac. parametri
        read( 10, "(A)" ) dati3   ! kontola profila
        read( 10, "(A)" ) datu4   ! ulazni podaci - kalibracija otpora
        read( 10, "(A)" ) dati7   ! rezultati: norm/krit dubine
        read( 10, "(A)" ) dati8   ! rezultati: linije nivoa
        read( 10, "(A)" ) dati9   ! grafika:   linije nivoa
        read( 10, "(A)" ) dati11  ! prenos rezultata za program Nanos
        open( UNIT =  1, FILE =  datu1, STATUS = "old"     )
        open( UNIT =  2, FILE =  datu2, STATUS = "old"     )
        open( UNIT =  3, FILE =  dati3, STATUS = "replace" )
        open( UNIT =  4, FILE =  datu4, STATUS = "old"     )     
        open( UNIT =  7, FILE =  dati7, STATUS = "replace" )
        open( UNIT =  8, FILE =  dati8, STATUS = "replace" )
        open( UNIT =  9, FILE =  dati9, STATUS = "replace" )
        open( UNIT = 11, FILE = dati11, STATUS = "replace" )
	    
		! Ucitavanje ulaznih podataka
        call UcitavanjeProfila ( 1, 3 )
        call UzduzniNagibDna   ( 3 ) 
        close( 3 )

1       print 111
        111 format(/5x,'Racunske opcije:'/,                               &
                    5x,'    1. Normalne/kriticne dubine'/,                &
                    5x,'    2. Linije nivoa + normalne/kriticne dubine'/, &
                    5x,'    3. Vrednost koefijenta otpora na deonici'/,   &
                    5x,'Izbor ? ')
        read *, kodr
        if( kodr /= 1 .and. kodr /= 2 .and. kodr /= 3 ) goto 1

        irun = 0
        allocate ( Zgr(irunmax, nprof ) )
        call IdentifikacijaProracuna    ( 2, 4, 7, 8, 9, 11, kodr )
2       call UcitavanjeParametara       ( 2, 4, kodr )

        if( eof >= 0 ) then
           irun = irun + 1

           ! Proracun normalne i kriticne dubine
           if( kodr == 1 .or. kodr == 2 ) then
              call NormalnaKriticnaZaglavlje ( 7 )
              do ip = 1, nprof
                 call neli( pp( ip )%zd+0.05, pp( ip )%zo, fkrit,      &
                            pp( ip )%Zk, eps, nit, ier)
                 if( ier == 1 ) stop 'Ne konvergira kriticna dubina!'
                 if( pp( ip )%Id <= 0. ) then
                     pp( ip )%Zn = 9999.999; nit = 0
                 else
                    call neli( pp( ip )%zd+0.05, pp( ip )%zo, fnorm,   &
                               pp( ip )%Zn, eps, nit, ier )
                    if( ier == 1 ) stop 'Ne konvergira normalna dubina!'
                 endif
                 call NormalnaKriticnaRezultati( 7, ip )
              enddo
           endif

           !Proracun linija nivoa
           if( kodr == 2 ) then
              call LinijaNivoaZaglavlje ( 8, 11 )
              call GranicniUslov( ip )
              call FunkcijeProfila( ip, pp( ip )%Z )   ! pocetni profil
			  pp( ip )%dEt = 0.; pp( ip )%dEl = 0.
              call LinijaNivoaRezultati (ip, 8, 11, ier )
              Zgr( irun, ip ) = pp( ip )%Z
              ier  = 0
              do while( (smerr == -1 .and. ip > 1) .or.                &
                        (smerr ==  1 .and. ip < nprof ) )              
                 ip = ip + smerr
                 call neli( pp( ip )%zd + .1, pp( ip )%zo, fener,      &
                            pp( ip )%Z, eps, nit, ier )
                 if( ier == 1 ) stop 'Ne moze da sracuna energetsku kotu !'
                 if(smerr == -1 .and. pp( ip )%Fr >= 1.  .or.          &
                    smerr == -1 .and. ier == 1           .or.          &
                    smerr ==  1 .and. pp( ip )%Fr <= 1.  .or.          &
                    smerr ==  1 .and. ier == 1 ) then
                    pp( ip )%Z = pp( ip )%Zk            ! fiktivne kote
                    call FunkcijeProfila ( ip, pp( ip )%Z ) 
                    ier = 1
                 endif
                 call LinijaNivoaRezultati ( ip, 8, 11, ier )
                 Zgr( irun, ip ) = pp( ip )%Z
              enddo
           elseif( kodr == 3 ) then
              call ManningovKoefDeonice ( 8 )
              do ip = 1, nprof
                 Zgr( irun, ip ) = pp( ip )%Zreg
              enddo
           endif

3          print 999
           999 format(/5x,'Opcije nastavka: '/,                           &
                       5x,'    0. Izlaz iz programa'/,                    &
                       5x,'    1. Nastavak sa novim podacima'/,           &
                       5x,'Izbor ? ')
           read *, izlaz
           if( izlaz /= 0 .and. izlaz /= 1 ) goto 3
           if( izlaz == 1 ) goto 2
        endif

        call LinijaNivoaGrafika ( 9, irun )
        close( 1 ); close( 2 ); close( 4 )
        do ii = 7, 11
           close( ii )
        enddo
     END PROGRAM Reke1

