 !----------------------------------------------------------------------------------------
 ! Program:   Reke3T
 ! Funkcija:  Neustaljeno tecenje u prizmaticnom kanalu trapeznog preseka
 ! Opis:      Resavanje St.Venant-ovih jednacina linijskog neustaljenog toka, 
 !            napisanih u konzervativnom obliku, obavlja se metodom konacnih
 !            razlika, primenom eksplicitnih racunskih shema drugog reda 
 !            tacnosti:
 !
 !               o Lax-Wendroff
 !               o MacCormack.
 !
 !            Pocetni uslov: zadati protok i konst. dubine u svim profilima
 !            Granicni uslov: naglo zatvaranje ustave na nizvodnom kraju 
 !            Detaljnije informacije se nalaze u fajlu Reke3T.pdf               
 !
 ! Autor:     M. Jovanovic, 12.04.1984,  19.05.1995., 4.08. 2002.
 ! Okruzenje: Microsoft Developer Stuido, Visual Fortran V.6,6.0, 2001.
 !----------------------------------------------------------------------------------------



 !----------------------------------------------------------------------------------------
  MODULE Kanal
   
     implicit none
	 character(75)      :: text
	 integer            :: i, ip, npp, npp1
     integer, parameter :: maxnpp = 500
     real,    parameter :: g = 9.81 
	 real               :: b, m, Id, CMan, CMan2, zd1, xL, dx

     	 
	 TYPE :: Profil
	    real :: x          ! stacionaza
		real :: Q          ! protok
        real :: h          ! dubina
		real :: A          ! povrsina
        real :: Bn         ! sirina nivoa
        real :: O          ! okvaseni obim
        real :: R          ! hidraulicki radijus
		real :: V          ! srednja profilska brzina
		real :: c          ! brzina prostiranja talasa
		real :: sm         ! staticki momenat
     END TYPE Profil

     TYPE( Profil ), dimension(:), allocatable :: pp


     contains


     SUBROUTINE PodaciKanal( ir )
        integer, intent( in ) :: ir
        read( ir, "(A)" ) text               ! identifikacija
		read( ir, * ) b, m, Id, CMan, xL     ! elementi kanala
		read( ir, * ) npp                    ! broj deonica
		npp1 = npp + 1
		if( npp1 > maxnpp ) stop 'Premasen maks. broj profila !'
		allocate( pp( npp1 ) ) 
		CMan2 = CMan * CMan
		dx   = xL / npp
     END SUBROUTINE PodaciKanal


      
     SUBROUTINE FunkcijeProfila ( ip, hh )
	    integer, intent( in ) :: ip
        real,    intent( in ) :: hh
        pp( ip )%Bn = b + 2. * hh * m                     ! sirina vodenog ogledala
		pp( ip )%A  = hh * ( b + m * hh )                 ! povrsina preseka
        pp( ip )%O  = b + 2. * hh * SQRT( 1. + m**2 )     ! okvaseni obim
        pp( ip )%R  = pp( ip )%A / pp( ip )%O             ! hidraulicki radijus
	 	pp( ip )%sm = hh * hh * ( b / 2. + hh * m / 3. )  ! staticki momenat     
		pp( ip )%V  = pp( ip )%Q / pp( ip )%A             ! srednja profilska brzina
		pp( ip )%c = SQRT( g * pp( ip )%A / pp( ip )%Bn ) ! brzina prostiranja talasa
     END SUBROUTINE FunkcijeProfila




     real FUNCTION Dubina( AA )
	    real, intent( in ):: AA
	    Dubina = ( -b + SQRT( b**2 + 4. * AA * m ) ) / 2. / m
     END FUNCTION Dubina
      



     SUBROUTINE StampanjeKanal( iw2 )
	    integer, intent( in ):: iw2
		write( iw2, "(A)"  ) text
		write( iw2, "(/5x,A/)" ) "Ulazni podaci:"
		write( iw2, 101) b, m, Id, Cman, xL
		101 format(5x,'Sirina kanala u dnu (m): ',f8.2/                   &
		           5x,'Nagib kosina          1: ',f8.2/                   &
				   5x,'Uzduzni nagib dna (-):   ',f8.4/                   &
				   5x,'Maningov koef. (m-1/3s): ',f8.3/                   &
				   5x,'Duzina kanala (m):       ',f8.1/ )                  
     END SUBROUTINE StampanjeKanal


  END MODULE Kanal
 !----------------------------------------------------------------------------------------




 !----------------------------------------------------------------------------------------
  MODULE ParametriProracuna
     use Kanal
     
	 integer :: iprint
     real    :: t, Tmax, dt, dtdx 


     contains


	 SUBROUTINE TrajanjeProracuna( ir )
	    integer, intent( in ) :: ir
	    read( ir, * ) Tmax, iprint  
     END SUBROUTINE TrajanjeProracuna



     real FUNCTION VremenskiKorak( ip, hh, QQ ) 
	    integer, intent( in ) :: ip
		real,    intent( in ) :: hh, QQ
		call FunkcijeProfila( ip, hh )
		VremenskiKorak = dx / ( ABS( QQ / pp( ip )%A ) + pp( ip )%c )
     END FUNCTION VremenskiKorak

		
  END MODULE ParametriProracuna  
 !----------------------------------------------------------------------------------------




 !----------------------------------------------------------------------------------------
  MODULE KonturniUslovi
    use ParametriProracuna
 
 	 real, allocatable, dimension(:) :: ANovo, QNovo, hNovo	
	 real                            :: Qo, ho  	 

     contains


     SUBROUTINE PocetniUslov( ir, iw2 )
	    integer, intent( in ) :: ir, iw2
	    integer :: i
		allocate( ANovo( npp1 ), QNovo( npp1 ), hNovo( npp1 ) )
		read( ir, * ) Qo, ho                 
		do i = 1, npp1
		   pp( i )%x = ( i - 1 ) * dx
		   pp( i )%Q = Qo
		   pp( i )%h = ho
		end do
		write( iw2, "(5x,'Pocetni protok (m3/s):',f10.2 )"  ) Qo
		write( iw2, "(5x,'Pocetna dubina (m):   ',f10.2/)"  ) ho		 
     END SUBROUTINE PocetniUslov



     SUBROUTINE UzvodniGranicniUslov
	    real :: cB, C2, IeB, V1	    
		hNovo( 1 ) = ho
		call FunkcijeProfila( 1, hNovo( 1 ) )
		call FunkcijeProfila( 2, pp( 2 )%h  )
		ANovo( 1 ) = pp( 1 )%A 
		cB = pp( 2 )%c
		IeB = CMan2 * pp( 2 )%Q * ABS( pp( 2 )%Q ) / pp( 2 )%A**2 / pp( 2 )%R**1.3333 
		C2 = pp( 2 )%V  - ( g / cB ) * pp( 2 )%h  + g * ( Id - IeB ) * dt
		V1 = C2 + ( g / cB ) * hNovo( 1 )
		QNovo( 1 ) = V1 * ANovo( 1 )
     END SUBROUTINE UzvodniGranicniUslov




     SUBROUTINE NizvodniGranicniUslov
	    real :: cA, C1, IeA
		QNovo( npp1 ) = 0.
		call FunkcijeProfila( npp, pp( npp )%h )
        cA = pp( npp )%c
        IeA = CMan2 * pp( npp )%Q * ABS( pp( npp )%Q ) / pp( npp)%A**2 / pp( npp )%R**1.3333 
		C1 = pp( npp )%V + ( g / cA ) * pp( npp )%h + g * ( Id - IeA ) * dt
		hNovo( npp1 ) = C1 / ( g / cA )
		call FunkcijeProfila( npp1, hNovo( npp1 ) )
        ANovo( npp1 ) = pp( npp1 )%A     
  	 END SUBROUTINE NizvodniGranicniUslov


  END MODULE KonturniUslovi
 !----------------------------------------------------------------------------------------




 !-----------------------------------------------------------------------------------------
  MODULE RacunskeSheme
     use KonturniUslovi
     
	 real :: theta

     contains


     SUBROUTINE LaxWendroff
	    integer :: i
        real :: Asr, Qsr, f2ip1, f2im1, Ieip1, Ieim1, s2i

		do i = 2,  npp 
		   call FunkcijeProfila( i-1,  pp( i-1 )%h )
		   call FunkcijeProfila( i+1,  pp( i+1 )%h )
		   Asr   = 0.5 * ( pp( i + 1 )%A + pp( i - 1 )%A )
		   Qsr   = 0.5 * ( pp( i + 1 )%Q + pp( i - 1 )%Q )
		   f2ip1 =  pp( i + 1 )%Q**2 / pp( i + 1 )%A + g * pp( i + 1 )%sm
		   f2im1 =  pp( i - 1 )%Q**2 / pp( i - 1 )%A + g * pp( i - 1 )%sm
		   Ieip1 = CMan2 * pp( i + 1 )%Q * ABS( pp( i + 1 )%Q ) /    &
		                   pp( i + 1 )%A**2 / pp( i + 1 )%R**1.333            
		   Ieim1 = CMan2 * pp( i - 1 )%Q * ABS( pp( i - 1 )%Q ) /    &
		                   pp( i - 1 )%A**2 / pp( i - 1 )%R**1.333    
		   s2i   = 0.5 * g * ( pp( i - 1 )%A * ( Id - Ieim1 ) +      &
		                       pp( i + 1 )%A * ( Id - Ieip1 ) ) 
           ANovo( i ) = theta * pp( i )%A + ( 1. - theta ) * Asr -   &
		                dtdx * 0.5 * ( pp( i + 1 )%Q - pp( i - 1 )%Q )
		   QNovo( i ) = theta * pp( i )%Q + ( 1. - theta ) * Qsr -   &
		                dtdx * 0.5 * ( f2ip1 - f2im1 )  + s2i * dt 
		   if ( m <= 0. ) then
              hNovo( i ) = ANovo( i ) / b
		   else
		      hNovo( i ) = Dubina( ANovo( i ) )
		   endif	  		 		             
        enddo
	 END SUBROUTINE LaxWendroff

     

	 SUBROUTINE MacCormack
	    real :: f2i, f2im1, Ie, s2i, hPrediktor, IePrediktor, AKorektor, QKorektor
        real, allocatable, dimension(:) :: APrediktor, QPrediktor 
		real, allocatable, dimension(:) :: f2Prediktor, s2Prediktor
  
		allocate( APrediktor( npp1 ),  QPrediktor( npp1 ) )
		allocate( f2Prediktor( npp1 ), s2Prediktor( npp1 ) )

        ! Prediktor        
		do i = 2,  npp 
		   call FunkcijeProfila( i, pp( i )%h )
		   call FunkcijeProfila( i-1, pp( i - 1 )%h )
           APrediktor( i ) = pp( i )%A - dtdx * ( pp( i )%Q - pp( i - 1 )%Q )
           f2i   = pp( i   )%Q**2 / pp( i   )%A + g * pp( i   )%sm
		   f2im1 = pp( i-1 )%Q**2 / pp( i-1 )%A + g * pp( i-1 )%sm
		   Ie  = CMan2 * pp( i )%Q * ABS( pp( i )%Q ) /  pp( i )%A**2 / pp( i )%R**1.333            
		   s2i = - g * pp( i )%A * ( Id - Ie ) * dt
		   QPrediktor( i ) = pp( i )%Q - dtdx * ( f2i - f2im1 ) - s2i
		   if( m <= 0. ) then
		      hPrediktor = APrediktor( i ) / b
           else
		      hPrediktor = Dubina( APrediktor( i ) )
           endif
		   call FunkcijeProfila( i, hPrediktor ) 
		   f2Prediktor( i ) =  QPrediktor( i )**2 / pp( i )%A + g * pp( i )%sm 
           IePrediktor = CMan2 * QPrediktor( i ) * ABS( QPrediktor( i ) ) /    &
		                 pp( i )%A**2 / pp( i )%R**1.333
		   s2Prediktor( i ) = g * APrediktor( i ) * ( Id - IePrediktor )      		                          		   
        end do
        QPrediktor( npp1 ) = 0.
		call FunkcijeProfila( npp1, hNovo( npp1 ) )
		f2Prediktor( npp1 ) = g * pp( npp1 )%sm
            
        ! Korektor i nove vrednosti
        do i = 2, npp
		   call FunkcijeProfila( i, pp( i )%h )
           AKorektor = pp( i )%A - dtdx * ( QPrediktor(  i + 1 ) - QPrediktor(  i ) )
           QKorektor = pp( i )%Q - dtdx * ( f2Prediktor( i + 1 ) - f2Prediktor( i ) ) &
		                         + s2Prediktor( i ) * dt
           ANovo( i ) = 0.5 * ( APrediktor( i ) + AKorektor )
           QNovo( i ) = 0.5 * ( QPrediktor( i ) + QKorektor )
		   if ( m <= 0. ) then
              hNovo( i ) = ANovo( i ) / b
		   else
		      hNovo( i ) = Dubina( ANovo( i ) )
		   endif	  		 
        enddo
     END SUBROUTINE MacCormack



   
     SUBROUTINE StampanjeRezultata( iw2, iw3, t )
	    integer, intent( in ):: iw2, iw3
		integer              :: i
		write( iw2, 107 ) t
        write( iw3, *   ) t
		write( iw2, 108 )
		do i = 1, npp1
		   call FunkcijeProfila( i, pp( i )%h )
		   write( iw2, 109 ) i, pp( i )%x, pp( i )%h, pp( i )%A, pp( i )%Bn,   &
			                    pp( i )%O, pp( i )%R, pp( i )%Q, pp( i )%V,    &
								( ABS( pp( i )%V ) + pp( i )%c ) * dt / dx 			    
           write( iw3, 110 )    pp( i )%x, pp( i )%h, pp( i )%Q
        end do
        107 format(/,25x,'t =',f10.2,' s')
        108 format(67('-'),/,1x,'Pr.    x',6x,'h',7x,'A',6x,'B',6x,'O',        &
                             5x,'R',8x,'Q',5x,'V',5x,'Cr',/,' (-)   (m)',      &
                             4x,'(m)',5x,'(m2)',3x,'(m)',4x,'(m)',3x,'(m)'     &
                             4x,'(m3/s)',1x,'(m/s)', 2x,'(-)'/67('=') )
        109 format( i3,f8.2,f7.3,f8.2,2f7.2,f6.2,f8.2,f7.3,f6.2)
        110 format( 3x,f8.3,2f10.2)
     END SUBROUTINE StampanjeRezultata



  END MODULE RacunskeSheme
 !----------------------------------------------------------------------------------------     




    PROGRAM Reke3T
        use RacunskeSheme

        character(12) :: datio, datu1, dati2, dati3, dati4
		integer       :: kod, kodr, k
		real          :: dtNovo

        ! Definicija I/O fajlova
        print "(5x,A)", "Ime fajla sa imenima I/O fajlova? "
        read "(A)", datio
        open( UNIT = 10, FILE = datio, STATUS = "old" )
        read( 10, "(A)" ) datu1 ! ulazni podaci
        read( 10, "(A)" ) dati2 ! rezultati - tabelarni prikaz
		read( 10, "(A)" ) dati3 ! rezultati - graficki  prikaz
		read( 10, "(A)" ) dati4 ! log fajl
        open( UNIT = 1, FILE = datu1, STATUS = "old" )
        open( UNIT = 2, FILE = dati2, STATUS = "replace" )
		open( UNIT = 3, FILE = dati3, STATUS = "replace" ) 
		open( UNIT = 4, FILE = dati4, STATUS = "replace" ) 
        
        ! Izbor racunske sheme
1       print 111
        111 format(/5x,'Racunske sheme:'/,                 &
                    5x,'    1. Lax-Wendroff'/,             &
                    5x,'    2. MacCormack  '/,             &
                    5x,'Izbor ? ')
        read *, kodr
        if( kodr /= 1 .and. kodr /= 2 .and. kodr /= 3 ) goto 1
		if( kodr == 1 ) then
2          print 222
           222 format(/5x,'Vrednost faktora theta ? ')
		   read *, theta
		   if( theta == 0 ) then
		      print *, 'Upozorenje: Lax-ova difuzivna shema ...'
           elseif( theta == 1 ) then
		      print *, 'Upozorenje: nestabilna shema ...'
			  goto 2
		   endif	   
		endif

        ! Ucitavanje ulaznih podataka
		call PodaciKanal    ( 1 )
		call StampanjeKanal ( 2 )

        ! Pocetni uslovi
        t = 0.
		k = 0
		call PocetniUslov( 1, 2 )
		dt = VremenskiKorak( 1, ho, Qo )
		dtdx = dt / dx

        ! Proracun po vremenu
		call TrajanjeProracuna( 1 )
		k = 1
		do while( t <= Tmax )
		   t = t + dt
           call UzvodniGranicniUslov
		   call NizvodniGranicniUslov
		   if( kodr == 1 ) call LaxWendroff
		   if( kodr == 2 ) call MacCormack	
		   kod = 0
		   i = 1
		   do while( i <= npp1 .and. kod == 0 )
              dtNovo = VremenskiKorak( i, hNovo( i ), QNovo( i ) )
		      if( dt <  0.75 * dtNovo ) dtNovo = 1.15 * dt
		      if( dt >= 0.75 * dtNovo ) dtNovo = dt
		      if( dt > dtNovo ) kod = 1
           i = i + 1
		   end do
		   if( kod == 1 ) then
		      t = t - dt
		      k = k - 1
		      dt = 0.9 * dtNovo
		   else
		      dt = dtNovo
			  k = k + 1
		      do i = 1, npp1
		         pp( i )%Q = QNovo( i ); pp( i )%h = hNovo( i )
              enddo
           endif
           dtdx = dt / dx 
		   print *, k,'     t = ', t
		   write( 4, 11 ) t, dt, dtNovo, dt/dtNovo
		   11 format(5x,'t = ',f8.2,5x,'dt = ',f8.2,5x,'dtNovo = ',f8.2,5x,'Cr = ',f6.2) 
           if( k / iprint * iprint == k ) call StampanjeRezultata( 2, 3, t )
        enddo 

        close( 1 );  close(	2 );  close( 3 ); close( 4 ); close( 10 )
     END PROGRAM Reke3T




