!     Last change:  MJ   13 Oct 2002    1:03 pm
 !----------------------------------------------------------------------------------------
 ! Program:   Reke4T
 ! Funkcija:  Neustaljeno tecenje u prizmaticnom kanalu trapeznog preseka
 ! Opis:      Resavanje St.Venant-ovih jednacina linijskog neustaljenog toka
 !            primenom implicitne Preissmann-ove sheme ("4 tacke"). 
 !         
 !            Pocetni uslov: zadati protok i racunate dubine u profilima
 !            Uzvodni  granicni uslov (profil br. "1") : hidrogram
 !            Nizvodni granicni uslov (profil br. "np"): hidrogram 
 !                                                       nivogram 
 !                                                       kriva protoka
 !            Ogranicenje programa:  miran rezim tecenja
 !            Teorijsko objasnjenje: videti fajl Reke4T.pdf               
 !
 ! Autor:     M. Jovanovic, 1.04.1983,  9.07.1987., 21.08. 2002.
 ! Okruzenje: Microsoft Developer Stuido, Visual Fortran V.6,6.0, 2001.
 !----------------------------------------------------------------------------------------



 !----------------------------------------------------------------------------------------
  MODULE Kanal
   
     implicit none
	 character(75)      :: text            ! identifikacija proracuna
	 integer            :: i, im1, ip1, ip ! indeksi 
	 integer            :: np              ! broj profila
     integer, parameter :: maxnp = 500     ! maksimalni broj profila
     real,    parameter :: g = 9.81        ! gravitaciono ubrzanje
	 real               :: b, m, Id        ! kanal: sirina, nagib kosina i dna
	 real               :: CMan            ! Manning-ov koeficijent
	 real               :: zd1             ! kota dna na nizvodnom kraju deonice
	 real               :: xL, dx          ! duzina deonice, razmak profila

     	 
	 TYPE :: Profil
	    real :: x       ! stacionaza
		real :: Q       ! protok
		real :: Z       ! kota nivoa
        real :: h       ! dubina
		real :: zd      ! kota dna
		real :: A       ! povrsina
        real :: Bn      ! sirina nivoa
        real :: O       ! okvaseni obim
        real :: R       ! hidraulicki radijus
		real :: V       ! srednja profilska brzina
		real :: E       ! ukupna energija
		real :: c       ! brzina prostiranja talasa
		real :: K       ! hidraulicki modul
		real :: dKdZ    ! izvod hidraulickog modula
     END TYPE Profil

     TYPE( Profil ), dimension(:), allocatable :: pp


     contains



     SUBROUTINE PodaciKanal( ir )
        integer, intent( in ) :: ir
        read( ir, "(A)" ) text                ! identifikacija
		read( ir, * ) b, m, Id, CMan, xL, zd1 ! elementi kanala
		read( ir, * ) np                      ! broj profila
		if( np > maxnp ) stop 'Premasen maks. broj profila !'
		allocate( pp( np ) ) 
     END SUBROUTINE PodaciKanal



      
     SUBROUTINE StacionazaKotaDna
	 	dx = xL / FLOAT( np - 1 )
        pp( 1 )%x = 0.
		pp( 1 )%zd = zd1 + xL * Id
        do i = 2, np
		   pp( i )%x  = pp( i-1 )%x + dx
		   pp( i )%zd = pp( 1 )%zd - Id * pp( i )%x
		end do
	 END SUBROUTINE StacionazaKotaDna




     SUBROUTINE FunkcijeProfila ( ip, hh )
	    integer, intent( in ) :: ip
        real,    intent( in ) :: hh
        pp( ip )%Bn   = b + 2. * hh * m                       ! sirina vodenog ogledala
		pp( ip )%A    = hh * ( b + m * hh )                   ! povrsina preseka
        pp( ip )%O    = b + 2. * hh * SQRT( 1. + m**2 )       ! okvaseni obim
        pp( ip )%R    = pp( ip )%A / pp( ip )%O               ! hidraulicki radijus     
		pp( ip )%V    = pp( ip )%Q / pp( ip )%A               ! srednja profilska brzina
		pp( ip )%c    = SQRT( g * pp( ip )%A / pp( ip )%Bn )  ! brzina prostiranja talasa
		pp( ip )%Z    = pp( ip )%zd + pp( ip )%h              ! kota nivoa
        pp( ip )%K    = pp( ip )%A * pp( ip )%R**0.6667 / CMan ! hidraulicki modul
		pp( ip )%dKdZ = ( pp( ip )%K / pp( ip )%A ) * &       ! izvod modula
		                ( 5. * pp( ip )%Bn - 4. * pp( ip )%R *  &
                        SQRT( 1. + m * m ) ) / 3.
     END SUBROUTINE FunkcijeProfila

      


     SUBROUTINE StampaKanal( iw2 )
	    integer, intent( in ):: iw2
		write( iw2, "(A)"  ) text
		write( iw2, "(/5x,A/)" ) "PODACI O KANALU:"
		write( iw2, 101) b, m, Id, Cman, xL
		101 format(5x,'Sirina u dnu (m):     ',10x,f8.2/              &
		           5x,'Nagib kosina 1:       ',10x,f8.2/              &
				   5x,'Uzduzni nagib dna (-):',10x,f8.4/              &
				   5x,'Maningov koeficijent: ',10x,f8.3/              &
				   5x,'Duzina (m):           ',10x,f8.2  )
     END SUBROUTINE StampaKanal


  END MODULE Kanal
 !----------------------------------------------------------------------------------------





  
 !----------------------------------------------------------------------------------------
  MODULE PocetniUslovi
     use Kanal

     integer :: nit, ier ! broj iteracija, indikator greske
	 real    :: Qo, ho   ! pocetni protok i dubina na nizvodnom kraju deonice
	 real    :: hn, hk   ! normalna, kriticna dubina        
     real, parameter :: eps = 0.001 ! zahtevana tacnost



     contains



     SUBROUTINE PocetniProtokDubina( ir, iw2 )
	    integer, intent( in ) :: ir, iw2
	    integer :: i
		read( ir, * ) Qo, ho                 
		do i = 1, np
		   pp( i )%Q = Qo
        enddo
		   pp( np )%h = ho
		   pp( np )%Z = zd1 + ho
		write( iw2, "(/5x,A/)" ) "POCETNI USLOVI:"
		write( iw2, "(5x,'Protok (m3/s):',18x,f8.2 )"  ) Qo
		write( iw2, "(5x,'Dubina (m):   ',18x,f8.2/)"  ) ho		 
     END SUBROUTINE PocetniProtokDubina




     SUBROUTINE NormalnaKriticnaDubina( iw2 )
	    integer, intent( in ) :: iw2
	    real, parameter :: hMin = 0.10, hMax = 20.
        call neli( hMin, hMax, fn, hn, eps, nit, ier )                      
        if( ier == 1 )       stop 'Ne konvergira normalna dubina !'
		call neli( hMin, hMax, fk, hk, eps, nit, ier )
		if( ier == 1 )       stop 'Ne konvergira kriticna dubina !'
		if( hn < 0.95 * hk ) stop 'Silovit rezim: hn < hk !'
		if( ho <= hk )       stop 'Silovit rezim: ho < hk !'
		write( iw2, "(/5x,A/)" ) "REZULTATI PRORACUNA:"
		write( iw2, "(5x,'Normalna dubina (m):',12x,f8.2 )"  ) hn
		write( iw2, "(5x,'Kriticna dubina (m):',12x,f8.2/)"  ) hk		 
	 END SUBROUTINE NormalnaKriticnaDubina
    



     SUBROUTINE PocetnaLinijaNivoa
        real, parameter :: hMin = 0.1, hMax = 20.
        do i = np, 2, -1
		   call FunkcijeProfila( i, pp( i )%h )
		   im1 = i - 1
           call neli( hMin, hMax, fe, pp( im1 )%h, eps, nit, ier )                      
           if( ier == 1 ) stop 'Ne konvergira pocetna linija nivoa !'
        enddo
     END SUBROUTINE PocetnaLinijaNivoa

	 

     real FUNCTION fn( hh )      
		call FunkcijeProfila( np, hh )
        fn = ( Qo / pp( np )%K )**2 - Id
	END FUNCTION fn




     real FUNCTION fk( hh )
        call FunkcijeProfila( np, hh )
        fk = 1. - ( Qo * Qo * pp( np )%Bn / g / pp( np )%A**3 )
     END FUNCTION fk




     real FUNCTION fe( hh )
		call FunkcijeProfila( im1, hh )
        fe = hh        + pp( im1 )%zd + pp( im1 )%V**2 / 2. / g -    &
             pp( i )%h - pp( i   )%zd - pp( i   )%V**2 / 2. / g -    &
             0.5 * ( (Qo / pp( im1 )%K )**2 + ( Qo / pp( i )%K )**2  ) * dx
     END FUNCTION fe
        




     SUBROUTINE neli(xd, xg, f, xo, eps, nit, ier)
        implicit none
        real,    external       :: f
        real,    intent( in  )  :: xd, xg, eps
        real,    intent( out )  :: xo
        integer, intent( out )  :: nit, ier
        integer                 :: it
        real                    :: xa, xb, xp, yd, yg, yo, p, po

        xa = xd;   xb = xg;   xp = xa
        if( xa >= xb ) then
           xa = xb;   xb = xp
        endif
        yd = f( xa );   yg = f( xb )
        p = yd * yg
        if( p == 0. ) then
           if( yd == 0. ) xo = xa
           if( yg == 0. ) xo = xb
        nit = 1; ier = 0;  return
        elseif( p < 0. ) then
           nit = INT( LOG( ( xb - xa ) / eps )/LOG(2.) ) + 1
           do it = 1, nit
              xo = 0.5 * ( xa + xb ); yo = f( xo )
              po = yd * yo
              if( po == 0. ) then
                 nit = it; ier = 0;  return
              elseif( po < 0. ) then
                 xb = xo; yg = yo
              else
                 xa = xo; yd = yo
              endif
           enddo
           xo = 0.5 * ( xa + xb )
           nit = it; ier = 0; return 
        else
           ier = 1; return
        endif
     END SUBROUTINE neli               



  END MODULE PocetniUslovi
 !----------------------------------------------------------------------------------------





 !----------------------------------------------------------------------------------------
  MODULE ParametriProracuna
     use PocetniUslovi
     
	 integer :: iprint ! broj koraka do stampanja rezultata
         INTEGER :: kodlog ! 1-stampa log fajl, 0-ne stampa log fajl
     real    :: t, Tmax, dt, dx2dt, theta


     contains


	 SUBROUTINE RacunskiParametri( ir, iw2 )
	    integer, intent( in ) :: ir, iw2
	    read( ir, * ) dt, Tmax, theta, iprint, kodlog
		write( iw2, 102) dt, Tmax, theta, iprint
		102 format(5x,'RACUNSKI PARAMETRI:'//                        &
                           5x,'Zadati vremenski korak (min):   ',f8.2 /      &
		           5x,'Trajanje proracuna (min):       ',f8.2 /      &
		           5x,'Tezinski faktor theta (-):      ',f8.3 /      &
                           5x,'Korak stampanja rezultata:      ',i8     )
		dt = dt * 60.;  Tmax = Tmax * 60. 
		dx2dt = dx / dt / 2.				                
     END SUBROUTINE RacunskiParametri




     SUBROUTINE VremenskiKorak( iw2 )
	    integer, intent( in ) :: iw2
        real :: Vabs, Cr, CV, dt1, dt2, dt3, dt4, dt5
		call FunkcijeProfila( 1, ho )
		Vabs = pp( 1 )%V + pp( 1 )%c
		Cr = Vabs * dt / dx
		CV = dx / Vabs
		dt1 = CV; dt2 = 2. * CV; dt3 = 3. * CV; dt4 = 4. * CV; dt5 = 5. * CV
	    print *
	    print *,'Ucitani vremenski korak dt: ', dt / 60.,' min'
	    print *,'Courant-ov broj: ', Cr
	    print *
	    print *,'  (  Cr = 1  ==> dt =', dt1 / 60.,' min ) '
	    print *,'  (  Cr = 2  ==> dt =', dt2 / 60.,' min ) '
	    print *,'  (  Cr = 3  ==> dt =', dt3 / 60.,' min ) '
	    print *,'  (  Cr = 4  ==> dt =', dt4 / 60.,' min ) '
	    print *,'  (  Cr = 5  ==> dt =', dt5 / 60.,' min ) '		
	    print *
	    print *,'Konacno usvojeni vremenski korak dt (min)? '
	    read *, dt; dt = dt * 60.; Cr = Vabs * dt / dx
		write( iw2, 103) dt / 60., Cr
		103 format(5x,'Usvojeni vremenski korak (min): ', f8.2 /     &
		           5x,'Courant-ov broj (-):            ', f8.2 )  
	 END SUBROUTINE VremenskiKorak


		
  END MODULE ParametriProracuna  
 !----------------------------------------------------------------------------------------






 !----------------------------------------------------------------------------------------
  MODULE GranicniUslovi
    use ParametriProracuna
 

     integer :: nuzv, nniz ! broj podataka za granicne uslove
     integer :: kodng      ! indikator: 1-hidrogram, 2-nivogram, 3-kriva protoka
	 real    :: Qug        ! uzvodni protok
	 real    :: Qng, Zng   ! nizvodni protok, kota nivoa
	 real    :: fQZ, dQdZ  ! interpolovana vrednost sa krive protoka, izvod
	 real, allocatable, dimension( : ) :: tuzv, Quzv	   ! uzvodni  hidrogram
	 real, allocatable, dimension( : ) :: tniz, Qniz, Zniz ! nizvodni hidrogram / nivogram
	 real, allocatable, dimension( : ) :: ZKrivaP, QKrivaP ! podaci za krivu protoka



     contains



     SUBROUTINE UzvodniGranicniUslov( ir )
	    integer, intent( in ) :: ir
		read( ir, * ) nuzv 
		allocate( tuzv( nuzv ), Quzv( nuzv ) )                     
		read( ir, * ) ( tuzv( i ), i = 1, nuzv ) 
		read( ir, * ) ( Quzv( i ), i = 1, nuzv ) 
        do i = 1, nuzv
		   tuzv( i ) = tuzv( i ) * 60.          
		enddo
     END SUBROUTINE UzvodniGranicniUslov




     SUBROUTINE NizvodniGranicniUslov( ir )
        integer, intent( in ) :: ir
        read( ir, * ) kodng 
        read( ir, * ) nniz
		allocate( tniz( nniz ), Qniz( nniz ), Zniz( nniz ) )
		allocate( ZKrivaP( nniz ), QKrivaP( nniz ) )
		SELECT CASE( kodng )
           case( 1 ) ! hidrogram
		      read( ir, * ) ( tniz( i ), i = 1, nniz )
			  read( ir, * ) ( Qniz( i ), i = 1, nniz )
			  do i = 1, nniz
			    tniz( i ) = tniz( i ) * 60.
			  enddo
           case( 2 ) ! nivogram
		      read( ir, * ) ( tniz( i ), i = 1, nniz )
			  read( ir, * ) ( Zniz( i ), i = 1, nniz )
			  do i = 1, nniz
			    tniz( i ) = tniz( i ) * 60.
			  enddo
           case( 3 ) ! kriva protoka
		      read( ir, * ) ( ZKrivaP( i ), i = 1, nniz )
			  read( ir, * ) ( QKrivaP( i ), i = 1, nniz )
        END SELECT
	 END SUBROUTINE NizvodniGranicniUslov 





     SUBROUTINE InterpolacijaUzvodnogGrUslova( iw4 )
	    integer, intent( in ) :: iw4

        if( t >= tuzv( nuzv ) ) then
		   Qug = Quzv( nuzv )
		   write( iw4, "(A)" ) "Ekstrapolacija uzvodnog granicnog uslova!"
		   return
		else
		   do i = 1, nuzv - 1
		      ip1 = i + 1
			  if(  ( t >= tuzv( i ) ) .and. ( t <= tuzv( ip1 ) ) )  then
                Qug = Quzv( i ) + ( Quzv( ip1 ) - Quzv( i ) ) *      &
				      ( t - tuzv( i ) ) / ( tuzv( ip1 ) - tuzv( i ))
			    return
			  endif
		   enddo   
           stop 'Interpolacija uzvodnog granicnog uslova ?!'
		endif
	 END SUBROUTINE InterpolacijaUzvodnogGrUslova


        


     SUBROUTINE InterpolacijaNizvodnogGrUslova( iw4 )
	    integer, intent( in ) :: iw4	    
		real :: ZZ

        SELECT CASE( kodng )
		   case( 1 ) ! hidrogram
              if( t >= tniz( nniz ) ) then
		         Qng = Qniz( nniz )
		         write( iw4, "(A)" ) "Ekstrapolacija nizvodnog granicnog uslova!"
		         return
		      else
		         do i = 1, nniz - 1
		            ip1 = i + 1
			        if(  ( t >= tniz( i ) ) .and. ( t <= tniz( ip1 ) ) )  then
                       Qng = Qniz( i ) + ( Qniz( ip1 ) - Qniz( i ) ) *         &
				            ( t - tniz( i ) ) / ( tniz( ip1 ) - tniz( i ))
			           return
			        endif
		         enddo   
                 stop 'Interpolacija nizvodnog granicnog uslova ?!'
		      endif
           case( 2 ) ! nivogram
              if( t >= tniz( nniz ) ) then
		         Zng = Zniz( nniz )
		         write( iw4, "(A)" ) "Ekstrapolacija nizvodnog granicnog uslova!"
		         return
		      else
		         do i = 1, nniz - 1
		            ip1 = i + 1
			        if(  ( t >= tniz( i ) ) .and. ( t <= tniz( ip1 ) ) )  then
                       Zng = Zniz( i ) + ( Zniz( ip1 ) - Zniz( i ) ) *         &
				            ( t - tniz( i ) ) / ( tniz( ip1 ) - tniz( i ))
                                            pp( np )%h = pp( np )%Z - pp( np )%zd
			           return
			        endif
		         enddo   
                 stop 'Interpolacija nizvodnog granicnog uslova ?!'
		      endif
           case( 3 ) !kriva protoka
		      ZZ = pp( np )%Z
              if( ZZ >= ZKrivaP( nniz ) ) then
			     fQZ = QKrivaP( nniz )
				 write( iw4, "(A)" ) "Ekstrapolacija krive protoka!"
				 return
			  else
			     do i = 1, nniz - 1
				    ip1 = i + 1
					if(  ( ZZ >= ZKrivaP( i ) ) .and. ( ZZ < ZKrivaP( ip1 ) ) ) then
					   fQZ = QKrivaP( i ) + ( QKrivaP( ip1 ) - QKrivaP( i ) ) *     &
					         ( ZZ - ZKrivaP( i ) ) / ( ZKrivaP( ip1 ) - ZKrivaP( i ) )
					   dQdZ = ( QKrivaP( ip1 ) - QKrivaP( i ) ) /    &
					          ( ZKrivaP( ip1 ) - ZKrivaP( i ) )
                       return
					endif
				 enddo
				 stop 'Interpolacija krive protoka ?!'
			  endif
        END SELECT
     
	 END SUBROUTINE InterpolacijaNizvodnogGrUslova



  END MODULE GranicniUslovi
 !----------------------------------------------------------------------------------------





 !-----------------------------------------------------------------------------------------
  MODULE Preissmann
     use GranicniUslovi
     

     real :: KoefA,  KoefB,  KoefC,  KoefD,  KoefG     ! koeficijenti u jednacini kontinuiteta
	 real :: KoefAp, KoefBp, KoefCp, KoefDp, KoefGp    ! koeficijenti u dinamickoj jednacini
	 real :: Hpom, Jpom                                ! pomocni koeficijenti
	 real, allocatable, dimension( : ) :: KoefL, KoefM, KoefN, KoefE, KoefF ! Metoda dv.prolaza
	 real, allocatable, dimension( : ) :: dQ, dZ       ! prirastaji zavisno promenljivih
     real, allocatable, dimension( : ) :: Qnovo, Znovo ! nove vrednosti zavisno promenljivih              



     contains



     SUBROUTINE AlociranjeNizova
	    allocate( KoefL( np ), KoefM( np ), KoefN( np ) )
		allocate( KoefE( np ), KoefF( np ) )
		allocate( dQ( np ), dZ( np ), Qnovo( np ), Znovo( np ) )
	 END SUBROUTINE AlociranjeNizova




	 SUBROUTINE MetodaDvostrukogProlaza( iw4 )
            integer, intent( in ) :: iw4


            ! inicijalizacija
            do i = 1, np
               pp( i )%h = pp( i )%Z - pp( i )%zd
               call FunkcijeProfila( i, pp( i )%h )
            end do



        ! Prolaz unapred: proracun vrednosti koeficijenata
	!-------------------------------------------------
		call InterpolacijaUzvodnogGrUslova( 4 )

		KoefE( 1 ) = 0.
		dQ   ( 1 ) = Qug - pp( 1 )%Q
		KoefF( 1 ) = dQ( 1 )

        ! Koeficijenti u jednacini kontinuiteta (skalari)
        do i = 1, np - 1
           KoefA = - theta
	   KoefB =   theta
           KoefC = dx2dt * pp( i )%Bn
	   KoefD = dx2dt * pp( i+1 )%Bn
	   KoefG = pp( i+1 )%Q - pp( i )%Q

           ! Koeficijenti u dinamickoj jednacini (skalari)

	   KoefAp = dx2dt / g / pp( i )%A -                                              &
                    theta * pp( i )%Q / g / pp( i )%A**2 +                               &
		    theta * dx * pp( i )%Q / pp( i )%K**2

           KoefBp = dx2dt / g / pp( i+1 )%A +                                            &
		            theta * pp( i+1 )%Q / g / pp( i+1 )%A**2 +                           &
					theta * dx * pp( i+1 )%Q / pp( i+1 )%K**2

	   KoefCp = - theta -                                                            &
		            dx2dt * pp( i )%Q * pp( i )%Bn / g / pp( i )%A**2 +                  &
					theta * pp( i )%Q**2 * pp( i )%Bn / g / pp( i )%A**3 -               &
					theta * dx * pp( i )%Q * ABS( pp( i )%Q ) *                          &
					        pp( i )%dKdZ / pp( i )%K**3
           KoefDp = theta -                                                              &
		            dx2dt * pp( i+1 )%Q * pp( i+1 )%Bn / g / pp( i+1 )%A**2 -            &
					theta * pp( i+1 )%Q**2 * pp( i+1 )%Bn / g / pp( i+1 )%A**3 -         &
					theta * dx * pp( i+1 )%Q * ABS( pp( i+1 )%Q ) *                      &
					        pp( i+1 )%dKdZ / pp( i+1 )%K**3

	   KoefGp = pp( i+1 )%Z - pp( i )%Z +                                            &
		            pp( i+1 )%Q**2 / 2. / g / pp( i+1 )%A**2 -                           &
			    pp( i   )%Q**2 / 2. / g / pp( i   )%A**2 +                           &
			    ( dx / 2. ) * (  pp( i   )%Q * ABS( pp( i   )%Q )  / pp( i   )%K **2 +           &
					     pp( i+1 )%Q * ABS( pp( i+1 )%Q )  / pp( i+1 )%K **2  )


           ! Za dovoljno mali korak dt, moze se pretpostaviti linearna veza:
		   ! dQ( i ) = KoefE( i ) * dZ( i ) + KoefF( i ), pa se dinamicka jednacina 
		   ! svodi na oblik: dZ( i ) = dZ( i+1 ) + KoefE( i ) * dQ( i+1 ) + KoefN( i )

           Hpom = - KoefAp * KoefE( i ) - KoefCp 
		   KoefL( i ) = KoefDp / Hpom
		   KoefM( i ) = KoefBp / Hpom
		   KoefN( i ) = ( KoefAp * KoefF( i ) + KoefGp ) / Hpom

           ! Uvodeci izraze za dQ( i ) i dZ( i ), jednacina kontinuiteta se svodi na:
		   ! dQ( i+1 ) = KoefE( i+1 ) * dZ( i+1 ) + KoefF( i+1 )


           Jpom = - KoefA * KoefE( i ) * KoefM( i ) - KoefB - KoefC * KoefM( i ) 
		   KoefE( i+1 ) = ( KoefA * KoefE( i ) * KoefL( i ) + KoefC * KoefL( i ) +       &
		                    KoefD ) / Jpom 
           KoefF( i+1 ) = ( KoefA * ( KoefE( i ) * KoefN( i ) + KoefF( i ) ) +           &
		                    KoefC * KoefN( i ) + KoefG ) / Jpom


           if( kodlog == 1 ) then
              write( iw4, 90 ) i, i+1
              write( iw4, 91 ) KoefA, KoefB, KoefC, KoefD, KoefG, KoefAp, KoefBp, KoefCp, KoefDp, KoefGp
              write( iw4, 92 ) KoefL( i ), KoefM( i ), KoefN( i ), KoefE( i+1 ), KoefF( i+1 )
           endif
           90 format(2x,'Profili:', 2i5 )
           91 format(2x,'Koeficijenti A, B, C, D, G:     ', 5f14.4 /            &
                     2x,'Koeficijenti Ap, Bp, Cp, Dp, Gp:', 5f14.4 )
           92 format(2x,'Koeficijenti L, M, N, E, F:     ', 5f14.4 )


        enddo

        ! Uvodjenje nizvodnog granicnog uslova
        call InterpolacijaNizvodnogGrUslova( 4 )
        call FunkcijeProfila( np, pp( np )%h )
		SELECT CASE( kodng )
		   case( 1 ) ! hidrogram
		      dQ( np ) = Qng - pp( np )%Q
			  dZ( np ) = ( dQ( np ) - KoefF( np ) ) / KoefE( np )
           case( 2 ) ! nivogram
              dZ( np ) = Zng - pp( np )%Z
			  dQ( np ) = KoefE( np ) * dZ( np ) + KoefF( np )
           case( 3 ) ! kriva protoka
		      dZ( np ) = ( fQZ - KoefF( np ) - pp( np )%Q ) / ( KoefE( np ) - dQdZ )
              dQ( np ) = KoefE( np ) * dZ( np ) + KoefF( np )
        END SELECT

        ! Prolaz unazad: proracun nepoznatih Z i Q
	!-----------------------------------------
        Qnovo( np ) = pp( np )%Q + dQ( np )
	Znovo( np ) = pp( np )%Z + dZ( np )

        if( kodlog == 1 ) then
           write( iw4, "(2x,A)" ) "Prirastaji i nove vrednosti Z, Q:"
           write( iw4, 93 ) np, dZ( np ), dQ( np ), Znovo( np ), Qnovo( np )
        endif
        93 format(i6,4f14.4)


        do i = np - 1, 1, -1
		   dZ( i ) = KoefL( i ) * dZ( i+1 ) + KoefM( i ) * dQ( i+1 ) + KoefN( i )
		   dQ( i ) = KoefE( i ) * dZ( i ) + KoefF( i )
		   Znovo( i ) = pp( i )%Z + dZ( i )
		   Qnovo( i ) = pp( i )%Q + dQ( i)

                   if( kodlog == 1 ) then
                      write( iw4, 93 ) i, dZ( i ), dQ( i ), Znovo( i ), Qnovo( i )
                   endif

		enddo

        ! Dodela novih vrednosti
		do i = 1, np
		   pp( i )%Z = Znovo( i )
		   pp( i )%Q = Qnovo( i )
		enddo
            return
	 END SUBROUTINE MetodaDvostrukogProlaza



   
     SUBROUTINE StampaRezultati( iw2, iw3, iw4 )
	    integer, intent( in ):: iw2, iw3, iw4
		write( iw2, 107 ) t
		write( iw2, 108 )
                write( iw3, "(5x,f10.2)" ) t
		IF( kodlog == 1 ) write( iw4, 107 ) t

		do i = 1, np
		   write( iw2, 109 ) i, pp( i )%x, pp( i )%Z, pp( i )%h, pp( i )%A, pp( i )%Bn,   &
			                pp( i )%O, pp( i )%R, pp( i )%Q, pp( i )%V,               &
					pp( i )%Q**2 * pp( i )%Bn / g / pp( i )%A**3
                   write( iw3, 110 )    pp( i )%x, pp( i )%h, pp( i )%Z, pp( i )%Q
                enddo
        107 format(/25x,'t =',f10.2,' s')
        108 format(76('-'),/,1x,'Pr.    x',8x,'Z',6x,'h',7x,'A',6x,'B',6x,'O',           &
                             5x,'R',7x,'Q',6x,'V',5x,'Fr',/,' (-)   (m)',5x,'(mnm)',     &
                             3x,'(m)',5x,'(m2)',3x,'(m)',4x,'(m)',3x,'(m)'               &
                             4x,'(m3/s)',1x,'(m/s)', 2x,'(-)'/76('=') )
        109 format( i3,f8.2,2x,2f7.3,f8.2,2f7.2,f6.2,f8.2,f7.3,f6.2)
        110 format( 3x,f8.3,3f14.2)
     END SUBROUTINE StampaRezultati


  END MODULE Preissmann
 !----------------------------------------------------------------------------------------     






    PROGRAM Reke4T
        use Preissmann

        integer :: k
        character(12) :: datio, datu1, dati2, dati3, dati4

        ! Definicija I/O fajlova
        print "(5x,A)", "Ime fajla sa imenima I/O fajlova? "
        read "(A)", datio
        open( UNIT = 10, FILE = datio, STATUS = "old" )
        read( 10, "(A)" ) datu1 ! ulazni podaci
        read( 10, "(A)" ) dati2 ! rezultati - tabelarni prikaz
	read( 10, "(A)" ) dati3 ! rezultati - graficki  prikaz
	read( 10, "(A)" ) dati4 ! log fajl
        open( UNIT = 1, FILE = datu1, STATUS = "old" )
        open( UNIT = 2, FILE = dati2, STATUS = "replace" )
	open( UNIT = 3, FILE = dati3, STATUS = "replace" )
	open( UNIT = 4, FILE = dati4, STATUS = "replace" )

        ! Ulazni podaci
		call PodaciKanal( 1 )
		call StampaKanal( 2 )
		call StacionazaKotaDna
		call PocetniProtokDubina( 1, 2 )
		call UzvodniGranicniUslov ( 1 )
		call NizvodniGranicniUslov( 1 )
		call RacunskiParametri( 1, 2 )
		call VremenskiKorak( 2 )
		call NormalnaKriticnaDubina( 2 )

        ! Pocetni uslovi
        t = 0.; k = 0
                call AlociranjeNizova
		call PocetnaLinijaNivoa
		call StampaRezultati( 2, 3, 4 )

        ! Proracun po vremenu
		do while( t <= Tmax )
		   t = t + dt; k = k + 1
           call MetodaDvostrukogProlaza( 4 )
           if( k / iprint * iprint == k ) call StampaRezultati( 2, 3, 4 )		
        enddo 

        close( 1 );  close( 2 );  close( 3 ); close( 4 )
     END PROGRAM Reke4T

