 !----------------------------------------------------------------------
 ! Program:   SKor
 ! Funkcija:  Odredjivanje geometrije stabilnog sljuncanog korita
 ! Opis:      Kriterijum: minimum snage toka po jedinici duzine
 !
 !            Pretpostavke:
 !               o Korito je trapeznog poprecnog preseka
 !               o Otpori trenja se definisu obrascem Bray-a:
 !                 Lambda^(-1/2) = 0.248 + 2.36 * log ( R/d )
 !               o Pronos nanosa se definise bezdimenzionim izrazom MPM:
 !                 Ampm * Fi^(2.3) = ko * Theta - Bmpm
 !                 gde je: 
 !                 Ampm  - nagib regresione prave (0.25)
 !                 Bmpm  - odsecak na ordinati (0.047)
 !                 ko    - korekcija ( nr/n )^(3/2)
 !
 !            Racunska metoda: iterativna, polovljenje intervala
 !
 ! Autor:     M. Jovanovic, 29.11.1999.
 ! Okruzenje: Microsoft Developer Stuido, Visual Fortran V.5, 1997.
 !----------------------------------------------------------------------




 !----------------------------------------------------------------------
  MODULE KarakteristikeKanala
   implicit none
	 character(75)      :: text
	 integer            :: i, ii
	 integer, parameter :: maxnb = 20
     real               :: Q   


     TYPE :: KanalTip
	    integer :: nb                   ! broj podataka
        real, dimension( maxnb ) :: b   ! sirina u dnu
        real :: m                       ! nagib kosine
        real :: A                       ! povrsina
        real :: Bn                      ! sirina nivoa
        real :: O                       ! okvaseni obim
        real :: R                       ! hidraulicki radijus
        real :: V                       ! srednja brzina
        real :: Fr                      ! Frudov broj
     END TYPE KanalTip

     TYPE( KanalTip ) :: kanal


     contains


     SUBROUTINE KanalUcitavanjePodataka ( ir )
        integer, intent( in ) :: ir
        read( ir, "(A)" ) text
		read( ir, * ) kanal%nb                             ! broj podataka
		if( kanal%nb > maxnb ) stop 'Previse podataka b !'
		read( ir, * ) ( kanal%b( i ), i = 1, kanal%nb )    ! racunske sirine
        read( ir, * ) kanal%m                              ! nagib obala
		read( ir, * ) Q                                    ! merodavni protok
     END SUBROUTINE KanalUcitavanjePodataka     



     SUBROUTINE FunkcijeProfila ( ii, h )
	    integer, intent( in ) :: ii
        real,    intent( in ) :: h
        kanal%Bn      =  kanal%b( ii ) + 2. * h * kanal%m
		kanal%A       =  h * ( kanal%b( ii ) + kanal%m * h )
        kanal%O       =  kanal%b( ii ) + 2. * h * SQRT( 1. + kanal%m**2 )
        kanal%R       =  kanal%A / kanal%O
        kanal%V       =  Q / kanal%A
        kanal%Fr      =  Q * Q * kanal%Bn / 9.81 / kanal%A**3
     END SUBROUTINE FunkcijeProfila

  END MODULE KarakteristikeKanala
 !----------------------------------------------------------------------




 !----------------------------------------------------------------------
  MODULE VuceniNanos
     use KarakteristikeKanala
     implicit none
     real    :: ros, d, Qv      ! gustina, srednji precnik, ulazni tovar
     real    :: Delta           ! relativna gustina
	 real    :: Fi              ! bezdimenzioni pronos 
	 real    :: Ampm, Bmpm, ko  ! parametri obrasca MPM	 
	 real, parameter :: ro = 1. ! gustina vode [t/m3]


     contains


	 SUBROUTINE NanosUcitavanjePodataka( ir )
        integer :: ir
		read( ir, * ) ros                     
		read( ir, * ) d                       
		read( ir, * ) Qv                                         
        read( ir, * ) Ampm, Bmpm, ko      
		if( ros > 2.7 ) stop 'Jedinice za gustinu nanosa !? '
		Delta = ros - 1.   ! relativna gustina nanosa
		d = d / 1000.      ! srednje zrno [m]		
	 END SUBROUTINE NanosUcitavanjePodataka



	 real FUNCTION BezdimenzioniPronos( ii )
	    integer, intent( in ) :: ii
		BezdimenzioniPronos = Qv / kanal%b( ii ) / SQRT( 9.81 * Delta * d**3 ) 
     END FUNCTION BezdimenzioniPronos


  END MODULE VuceniNanos
 !----------------------------------------------------------------------





 !----------------------------------------------------------------------
  MODULE NormalnaKriticnaDubina
     use VuceniNanos
     implicit none
	 real :: Lambda, Id, Theta, Tau


     contains


     real FUNCTION fn( h )
		real, intent( in ) :: h
		call FunkcijeProfila( ii, h )
		Lambda = ( 2.36 * ALOG10( kanal%R / d ) + 0.248 )**(-2)
	    Id = ( Ampm * Fi**0.67 + Bmpm ) * Delta * d / kanal%R / ko                  
        fn = ( Lambda * kanal%V * kanal%V ) / ( 78.48 * kanal%R ) - Id
	END FUNCTION fn



    SUBROUTINE TangencijalniNapon( ii, hn )
	    integer, intent( in ) :: ii
		real,    intent( in ) :: hn
	    call FunkcijeProfila( ii, hn )
		Tau   = ro * 9.81 * kanal%R * Id
		Theta = Tau / ( 9.81 * ( ros - ro ) * d )
	END SUBROUTINE TangencijalniNapon      



     real FUNCTION fk ( h )
        real, intent( in ) :: h
        call FunkcijeProfila( ii, h )
        fk = 1. - kanal%Fr
     END FUNCTION fk



     SUBROUTINE StampanjeUlaznihPodataka( iw )
        integer, intent( in ) :: iw 
		write( iw, "(2x,A/)" ) text       
        write( iw, fmt=100 ) Q, Qv, ros, d
        100 format( 10x,'Merodavni protok (m3/s):       ',f10.2/     &
                    10x,'Pronos vucenog nanosa (m3/s):  ',f10.4/     &
					10x,'Gustina nanosa (t/m3):         ',f10.3/     &
					10x,'Srednji precnik zrna (m):      ',f10.3/)
     END SUBROUTINE StampanjeUlaznihPodataka



     SUBROUTINE StampanjeZaglavlja ( iw )
        integer, intent( in ) :: iw        
        write( iw, fmt=200)
        200 format (4x,'b',4x,'hk',4x,'hn',4x,'R',5x,'V',5x,'Fr',    &
        6x,'Id',6x,'Lambda',5x,'n',5x,'Tau',6x,'Fi',4x,'Theta' /     &
        3x,'(m)',3x,'(m)',3x,'(m)',2x,'(m)',2x,'(m/s)',3x,'(-)',     &
		5x,'(-)',7x,'(-)',2x,'(m-1/3s)',2x,'(Pa)',5x,'(-)',4x,'(-)' /)                    
     END SUBROUTINE StampanjeZaglavlja



     SUBROUTINE StampanjeRezultata ( iw, iwg, hk, hn )
        integer :: iw, iwg
		real    :: hk, hn
        write( iw, fmt=200 ) kanal%b( ii ), hk, hn, kanal%R,         &
		kanal%V, kanal%Fr, Id, Lambda, SQRT( Lambda / 78.48 ) *      &
		kanal%R**0.167, Tau, Fi, Theta
        200 format( 5f6.2, f7.3, f10.6, 5f8.4 )
		write( iwg, fmt=300 ) kanal%b ( ii ), hn, Id
		300 format( 2f10.2, f14.6 )
     END SUBROUTINE StampanjeRezultata

  END MODULE NormalnaKriticnaDubina
 !----------------------------------------------------------------------





     SUBROUTINE neli(xd, xg, f, xo, eps, nit, ier)
        implicit none
        real,    external       :: f
        real,    intent( in  )  :: xd, xg, eps
        real,    intent( out )  :: xo
        integer, intent( out )  :: nit, ier
        integer                 :: it
        real                    :: xa, xb, xp, yd, yg, yo, p, po

        xa = xd;   xb = xg;   xp = xa
        if( xa >= xb ) then
           xa = xb;   xb = xp
        endif
        yd = f( xa );   yg = f( xb )
        p = yd * yg
        if( p == 0. ) then
           if( yd == 0. ) xo = xa
           if( yg == 0. ) xo = xb
        nit = 1; ier = 0;  return
        elseif( p < 0. ) then
           nit = INT( LOG( ( xb - xa ) / eps )/LOG(2.) ) + 1
           do it = 1, nit
              xo = 0.5 * ( xa + xb ); yo = f( xo )
              po = yd * yo
              if( po == 0. ) then
                 nit = it; ier = 0;  return
              elseif( po < 0. ) then
                 xb = xo; yg = yo
              else
                 xa = xo; yd = yo
              endif
           enddo
           xo = 0.5 * ( xa + xb )
           nit = it; ier = 0; return 
        else
           ier = 1; return
        endif
     END SUBROUTINE neli    
   
   
   
   
     
     PROGRAM Skor
        use NormalnaKriticnaDubina
        character(12)   :: datio, datu1, dati2, dati3
		real, parameter :: hMin = 0.1, hMax = 20. 
		real            :: hn, hk
	    integer         :: nit, ier
        real, parameter :: eps = 1.e-3


        ! Definicija I/O fajlova
        print "(5x,A)", "Ime fajla sa imenima I/O fajlova? "
        read "(A)", datio
        open( UNIT = 10, FILE = datio, STATUS = "old" )
        read( 10, "(A)" ) datu1 ! ulazni podaci
        read( 10, "(A)" ) dati2 ! rezultati - tabelarni prikaz
		read( 10, "(A)" ) dati3 ! rezultati - graficki  prikaz
        open( UNIT = 1,  FILE = datu1,  STATUS = "old" )
        open( UNIT = 2,  FILE = dati2,  STATUS = "replace")
        open( UNIT = 3,  FILE = dati3,  STATUS = "replace")


        ! Ucitavanje ulaznih podataka
        call KanalUcitavanjePodataka ( 1 )
		call NanosUcitavanjePodataka ( 1 )
		call StampanjeUlaznihPodataka( 2 )
		call StampanjeZaglavlja      ( 2 )


        ! Variranje sirine korita
        do ii = 1, kanal%nb
		   Fi = BezdimenzioniPronos( ii )
           call neli( hMin, hMax, fn, hn, eps, nit, ier )                      
           if( ier == 1 ) stop 'Skor: ne konvergira normalna dubina !'           
		   call neli( hMin, hMax, fk, hk, eps, nit, ier )
		   if( ier == 1 ) stop 'Skor: ne konvergira kriticna dubina !'
		   call TangencijalniNapon( ii, hn )
           call StampanjeRezultata( 2, 3, hk, hn )
        end do

        close( 1 ); close (	2 ); close( 3 )
     END PROGRAM Skor