 !----------------------------------------------------------------------
 ! Program:   TKanal
 ! Funkcija:  Normalna/kriticna dubina u kanalu trapeznog preseka
 ! Opis:      Iterativno resavanje jednacine jednolikog tecenja
 !            metodom polovljenja intervala
 !
 ! Autor:     M. Jovanovic, 13.07.1983, 29.11.1999.
 ! Okruzenje: Microsoft Developer Stuido, Visual Fortran V.5, 1997.
 !----------------------------------------------------------------------




 !----------------------------------------------------------------------
  MODULE NormalnaKriticnaDubina
   
     implicit none
	 character(75)                   :: text
	 integer                         :: nQ, kod, nit, ier
	 integer, parameter              :: maxnQ = 20
     real, allocatable, dimension(:) :: Q   
     real, parameter                 :: ro = 1000., g = 9.81, eps = 1.e-3
	 real                            :: QQ
	 

     TYPE :: KanalTip
        real :: b                     ! sirina u dnu
        real :: m                     ! nagib kosine
		real :: Id                    ! nagib dna
		real :: CMan                  ! Maningov koef.
        real :: A                     ! povrsina
        real :: Bn                    ! sirina nivoa
        real :: O                     ! okvaseni obim
        real :: R                     ! hidraulicki radijus
		real :: K                     ! hidraulicki modul
        real :: V                     ! srednja brzina
        real :: Fr                    ! Frudov broj
		real :: Tauo                  ! tangencijalni napon
     END TYPE KanalTip

     TYPE( KanalTip ) :: kanal


     contains


     SUBROUTINE UcitavanjeUlaznihPodataka ( ir )
        integer :: i, ir
        read( ir, "(A)" ) text
		read( ir, * ) kanal%b, kanal%m, kanal%Id, kanal%CMan
		if( kanal%Id <= 0. )   stop 'Ne postoji normalna dubina !'
		read( ir, * ) nQ
		if( nQ > maxnQ ) stop 'Previse podataka za protok !'
		allocate( Q( nQ ) )
		read( ir, * ) ( Q( i ), i = 1, nQ )
     END SUBROUTINE UcitavanjeUlaznihPodataka     



     SUBROUTINE FunkcijeProfila ( h )
        real, intent( in ) :: h
        kanal%Bn   = kanal%b + 2. * h * kanal%m
		kanal%A    = h * ( kanal%b + kanal%m * h )
        kanal%O    = kanal%b + 2. * h * SQRT( 1. + kanal%m**2 )
        kanal%R    = kanal%A / kanal%O
        kanal%V    = QQ / kanal%A
		kanal%K    = kanal%A * kanal%R**0.667 / kanal%CMan
        kanal%Fr   = QQ * QQ * kanal%Bn / g / kanal%A**3
		kanal%Tauo = ro * g * kanal%R * kanal%Id
     END SUBROUTINE FunkcijeProfila



     real FUNCTION fn( h )
		real, intent( in ) :: h        
		call FunkcijeProfila( h )
        fn = ( QQ / kanal%K )**2 - kanal%Id
	END FUNCTION fn




     real FUNCTION fk ( h )
        real, intent( in ) :: h
        call FunkcijeProfila( h )
        fk = 1. - kanal%Fr
     END FUNCTION fk




     SUBROUTINE neli(xd, xg, f, xo, eps, nit, ier)
        implicit none
        real,    external       :: f
        real,    intent( in  )  :: xd, xg, eps
        real,    intent( out )  :: xo
        integer, intent( out )  :: nit, ier
        integer                 :: it
        real                    :: xa, xb, xp, yd, yg, yo, p, po

        xa = xd;   xb = xg;   xp = xa
        if( xa >= xb ) then
           xa = xb;   xb = xp
        endif
        yd = f( xa );   yg = f( xb )
        p = yd * yg
        if( p == 0. ) then
           if( yd == 0. ) xo = xa
           if( yg == 0. ) xo = xb
        nit = 1; ier = 0;  return
        elseif( p < 0. ) then
           nit = INT( LOG( ( xb - xa ) / eps )/LOG(2.) ) + 1
           do it = 1, nit
              xo = 0.5 * ( xa + xb ); yo = f( xo )
              po = yd * yo
              if( po == 0. ) then
                 nit = it; ier = 0;  return
              elseif( po < 0. ) then
                 xb = xo; yg = yo
              else
                 xa = xo; yd = yo
              endif
           enddo
           xo = 0.5 * ( xa + xb )
           nit = it; ier = 0; return 
        else
           ier = 1; return
        endif
     END SUBROUTINE neli               




     SUBROUTINE StampanjeUlaznihPodataka( iw )
	    integer, intent( in ):: iw
		write( iw, "(A)"  ) text
		write( iw, "(/1x,A/)" ) "Ulazni podaci:"
		write( iw, 101) kanal%b, kanal%m, kanal%Id, kanal%Cman
		101 format(3x,'Sirina kanala u dnu (m): ',f6.2/                   &
		           3x,'Nagib kosina:          1:',f6.2/                   &
				   3x,'Uzduzni nagib dna (-):   ',f6.4/                   &
				   3x,'Maningov koef. (m-1/3s): ',f6.3/ )               
     END SUBROUTINE StampanjeUlaznihPodataka




     SUBROUTINE StampanjeZaglavlja ( iw, kod )
        integer, intent( in ) :: iw, kod
		if( kod == 1 ) write( iw, "(/1x,A/)" ) "Normalna dubina"
		if( kod == 2 ) write( iw, "(/1x,A/)" ) "Kriticna dubina"        
        write( iw, fmt=100)
        100 format (5x,'Q',5x,'h',7x,'A',7x,'B',7x,'O',6x,'R',6x,'V',&
		7x,'Fr',4x,'Tauo'/2x,'(m3/s)',2x,'(m)',5x,'(m2)',4x,'(m)',   &
		5x,'(m)',4x,'(m)',3x,'(m/s)',4x,'(-)',4x,'(Pa)'/)                    
     END SUBROUTINE StampanjeZaglavlja




     SUBROUTINE StampanjeRezultata ( iw, h )
        integer, intent( in ) :: iw  
		real,    intent( in ) :: h
        write( iw, fmt=200 ) QQ, h, kanal%A, kanal%Bn, kanal%O,      &
        kanal%R, kanal%V, kanal%Fr, kanal%Tauo
        200 format(2f7.2, 3f8.2, 2f7.2, 2f8.3) 
     END SUBROUTINE StampanjeRezultata



  END MODULE NormalnaKriticnaDubina
 !----------------------------------------------------------------------




     
     PROGRAM TKanal
        use NormalnaKriticnaDubina
        character(12)   :: datio, datu1, dati2
		real, parameter :: hMin = 0.01, hMax = 20. 
		real            :: hn, hk   

        ! Definicija I/O fajlova
        print "(5x,A)", "Ime fajla sa imenima I/O fajlova? "
        read "(A)", datio
        open( UNIT = 10, FILE = datio, STATUS = "old" )
        read( 10, "(A)" ) datu1 ! ulazni podaci
        read( 10, "(A)" ) dati2 ! rezultati - tabelarni prikaz
        open( UNIT = 1,  FILE = datu1,  STATUS = "old" )
        open( UNIT = 2,  FILE = dati2,  STATUS = "replace")
        
        ! Ucitavanje ulaznih podataka
        call UcitavanjeUlaznihPodataka( 1 )
		call StampanjeUlaznihPodataka ( 2 )

        ! Normalna dubina
		call StampanjeZaglavlja( 2, 1 )
        do i = 1, nQ
		   QQ = Q( i )
           call neli( hMin, hMax, fn, hn, eps, nit, ier )                      
           if( ier == 1 ) stop 'Skor: ne konvergira normalna dubina !'
		   call StampanjeRezultata( 2, hn )
		end do

        ! Kriticna dubina
		call StampanjeZaglavlja( 2, 2 )
        do i = 1, nQ
		   QQ = Q( i )
		   call neli( hMin, hMax, fk, hk, eps, nit, ier )
		   if( ier == 1 ) stop 'Skor: ne konvergira kriticna dubina !'
           call StampanjeRezultata( 2, hk )
        end do

        close( 1 );  close(	2 );  close( 3 )
     END PROGRAM TKanal