 !----------------------------------------------------------------------------------------
 ! Program:   TTA
 ! Funkcija:  Transformacija talasa u akumulaciji
 ! Opis:      Zadaje se ulazni hidrogram Qu(t) i kriva povrsine Omega(z); 
 !            racuna se izlazni protok Qi(t), koji zavisi od karakterisitika 
 !            evakuacionih objekata (ispusta i preliva).
 !       
 !            Pretpostavke o evakuacionim objektima:
 !              o postoji najvise 2 preliva i 2 ispusta
 !              o prelivanje je uvek nepotopljeno
 !              o isticanje kroz ispust je uvek potopljeno
 !
 !            Racunski model: 
 !            Numericko resavanje diferencijalne jednacine I. reda:
 !                dZ / dt = ( Qu - Qi) / Omega(Z)
 !                Z(0) = Zo (zadati pocetni uslov),
 !            uz interpolaciju zadatih funkcija Qu(t) i Omega(Z) i
 !            proracun funkcije Qi(t) po jednacinama koje definisu
 !            kapacitet evakuacionih objekata.                 
 !                            
 ! Autor:     M. Jovanovic, 2.10.1987, 19.12.1998, 19.07.2002.
 ! Okruzenje: Microsoft Developer Studio, Visual Fortran V.5, 1997.
 !----------------------------------------------------------------------------------------




!----------------------------------------------------------------------------------------
  MODULE RacunskiParametri 

     implicit none
	 integer            :: ndt, npr
	 integer, parameter :: maxit = 50
	 real               :: tmax, dt
     real, parameter    :: eps = 0.005
     
     contains


     SUBROUTINE UcitavanjeRacunskiParametri ( ir )
        implicit none
        integer, intent( in ) :: ir
        read( ir, * ) tmax, ndt, npr
        dt = tmax / ndt * 3600.
     END SUBROUTINE UcitavanjeRacunskiParametri


  END MODULE RacunskiParametri
 !----------------------------------------------------------------------------------------




 !----------------------------------------------------------------------------------------
  MODULE Akumulacija

     implicit none
	 character(75) :: text
	 integer       :: nzOmega, nQt
	 real          :: Zo, BranaSirina, Qo
     real, allocatable, dimension (:) :: zn, Omega
     real, allocatable, dimension (:) :: tu, Qu
    

     contains


     SUBROUTINE UcitavanjeAkumulacija( ir, iga )
        implicit none
        integer, intent( in ) :: ir, iga
        integer, parameter    :: nzOmegamax = 50
        integer               :: i
        
        read( ir, "(A)") text
        read( ir, *    ) nzOmega
        if( nzOmega > nzOmegamax ) stop 'Premasen broj tacaka krive povrsine!'
        allocate( zn(nzOmega), Omega(nzOmega) )
        read( ir, * ) ( zn(i),    i = 1, nzOmega ) ! kote nivoa
        read( ir, * ) ( Omega(i), i = 1, nzOmega ) ! povrsina (u hektarima)
        read( ir, * ) Zo                           ! pocetna kota
		read( ir, * ) Qo                           ! pocetni protok u koritu
        do i = 1, nzOmega
           write( iga, * ) Omega(i), zn(i)
        enddo
     END SUBROUTINE UcitavanjeAkumulacija




     SUBROUTINE UcitavanjeUlazniHidrogram( ir )
        implicit none
        integer, intent( in ) :: ir
        integer, parameter    :: nQtmax = 100
        integer               :: i

        read( ir, * ) nQt
        if( nQt > nQtmax ) stop 'Premasen broj tacaka krive Qu(t)!'
        allocate( tu(nQt), Qu(nQt) )
        read( ir, * ) ( tu(i), i = 1, nQt )   ! vreme (u satima)
        read( ir, * ) ( Qu(i), i = 1, nQt )   ! dotok u akumulaciju
     END SUBROUTINE UcitavanjeUlazniHidrogram



  END MODULE Akumulacija
 !----------------------------------------------------------------------------------------





 !----------------------------------------------------------------------------------------
  MODULE RecnoKorito

     implicit none
	 integer, parameter        :: maxt = 50

     TYPE :: ProfilTip
        integer                :: nt   ! broj koordinatnih tacaka
        real, dimension (maxt) :: zp   ! kota dna
        real, dimension (maxt) :: yp   ! odstojanje
		real                   :: zd   ! najniza kota dna
		real                   :: zo   ! najniza kota obale
        real                   :: Id   ! lokalni nagib dna 
        real                   :: Zn   ! kota normalne dubine
		real                   :: Q    ! protok
        real                   :: A    ! povrsina preseka
        real                   :: B    ! sirina vodenog ogledala
        real                   :: O    ! okvaseni obim
        real                   :: R    ! hidraulicki radijus
        real                   :: K    ! modul protoka 
        real                   :: CMan ! Manningov koef. rapavosti        
        real                   :: V    ! srednja brzina
        real                   :: V2g  ! brzinska visina        
        real                   :: Fr   ! Frudov broj
     END TYPE ProfilTip

     TYPE( ProfilTip ) :: Profil



     contains



     SUBROUTINE UcitavanjeProfil( ir )
        implicit none
		integer, intent( in ) :: ir 
		integer               :: j

		read( ir, * ) Profil%nt
        if( Profil%nt > maxt ) stop 'Profil: broj tacaka ?'
        read( ir, * ) ( Profil%zp( j ), j = 1, Profil%nt )
        read( ir, * ) ( Profil%yp( j ), j = 1, Profil%nt )
		read( ir, * ) Profil%Id
		if( Profil%Id <= 0 ) stop 'Profil: uzduzni nagib dna korita ?'
		read( ir, * ) Profil%CMan
		Profil%zd = MINVAL( RESHAPE( Profil%zp, (/Profil%nt/) ) )
		Profil%zo = MIN( Profil%zp( 1 ), Profil%zp( Profil%nt ) )
     END SUBROUTINE UcitavanjeProfil


     

     SUBROUTINE FunkcijeProfila( ZZ )
        integer :: i, k 
        real    :: ZZ, Zp, Zpp, dy, dz, ai, bi

        if( ZZ > Profil%zo ) stop ' Izlivanje u profilu !'
		if( ZZ < Profil%zd ) stop ' Nivo ispod najnize kote dna !'
		Profil%A = 0.; Profil%B = 0.; Profil%O = 0.
       
        do i = 1, Profil%nt - 1
           dy = Profil%yp( i+1 ) - Profil%yp( i )
           dz = Profil%zp( i+1 ) - Profil%zp( i )
           Zp = ( ZZ - Profil%zp( i ) ) * ( ZZ - Profil%zp( i+1 ) )

           if( Zp >  0. .and. ZZ < Profil%zp( i ) ) k = 1 ! seg. iznad nivoa
           if( Zp >  0. .and. ZZ > Profil%zp( i ) ) k = 2 ! seg. ispod nivoa
           if( Zp <= 0. .and. dz /= 0. )            k = 3 ! nivo sece segment
           if( Zp == 0. .and. dz == 0. )            k = 4 ! nivo = horiz.seg.

           SELECT CASE ( k )
              case ( 1 )
              case ( 2 )
                 bi = dy
                 Profil%B = Profil%B + bi
                 Profil%A = Profil%A + bi * ( ZZ - 0.5 *                  &
				            ( Profil%zp( i+1 ) + Profil%zp( i ) ) )
                 Profil%O = Profil%O + SQRT( bi**2 + dz**2 )
              case ( 3 )
                 if( ZZ > Profil%zp( i   ) )   Zpp = Profil%zp( i   )
                 if( ZZ > Profil%zp( i+1 ) )   Zpp = Profil%zp( i+1 )
                 bi = dy * ( ZZ - Zpp ) / ABS( dz )
                 ai = bi * 0.5 * ( ZZ - Zpp )
                 Profil%B = Profil%B + bi
                 Profil%A = Profil%A + ai
                 Profil%O = Profil%O + SQRT( bi**2 + ( ZZ - Zpp )**2 )
              case ( 4 )
                 Profil%B = Profil%B + dy
                 Profil%O = Profil%O + dy
           END SELECT
        enddo
        Profil%R   = Profil%A / Profil%O
        Profil%K   = Profil%A * Profil%R**.666667 / Profil%CMan 
        Profil%V   = Profil%Q / Profil%A
        Profil%Fr  = Profil%V**2 * Profil%B / 9.81 / Profil%A
        Profil%V2g = Profil%V**2 / 19.62            
     END SUBROUTINE FunkcijeProfila




     real FUNCTION NormalnaDubina ( ZZ )
        real, intent( in ) :: ZZ
        call FunkcijeProfila ( ZZ )
        NormalnaDubina = ( Profil%Q / Profil%K )**2 - Profil%Id
     END FUNCTION NormalnaDubina




     SUBROUTINE neli(xd, xg, f, xo, eps, nit, ier)
        implicit none
        real,    external    :: f
        real,    intent(in)  :: xd, xg, eps
        real,    intent(out) :: xo
        integer, intent(out) :: nit, ier
        integer              :: it
        real                 :: xa, xb, xp, yd, yg, yo, p, po

        xa = xd;   xb = xg;   xp = xa
        if( xa >= xb ) then
           xa = xb;   xb = xp
        endif
        yd = f( xa );   yg = f( xb )
        p = yd * yg
        if( p == 0. ) then
           if( yd == 0. ) xo = xa
           if( yg == 0. ) xo = xb
        nit = 1; ier = 0;  return
        elseif( p < 0. ) then
           nit = INT( LOG( ( xb - xa ) / eps )/LOG(2.) ) + 1
           do it = 1, nit
              xo = 0.5 * ( xa + xb ); yo = f( xo )
              po = yd * yo
              if( po == 0. ) then
                 nit = it; ier = 0;  return
              elseif( po < 0. ) then
                 xb = xo; yg = yo
              else
                 xa = xo; yd = yo
              endif
           enddo
           xo = 0.5 * ( xa + xb )
           nit = it; ier = 0; return 
        else
           ier = 1; return
        endif
     END SUBROUTINE neli   



  END MODULE RecnoKorito
 !----------------------------------------------------------------------------------------





 !----------------------------------------------------------------------------------------
  MODULE EvakuacioniObjekti
     use RecnoKorito
 
     implicit none
	 integer, parameter  :: nQmax = 50

     TYPE :: PrelivTip
	    integer :: kodp   ! indikator krive protoka: 0-ne postoji, 1-racunata, 2-zadata
		integer :: nQ     ! broj tacaka zadate krive protoka
	    real    :: Zp     ! kota krune preliva
		real    :: Bp     ! sirina prelivne ivice
		real    :: CQp    ! koeficijent protoka
        real, dimension (nQmax) :: ZZP, QQP ! zadate tacke krive protoka
     END TYPE PrelivTip

	 TYPE( PrelivTip ) :: Preliv, Preliv1, Preliv2

	 TYPE :: IspustTip
	    integer :: kodi   ! indikator ispusta: 0 - ne postoji, 1-racunski kapacitet
	    real    :: Zi     ! kota osovine ispusne cevi
		real    :: Ai     ! povrsina preseka cevi
		real    :: CQi    ! koeficijent protoka
	 END TYPE IspustTip

	 TYPE( IspustTip ) :: Ispust, Ispust1, Ispust2


   
     contains



     SUBROUTINE UcitavanjeEvakuacioniObjekti ( ir, Preliv, Ispust )
        implicit none
        integer, intent( in ) :: ir
        integer               :: i
	    TYPE( PrelivTip )     :: Preliv
		TYPE( IspustTip )     :: Ispust
				
        read( ir, * ) Preliv%kodp
        if( Preliv%kodp == 0 ) return
		if( Preliv%kodp == 1 ) then
           read( ir, * ) Preliv%Zp, Preliv%Bp, Preliv%CQp      
        elseif( Preliv%kodp == 2 ) then
           read( ir, * ) Preliv%nQ
           if( Preliv%nQ > nQmax ) stop ' Preliv: broj tacaka krive protoka? '
           read( ir, * ) ( Preliv%ZZP( i ), i = 1, Preliv%nQ )
           read( ir, * ) ( Preliv%QQP( i ), i = 1, Preliv%nQ )
           Preliv%Zp = Preliv%ZZP( 1 )  ! kota krune
		   Preliv%Bp  = 0.
		   preliv%CQp = 0.
        else
		   stop ' Preliv: kodp = ?'
        endif

		read( ir, * ) Ispust%kodi
		if( Ispust%kodi == 0 ) return
        read( ir, * ) Ispust%Zi, Ispust%Ai, Ispust%CQi 
     END SUBROUTINE UcitavanjeEvakuacioniObjekti




     SUBROUTINE StampanjeEvakuacioniObjekti ( iw, Preliv, Ispust, ii )
        implicit none
        integer, intent( in ) :: iw, ii     
        integer               :: i
		TYPE( PrelivTip )     :: Preliv
		TYPE( IspustTip )     :: Ispust

        if( Preliv%kodp == 0) return
        if( ii == 1 ) write( iw, fmt = "(/5x,A)" ) "Preliv br. 1"
		if( ii == 2 ) write( iw, fmt = "(/5x,A)" ) "Preliv br. 2"
        if(     Preliv%kodp == 1 ) then
           write( iw, 100 )  Preliv%Zp, Preliv%Bp, Preliv%CQp        
        elseif( Preliv%kodp == 2 ) then
           write( iw, 200 ) ( Preliv%ZZP( i ), i = 1, Preliv%nQ )
           write( iw, 300 ) ( Preliv%QQP( i ), i = 1, Preliv%nQ )
        endif

        if( Ispust%kodi == 0 ) return
        if ( ii == 1 ) write( iw, fmt = "(/5x,A)" ) "Ispust br. 1"
        if ( ii == 2 ) write( iw, fmt = "(/5x,A)" ) "Ispust br. 2"
        write( iw, 400 ) Ispust%Zi, Ispust%Ai, Ispust%CQi
        
        100 format( 10x,'Kota krune (mnm):       ',f10.2/            &
                    10x,'Sirina (m):             ',f10.2/            &
                    10x,'Koef. protoka (-):      ',f10.3)
        200 format( 10x,'Kota   Z  (mnm):',5f10.2)
        300 format( 10x,'Protok Q (m3/s):',5f10.2)
        400 format( 10x,'Kota osovine (mnm):     ',f10.2/            &
                    10x,'Povrsina preseka (m2):  ',f10.2/            &
                    10x,'Koef. protoka (-):      ',f10.3)    
     END SUBROUTINE StampanjeEvakuacioniObjekti



     
	 real FUNCTION Qpreliv( ZZ, Preliv )
	    implicit none
		real, intent( in ) :: ZZ
		real               :: flin1, err, OdnosDubina
		TYPE( PrelivTip )  :: Preliv

        OdnosDubina = ( ZZ - Preliv%Zp ) / ( Profil%Zn - Preliv%Zp )
		if( OdnosDubina > 0.67 ) stop 'Potpoljeno prelivanje !'

	    SELECT CASE ( Preliv%kodp )
		   CASE ( 0 )
		         Qpreliv = 0.
           CASE ( 1 ) 
              if( ZZ > Preliv%Zp ) then
			     Qpreliv = Preliv%CQp * Preliv%Bp * 4.429 * ( ZZ - Preliv%Zp )**1.5
              else
			     Qpreliv = 0.
              endif
           CASE ( 2 ) 
              if( ZZ > Preliv%Zp ) then
                 Qpreliv = flin1( Preliv%nQ, Preliv%ZZP, Preliv%QQP, ZZ, err )
                 if( err == 1) stop 'Ekstrapolacija krive preliva ... Proveri ndt, dt !'
              else
			     Qpreliv = 0.
              endif
        END SELECT
     END FUNCTION Qpreliv




     real FUNCTION Qispust( ZZ, Ispust )
	    implicit none
		real, intent( in ) :: ZZ
		TYPE( IspustTip )  :: Ispust
		      
		SELECT CASE( Ispust%kodi )
		   CASE ( 0 )
		      Qispust = 0.
           CASE ( 1 )
		      if( ZZ > Ispust%Zi ) then
		         Qispust = Ispust%CQi * Ispust%Ai * 4.429 * ( ZZ - Profil%Zn )**0.5
		      else
		         Qispust = 0.
              endif
        END SELECT
     END FUNCTION Qispust


  END MODULE EvakuacioniObjekti
 !----------------------------------------------------------------------------------------




	  real FUNCTION flin1( nx, x, y, xx, err)
     implicit none
     integer,             intent(in)  :: nx
     real, dimension(nx), intent(in)  :: x, y
     real                             :: xx
     integer                          :: i, ip, ik, err

     if( xx < x( 1 )  .or.  xx > x( nx ) ) then
        err = 1; return
     endif
     do i = 2, nx
        if( xx <= x (i) ) then
           ip = i-1;  ik = i;  exit
        endif
     enddo
     flin1 = y( ip ) + ( y( ik ) - y( ip ) )/( x( ik ) - x( ip ) ) * ( xx - x( ip ) )
     err = 0
  END FUNCTION flin1   

 

              
  PROGRAM TTA
	 use Akumulacija
	 use EvakuacioniObjekti
	 use RacunskiParametri
 
     implicit none
	 character(12)  :: datio, datu1, dati2, dati4, dati7
     integer        :: i, j, it, nit, err, ier
     real           :: Qu1, Qu2, Omega1, Omega2, Z1, Z2, Z2korigovano, t
     real           :: Qp11, Qp21, Qp12, Qp22, Qi11, Qi21, Qi12, Qi22
     real           :: QprelivSum1, QprelivSum2, QispustSum1, QispustSum2
	 real, external :: flin1    

     
     ! Deklaracija I/O fajlova
     print "(5x,A)", "Ime fajla sa imenima I/O fajlova? "
     read "(A)", datio
     open( UNIT = 10, FILE = datio, STATUS = "old")
     read( 10,"(A)" ) datu1   ! ulazni podaci
     read( 10,"(A)" ) dati2   ! tabelarni prikaz rezultata - protoci
	 read( 10,"(A)" ) dati4   ! graficki prikaz krive povrsine akumulacije
	 read( 10,"(A)" ) dati7   ! graficki prikaz hidrograma
     open( UNIT = 1, FILE = datu1, STATUS = "old" )
     open( UNIT = 2, FILE = dati2, STATUS = "replace" )
	 open( UNIT = 4, FILE = dati4, STATUS = "replace" )
	 open( UNIT = 7, FILE = dati7, STATUS = "replace" )


     ! Ucitavanje ulaznih podataka
     call UcitavanjeAkumulacija        ( 1, 4 )
     call UcitavanjeUlazniHidrogram    ( 1 )
     call UcitavanjeEvakuacioniObjekti ( 1, Preliv1, Ispust1 )
	 call UcitavanjeEvakuacioniObjekti ( 1, Preliv2, Ispust2 )
     call UcitavanjeProfil             ( 1 )
     call UcitavanjeRacunskiParametri  ( 1 )
     close ( 1 )


	 ! Stampanje ulaznih podataka i zaglavlja tabela
     write( 2, "(A)" ) text
	 call StampanjeEvakuacioniObjekti  ( 2, Preliv1, Ispust1, 1 )
	 call StampanjeEvakuacioniObjekti  ( 2, Preliv2, Ispust2, 2 )
     call StampanjeZaglavljaTabele( 2 )


     ! Pocetni uslovi
	 t = 0.; Z1 = Zo

        ! Ulazni hidrogram / Akumulacija
        Qu1 = flin1 ( nQt, tu, Qu, t, err )    
        if( err == 1 ) stop 'Ekstrapolacija ulaznog hidrograma!'
        Omega1 = flin1 ( nzOmega, zn, Omega, Z1, err )
        if( err == 1 ) stop 'Eksptrapolacija krive povrsine ... Proveri ndt, dt !'
        Omega1 = Omega1 * 10000.

        ! Pocetna kota donje vode
	    Profil%Q = Qo 
		if( Profil%Q == 0. ) stop 'Protok donje vode = 0 !?'
		call neli( Profil%zd+0.01, Profil%zo, NormalnaDubina, Profil%Zn, eps, nit, ier )
		if( ier == 1 ) stop 'Pocetno stanje (0) ... ne konvergira normalna dubina !'

        ! Preliv
		Qp11 = Qpreliv( Z1, Preliv1 )
		Qp21 = Qpreliv( Z1, Preliv2 )
        QprelivSum1 = Qp11 + Qp21    ! zbir protoka preko dva preliva

        ! Ispust
		Profil%Q = Qo + QprelivSum1
		if( Profil%Q == 0. ) stop 'Protok donje vode = 0 !?'
		call neli( Profil%zd+0.01, Profil%zo, NormalnaDubina, Profil%Zn, eps, nit, ier )
		if( ier == 1 ) stop 'Pocetno stanje (1) ... ne konvergira normalna dubina !'
		Qi11 = Qispust( Z1, Ispust1 )
		Qi21 = Qispust( Z1, Ispust2 )
        QispustSum1  = Qi11 + Qi21   ! zbir protoka kroz dva ispusta


		! Ukupni izlazni protok i kota donje vode
        Profil%Q = Qo + QprelivSum1 + QispustSum1
		call neli( Profil%zd+0.01, Profil%zo, NormalnaDubina, Profil%Zn, eps, nit, ier )
		if( ier == 1 ) stop 'Pocetno stanje (2) ... ne konvergira normalna dubina !'

        ! Stampanje pocetnog stanja
        call StampanjeRezultata( 2, 7,  t, Z1, Qu1, Qp11, Qp21, QprelivSum1,           &
		                                            Qi11, Qi21, QispustSum1, Profil%Zn )                         

     ! Ciklus po vremenu
     do i = 1, ndt
        t = i * dt / 3600.

        Qu2 = flin1 ( nQt, tu, Qu, t, err )  
        if( err == 1 ) stop 'Ekstrapolacija ulaznog hidrograma!'

        ! Iterativni ciklus
        do it = 1, maxit
           if( it > 1 ) then

		      ! Akumulacija
              Omega2 = flin1 ( nzOmega, zn, Omega, Z2, err )
              if( err == 1 ) stop 'Ekstrapolacija krive povrsine ... Proveri ndt, dt !'
              Omega2 = Omega2 * 10000.

              ! Preliv
	          Qp12 = Qpreliv( Z2, Preliv1 )
		      Qp22 = Qpreliv( Z2, Preliv2 )
              QprelivSum2 = Qp12 + Qp22    ! zbir protoka preko dva preliva

              ! Ispust
		      Qi12 = Qispust( Z2, Ispust1 )
		      Qi22 = Qispust( Z2, Ispust2 )
              QispustSum2  = Qi12 + Qi22   ! zbir protoka kroz dva ispusta           

			  ! Ukupni izlazni protok
			  Profil%Q = Qo + QprelivSum2 + QispustSum2 
              call neli( Profil%zd+0.01, Profil%zo, NormalnaDubina, Profil%Zn, eps, nit, ier )
              if( ier == 1 ) stop 'Ciklus po vremenu ... ne konvergira normalna dubina !'

           elseif( it == 1 ) then
		   		         
			  QprelivSum2 = QprelivSum1; QispustSum2 = QispustSum1; Omega2 = Omega1

           endif

           Z2 = Z1 + ( ( Qu2 + Qu1 ) - ( QprelivSum2 + QprelivSum1 ) -                   &
		               ( QispustSum2 + QispustSum1 )  ) / ( Omega2 + Omega1 ) * dt

           if( it > 1 .and. ABS( Z2 - Z2korigovano ) < eps ) goto 1
           Z2korigovano = Z2
        enddo
        stop 'Ne konvergira u datom broju iteracija!'
    

        ! Stampanje rezultata
1       if( i/npr * npr == i ) then
           call StampanjeRezultata( 2, 7, t, Z2, Qu2, Qp12, Qp22, QprelivSum2,           &
		                                              Qi12, Qi22, QispustSum2, Profil%Zn )                         
	    endif    

		    
		! Prenos vrednosti u sledeci racunski korak
        Z1 = Z2;  Omega1 = Omega2;  Qu1 = Qu2; 
		QprelivSum1 = QprelivSum2;  QispustSum1 = QispustSum2
     enddo


     close( 2 ); close( 4 ); close( 7 )
  END PROGRAM TTA




      SUBROUTINE StampanjeZaglavljaTabele( iw2 )
	    implicit none
		integer, intent( in ) :: iw2 
        write( iw2, 111 )
        111 format(//5x,'REZULTATI PRORACUNA:'//5x,'t',6x,'Z',7x,'Qu',6x,'Qp1',5x,'Qp2',      &
		6x,'Qp',5x,'Qi1',5x,'Qi2',6x,'Qi',5x,'Qp+Qi',3x,'Zdv' /  4x,'(h)',3x,'(mnm)',         &
		2(3x,'(m3/s)'),6(2x,'(m3/s)'),2x,'(mnm)' / ) 
     END SUBROUTINE StampanjeZaglavljaTabele




     SUBROUTINE StampanjeRezultata( iw2, iw7, tw, Zw, Quw, Qp1w, Qp2w, Qpw, Qi1w, Qi2w, Qiw, Zdvw )                         
        implicit none
		integer, intent( in ) :: iw2, iw7
		real,    intent( in ) :: tw, Zw, Quw, Qp1w, Qp2w, Qpw, Qi1w, Qi2w, Qiw, Zdvw 
        write( iw2, fmt = "(12f8.2)" ) tw, Zw, Quw, Qp1w, Qp2w, Qpw, Qi1w, Qi2w, Qiw, Qpw+Qiw, Zdvw                                                    
		write( iw7, fmt = "(7f10.2)" ) tw, Zw, Quw, Qpw, Qiw, Qpw + Qiw  
     END SUBROUTINE StampanjeRezultata


