 !----------------------------------------------------------------------
 ! Program:   TravaQh
 ! Funkcija:  Propusna moc zatravljenog kanala 
 ! Opis:      Korito prizmaticno, trapeznog poprecnog preseka
 !            Racunski model : jednacina jednolikog tecenja,
 !                             koeficijenti trenja na osnovu
 !                             biomehanickih svojstava trave
 !            Racunska metoda: iterativna, polovljenje intervala
 ! Autor:     M. Jovanovic, 19.12.1998.
 ! Okruzenje: Lahey Fortran90, LF90, V.2.00a, 1995.
 !            Microsoft Developer Studio, Visual Fortran V.5, 1997.
 !----------------------------------------------------------------------



 !----------------------------------------------------------------------
  MODULE karakteristike_kanala

     implicit none

     TYPE :: kanal_tip
        character(10) :: oblik_pop_preseka  
        real :: sirina   ! sirina u dnu                 
        real :: kosina   ! kotangens ugla kosine                   
        real :: pad      ! uzduzni nagib dna                   
        real :: A        ! povrsina preseka                   
        real :: Bn       ! sirina nivoa                   
        real :: O        ! okvaseni obim                   
        real :: R        ! hidraulicki radijus                   
        real :: AR23     ! funkcija A*R**(2/3)                   
     END TYPE kanal_tip

     TYPE(kanal_tip) :: kanal
     contains

     SUBROUTINE funkcije_pop_preseka(kanal,h)
        implicit none
        TYPE(kanal_tip), intent(inout)  :: kanal
        real,            intent(in)     :: h

        kanal%A    = h*( kanal%sirina + kanal%kosina*h)
        kanal%Bn   = kanal%sirina + 2.* h * kanal%kosina
        kanal%O    = kanal%sirina + 2.* h * SQRT(kanal%kosina**2 + 1.)
        kanal%R    = kanal%A / kanal%O 
        kanal%AR23 = kanal%A * kanal%R**.6667
     END SUBROUTINE funkcije_pop_preseka

  END MODULE karakteristike_kanala
 !----------------------------------------------------------------------


 !----------------------------------------------------------------------
  MODULE karakteristike_trave

     implicit none

     TYPE :: trava_tip
        character(5) :: vrsta
        real :: visina  ! prosecna visina lista
        real :: MEI     ! sumarna krutost
        real :: uzv_c   ! kriticna smicuca brzina
        real :: Av, Bv  ! koef.logaritamskog rasporeda brzine
     END TYPE trava_tip

     TYPE(trava_tip) :: trava
     contains

     SUBROUTINE krutost(trava)
        implicit none
        TYPE(trava_tip), intent(inout)  :: trava

        if(trava%vrsta == "sveza") then
           trava%MEI = 319. * trava%visina ** 3.3
        elseif(trava%vrsta == "uvela") then
           trava%MEI = 25.4 * trava%visina ** 2.26
        else
           stop "Greska u proracunu krutosti !"
        endif
        trava%uzv_c = MIN( 0.028+6.33*trava%MEI,     &
                           0.23*trava%MEI**0.106 )
     END SUBROUTINE krutost


     SUBROUTINE konstante(trava,uzv)
        implicit none
        TYPE(trava_tip), intent(inout) :: trava
        real,            intent(in)    :: uzv
        real                           :: uu

        uu = uzv / trava%uzv_c
        if(uu <= 1.) then
           trava%Av = 1.85;  trava%Bv = 0.15
        elseif(uu > 1.  .and.  uu <= 1.5) then
           trava%Av = 2.70;  trava%Bv = 0.20
        elseif(uu > 1.5 .and.  uu <= 2.5) then
           trava%Av = 3.08;  trava%Bv = 0.28
        elseif(uu > 2.5) then
           trava%Av = 3.50;  trava%Bv = 0.29
        endif
     END SUBROUTINE konstante

  END MODULE karakteristike_trave 
 !----------------------------------------------------------------------


 !----------------------------------------------------------------------
  MODULE trenje

     implicit none
     real :: tau, kv

     TYPE :: koef_trenja_tip
        real :: lambda
        real :: Manning
     END TYPE koef_trenja_tip

     TYPE(koef_trenja_tip) :: koef_trenja
     contains

     SUBROUTINE koef_trenja_racun(koef_trenja,h,uzv)
        USE karakteristike_trave
        implicit none
        TYPE(koef_trenja_tip), intent(inout) :: koef_trenja
        real,                  intent(in)    :: h, uzv
        real, parameter :: g = 9.81, ro = 1000.
        
        tau = ro * uzv * uzv
        kv  = 0.14 * trava%visina *                                    &
              ( (trava%MEI / tau)**0.25 / trava%visina )**1.59   
        koef_trenja%lambda  = 1./( trava%Av * LOG( h/kv ) + trava%Bv )**2
        koef_trenja%Manning = SQRT( koef_trenja%lambda/ 8./ g ) * h**.1667 
     END SUBROUTINE koef_trenja_racun

  END MODULE trenje
 !----------------------------------------------------------------------


 !----------------------------------------------------------------------
  MODULE racun

     real, allocatable, dimension(:) :: Qu;    real :: Q

     contains

     FUNCTION fh(h) 
        USE karakteristike_kanala
        USE karakteristike_trave
        USE trenje                
        real, parameter :: g = 9.81
        real :: fh, uzv
        uzv = SQRT( g * h * kanal%pad )
        CALL konstante(trava, uzv)
        CALL koef_trenja_racun(koef_trenja, h, uzv)
        CALL funkcije_pop_preseka(kanal, h)   
        fh = (koef_trenja%Manning * Q / kanal%AR23)**2 - kanal%pad
     END FUNCTION fh


     SUBROUTINE neli(xd, xg, f, xo, eps, nit, ier)
        implicit none
        real,    external    :: f
        real,    intent(in)  :: xd, xg, eps
        real,    intent(out) :: xo
        integer, intent(out) :: nit, ier
        integer              :: it
        real                 :: xa, xb, xp, yd, yg, yo, p, po

        xa = xd;   xb = xg;   xp = xa
        if( xa >= xb ) then
           xa = xb;   xb = xp
        endif
        yd = f(xa);   yg = f(xb)
        p = yd*yg
        if( p == 0. ) then
           if( yd == 0. ) xo = xa
           if( yg == 0. ) xo = xb
        nit = 1; ier = 0;  return
        elseif( p < 0. ) then
           nit = INT( LOG( (xb-xa)/eps )/LOG(2.) ) + 1
           do it = 1, nit
              xo = 0.5 * (xa+xb); yo = f(xo)
              po = yd*yo
              if( po == 0. ) then
                 nit = it; ier = 0;  return
              elseif( po < 0. ) then
                 xb = xo; yg = yo
              else
                 xa = xo; yd = yo
              endif
           enddo
           xo = 0.5 * (xa+xb)
           nit = it; ier = 0; return 
        else
           ier = 1; return
        endif
     END SUBROUTINE neli               

  END MODULE racun
 !----------------------------------------------------------------------


  SUBROUTINE ucitavanje_podataka(nq)
     use karakteristike_kanala
     use karakteristike_trave
     use racun
     integer, parameter :: nqmax = 50
     integer            :: nq
     character(12)      :: datio, datu, dati1, dati2
     character(80)      :: text

     NAMELIST/podaci_kanal/ kanal
     NAMELIST/podaci_trava/ trava

     print "( 5x,A )", "Ime fajla sa imenima I/O fajlova?  "
     read  "(A)", datio
     open( UNIT=10,FILE=datio,STATUS="old" )
     read( 10,"(A)" ) datu
     read( 10,"(A)" ) dati1
     read( 10,"(A)" ) dati2
     open( UNIT=1,FILE=datu ,STATUS="old" )
     open( UNIT=2,FILE=dati1,STATUS="replace" )
     open( UNIT=3,FILE=dati2,STATUS="replace" )

     read( 1,fmt="(A)" ) text
     read( 1,NML=podaci_kanal )
     read( 1,NML=podaci_trava )

     read( 1,* ) nq
     if( nq > nqmax) stop 'Premasen maks. broj protoka!'
     allocate( Qu(nq) )
     read( 1,* ) Qu

     close( UNIT=1 )

     write( 2,fmt="(A)" ) text
     write( 3,fmt="(A)" ) text
     write( 2,fmt="(/2x,A,/2x,2A,2(/2x,A,f6.2),/2x,A,f6.4)"  )         &
                 "Podaci o kanalu:",                                   &
                 "Oblik poprecnog preseka: ", kanal%oblik_pop_preseka, &
                 "Sirina u dnu (m):        ", kanal%sirina,            &
                 "Nagib kosine 1:          ", kanal%kosina,            &
                 "Poduzni nagib korita:    ", kanal%pad
     write( 2,fmt="(/2x,A/2x,2A/2x,A,f6.2)" )                          &
                 "Podaci o travi:",                                    &
                 "Vrsta trave:      ", trava%vrsta,                    &
                 "Visina trave (m): ", trava%visina
  END SUBROUTINE ucitavanje_podataka



  PROGRAM TravaQh
    use karakteristike_kanala
    use karakteristike_trave
    use trenje; use racun
    integer                :: nq, nit, ier = 0
    real, parameter        :: g = 9.81, eps = 0.001
    real                   :: hmin = .001, hmax = 20., h
    real                   :: V, Fr

    CALL ucitavanje_podataka( nq )
    CALL krutost( trava )
    write( 3,103 ) trava%vrsta, trava%visina, trava%MEI, trava%uzv_c

    write( 3,101 )             ! zaglavlje - pomocni rezultati
    write( 2,100 )             ! zaglavlje - osnovni rezultati

    do i = 1, nq
       Q = Qu( i )
       call neli(hmin, hmax, fh, h, eps, nit, ier)
       if(ier /= 0) stop 'Nema resenja za normalnu dubinu!'
       uzv = SQRT( g * h * kanal%pad )
       call funkcije_pop_preseka(kanal, h)
       V  = Q / kanal%A
       Fr = Q * Q * kanal%Bn / g /kanal%A**3
       write( 2, fmt = "(2f7.2,3f8.2,2f7.2,2f8.3)"  )                &
            Q, h, kanal%A, kanal%Bn, kanal%O, kanal%R, V, Fr,        &
            koef_trenja%Manning
       write( 3, fmt = "(2f7.2,f8.3,2f7.2,3f8.3,i5)"  )              &
            Q, h, uzv, trava%Av, trava%Bv, kv,                       &
            koef_trenja%lambda, koef_trenja%Manning, nit
    enddo

    100  format(//4x,'Q',6x,'h',7x,'A',7x,'B',7x,'O',6x,'R',6x,'V',  &
         6x,'Fr',6x,'n'/1x,'(m3/s)',3x,'(m)',5x,'(m2)',4x,'(m)',5x,  &    
         '(m)',4x,'(m)',3x,'(m/s)',4x,'(-)',4x,'(..)'/)              

    101  format(//4x,'Q',6x,'h',6x,'u*',6x,'Av',5x,'Bv',5x,'kv',     &
         4x,'lambda',4x,'n',5x,'nit'/1x,'(m3/s)',3x,'(m)',4x,'(m/s)',&
         4x,'(-)',4x,'(-)',4x,'(m)',4x,'(-)',5x,'(..)',3x,'(-)'/)       
    
    103  format(//5x,'Vrsta trave: ',A,                              &
                 /5x,'Visina (m):  ',f6.2,                           &
                 /5x,'Krutost MEI (N m2): ', f8.4,                   &
                 /5x,'Kriticna smicuca brzina u*_c (m/s): ',f8.4/)
    
    close( UNIT=2 ); close( UNIT=3 )
  END PROGRAM TravaQh
 


