 !----------------------------------------------------------------------
 ! Program:   Utka
 ! Funkcija:  Nejednoliko tecenje u prizmaticnom kanalu trapeznog preseka
 ! Opis:      Resavanje jednacine odrzanja kolicine kretanja po odstojanju
 !            dx, metodom konacnih razlika
 !
 ! Autor:     M. Jovanovic, 20.07.1983,  29.11.1999.
 ! Okruzenje: Microsoft Developer Stuido, Visual Fortran V.5, 1997.
 !----------------------------------------------------------------------




 !----------------------------------------------------------------------
  MODULE Kanal
   
     implicit none
	 character(75)      :: text
	 integer            :: i, ip, nit, ier, npp, ismer
     integer, parameter :: maxnpp = 500
     real,    parameter :: ro = 1., g = 9.81, eps = 1.e-3
	 real               :: b, m, Id, CMan, Q, zd1
     real, dimension(:), allocatable :: dx	 

     	 
	 TYPE :: Profil
	    real :: x       ! stacionaza
        real :: h       ! dubina
		real :: zd      ! kota dna
		real :: Z       ! kota nivoa
        real :: A       ! povrsina
        real :: Bn      ! sirina nivoa
        real :: O       ! okvaseni obim
        real :: R       ! hidraulicki radijus
		real :: K       ! hidraulicki modul
        real :: V       ! srednja brzina
        real :: Fr      ! Frudov broj
		real :: Tauo    ! tangencijalni napon
		real :: fs      ! funkcija skoka
     END TYPE Profil

     TYPE( Profil ), dimension(:), allocatable :: pp


     contains



     SUBROUTINE UcitavanjeUlaznihPodataka ( ir )
        integer, intent( in ) :: ir
        read( ir, "(A)" ) text               ! identifikacija
		read( ir, * ) b, m, Id, CMan         ! elementi kanala
		if( Id <= 1.e-6 ) stop 'Ne postoji normalna dubina !'		
		read( ir, * ) Q                      ! protok						
		read( ir, * ) npp                    ! broj profila
		if( npp > maxnpp ) stop 'Premasen dozvoljeni broj profila !'
		allocate( pp( npp ) )                ! zadati broj profila            
        read( ir, * ) pp( 1 )%h, pp( npp )%h ! granicne dubine
		read( ir, * ) pp( 1 )%x, zd1, ismer  ! poc.stac., kota dna, smer
		                                     ! ismer = +1 - racun uzvodno
											 ! ismer = -1 - racun nizvodno
        if( ismer /= 1 .and. ismer /= -1 ) stop 'smer = +1 ili -1 ?!'
		pp( 1 )%zd = zd1                     ! pocetna kota dna
		pp( 1 )%Z = zd1 + pp( 1 )%h          ! pocetna kota nivoa
     END SUBROUTINE UcitavanjeUlaznihPodataka     




     SUBROUTINE FunkcijeProfila ( ip, hh )
	    integer, intent( in ) :: ip
        real,    intent( in ) :: hh
		real                  :: ht
        pp( ip )%Bn   = b + 2. * hh * m
		pp( ip )%A    = hh * ( b + m * hh )
        pp( ip )%O    = b + 2. * hh * SQRT( 1. + m**2 )
        pp( ip )%R    = pp( ip )%A / pp( ip )%O
        pp( ip )%V    = Q / pp( ip )%A
		pp( ip )%K    = pp( ip )%A * pp( ip )%R**0.667 / CMan
        pp( ip )%Fr   = Q * Q * pp( ip )%Bn / g / pp( ip )%A**3
		pp( ip )%Tauo = ro * g * pp( ip )%R * Id
		ht = hh * ( 3. * b + 2. * m * hh ) / ( b + m * hh ) / 6.
		pp( ip )%fs   = ( ht * pp( ip )%A ) + ( Q * Q / g / pp( ip )%A ) 
     END SUBROUTINE FunkcijeProfila





     SUBROUTINE LinijaNivoa( ismer )
	    integer, intent( in ) :: ismer
	    integer               :: npp1
		real                  :: hsr, Asr, Osr, Rsr, Ie

	     npp1 = npp - 1
		 allocate( dx( npp1 ) )

        ! Zadavanje dubina i proracun hidraulickih funkcija      
         do i = 1, npp
            pp( i )%h = pp( 1 )%h + ( pp( npp )%h - pp( 1 )%h ) * ( i - 1 ) / npp1
            call FunkcijeProfila( i, pp( i )%h )
         end do

         ! Proracun odstajanja dx
         do i = 1, npp1
            hsr = 0.5 * ( pp( i )%h + pp( i + 1 )%h ) 
            Asr = hsr * ( b + hsr * m )
            Osr = b + 2. * hsr * SQRT( m * m + 1. )
            Rsr = Asr / Osr
            Ie  = ( Q * CMan )**2 / ( Asr**2 * Rsr**1.3333 )
            dx( i ) = (  pp( i + 1 )%h + pp( i + 1 )%V**2 / 2. / g -      & 
			             pp( i )%h     - pp( i )%V**2     / 2. / g  )     &
                      / ( Id - Ie )
         end do 

         ! Stacionaza = kumulativno odstojanje
         do i = 1, npp1
            if( ismer ==  1 ) pp( i + 1 )%x = pp( i )%x - ismer * dx( i )
            if( ismer == -1 ) pp( i + 1 )%x = pp( i )%x + ismer * dx( i )
         end do

         ! Kote dna i nivoa 
		 do i = 2, npp
           pp( i )%zd = zd1 + ismer * ABS( pp( i )%x ) * Id
           pp( i )%Z = pp( i )%zd + pp( i )%h
        end do

     END SUBROUTINE LinijaNivoa




     real FUNCTION fn( hh )
		real, intent( in ) :: hh        
		call FunkcijeProfila( 1, hh )
        fn = ( Q / pp( 1 )%K )**2 - Id
	END FUNCTION fn




     real FUNCTION fk ( hh )
        real, intent( in ) :: hh
        call FunkcijeProfila( 1, hh )
        fk = 1. - pp( 1 )%Fr
     END FUNCTION fk




     SUBROUTINE neli(xd, xg, f, xo, eps, nit, ier)
        implicit none
        real,    external       :: f
        real,    intent( in  )  :: xd, xg, eps
        real,    intent( out )  :: xo
        integer, intent( out )  :: nit, ier
        integer                 :: it
        real                    :: xa, xb, xp, yd, yg, yo, p, po

        xa = xd;   xb = xg;   xp = xa
        if( xa >= xb ) then
           xa = xb;   xb = xp
        endif
        yd = f( xa );   yg = f( xb )
        p = yd * yg
        if( p == 0. ) then
           if( yd == 0. ) xo = xa
           if( yg == 0. ) xo = xb
        nit = 1; ier = 0;  return
        elseif( p < 0. ) then
           nit = INT( LOG( ( xb - xa ) / eps )/LOG(2.) ) + 1
           do it = 1, nit
              xo = 0.5 * ( xa + xb ); yo = f( xo )
              po = yd * yo
              if( po == 0. ) then
                 nit = it; ier = 0;  return
              elseif( po < 0. ) then
                 xb = xo; yg = yo
              else
                 xa = xo; yd = yo
              endif
           enddo
           xo = 0.5 * ( xa + xb )
           nit = it; ier = 0; return 
        else
           ier = 1; return
        endif
     END SUBROUTINE neli               




     SUBROUTINE StampanjeUlaznihPodataka( iw )
	    integer, intent( in ):: iw
		write( iw, "(A)"  ) text
		write( iw, "(/5x,A/)" ) "Ulazni podaci:"
		write( iw, 101) b, m, Id, Cman, Q
		101 format(5x,'Sirina kanala u dnu (m): ',f8.2/                   &
		           5x,'Nagib kosina          1: ',f8.2/                   &
				   5x,'Uzduzni nagib dna (-):   ',f8.4/                   &
				   5x,'Maningov koef. (m-1/2s): ',f8.3/                   &
				   5x,'Protok (m3/s):           ',f8.2/ )
     END SUBROUTINE StampanjeUlaznihPodataka





     SUBROUTINE StampanjeZaglavlja( iw )
	    integer, intent( in ) :: iw
        write( iw, 102 )
        102 format(7x,'x',7x,'zd',7x,'Z',7x,'h',7x,'A',6x,'O',            &
		6x,'R',6x,'V',6x,'E',6x,'Fr',6x,'fs',6x,'Tauo'/                   &
        6x,'(m)',4x,'(mnm)',4x,'(mnm)',4x,'(m)',5x,'(m2)',3x,'(m)',       &
		4x,'(m)',3x,'(m/s)',2x,'(mnm)',4x,'(-)',4x,'(m3)',5x,'(kPa)'/)
     END SUBROUTINE StampanjeZaglavlja





     SUBROUTINE StampanjeRezultata( iw, iwg )
        integer, intent( in ) :: iw, iwg  
        do  i = 1, npp
           write( iw,  103 ) pp( i )%x,  pp( i )%zd, pp( i )%Z, pp( i )%h,& 
			                 pp( i )%A,  pp( i )%O,  pp( i )%R, pp( i )%V,&
                             pp( i )%Z + pp( i )%V * pp( i )%V / 2. / g,  &
						     pp( i )%Fr, pp( i )%fs, pp( i )%Tauo
           write( iwg, 104 ) pp( i )%x,  pp( i )%zd, pp( i )%Z,           &
		                     pp( i )%Z + pp( i )%V * pp( i )%V / 2. / g
        end do
        103 format(f10.3,2f9.3,f7.3,f8.2,3f7.2,f8.3,f7.3,f8.3,f10.6)
		104 format(4f12.3)
     END SUBROUTINE StampanjeRezultata


  END MODULE Kanal
 !----------------------------------------------------------------------




     
     PROGRAM Utka
        use Kanal

        character(12)   :: datio, datu1, dati2, dati3
		real, parameter :: hMin = 0.01, hMax = 20. 
		real            :: hn, hk   

        ! Definicija I/O fajlova
        print "(5x,A)", "Ime fajla sa imenima I/O fajlova? "
        read "(A)", datio
        open( UNIT = 10, FILE = datio, STATUS = "old" )
        read( 10, "(A)" ) datu1 ! ulazni podaci
        read( 10, "(A)" ) dati2 ! rezultati - tabelarni prikaz
		read( 10, "(A)" ) dati3 ! rezultati - graficki  prikaz
        open( UNIT = 1, FILE = datu1, STATUS = "old" )
        open( UNIT = 2, FILE = dati2, STATUS = "replace" )
		open( UNIT = 3, FILE = dati3, STATUS = "replace" ) 
        
        ! Ucitavanje ulaznih podataka
        call UcitavanjeUlaznihPodataka( 1 )
		call StampanjeUlaznihPodataka ( 2 )

        ! Normalna i kriticna dubina
        call neli( hMin, hMax, fn, hn, eps, nit, ier )                      
        if( ier == 1 ) stop 'Skor: ne konvergira normalna dubina !'
		call neli( hMin, hMax, fk, hk, eps, nit, ier )
		if( ier == 1 ) stop 'Skor: ne konvergira kriticna dubina !'
		write( 2, "(5x,A/)" ) "Rezultati:"
        write( 2, "(5x,A,f12.3/5x,A,f12.3/)" ) "Normalna dubina (m): ", hn, &
		                                       "Kriticna dubina (m): ", hk
        ! Proracun linije nivoa
		call LinijaNivoa( ismer )       
		 
        ! Stampa
		call StampanjeZaglavlja( 2 )
        call StampanjeRezultata( 2, 3 )

        close( 1 );  close(	2 );  close( 3 )
     END PROGRAM Utka
























         