 !----------------------------------------------------------------------------------
 ! Program:   WW
 ! Funkcija:  Brzina neometanog talozenja cestice u mirnoj vodi
 ! Opis:      Koriste se tri metode proracuna:
 !            (1) Iterativni algoritam na bazi koeficijenta otpora
 !            (2) Direktni postupak na bazi empirijskih izraza
 !                J.Valembois, LA HOUILLE BLANCHE No.2 1983.
 !            (3) Direktni postupak DELFT HYDRAULICS
 ! Autor:     M. Jovanovic, 19.11.1990., 3.12.1998.
 ! Okruzenje: Lahey Fortran90, LF90, V.2.00a, 1995.
 !            Microsoft Developer Studio, Visual Fortran V.6,6 2001.
 !----------------------------------------------------------------------------------



 !----------------------------------------------------------------------------------
  MODULE UlazniPodaci

     implicit none
     integer                          :: nd          ! ukupno cestica
     integer, parameter               :: ndmax = 50  ! ogranicenje
     real, allocatable, dimension (:) :: d           ! precnik
     real                             :: ros         ! gustina cestice
	 real                             :: T           ! temperatura vode

     contains        

     SUBROUTINE Ulaz1 
        character(75) :: text
        character(12) :: datu, dati

        print "(5x,A)", "Ime fajla sa ulaznim podacima? "
        read  "(A)", datu
        print "(5x,A)", "Ime fajla sa rezultatima?      "
        read  "(A)", dati
        open(1, FILE = datu, STATUS = "old"    ) ! ulazni podaci
        open(2, FILE = dati, STATUS = "replace") ! rezultati 

        read ( 1, "(A)" ) text ! identifikacija
        write( 2, "(A)" ) text 
        read ( 1, * ) T, ros   ! temperatura, gustina cestice
        read ( 1, * ) nd       ! ukupno podataka
        if( nd > ndmax )  stop 'Previse podataka! '
        allocate ( d( nd ) )   ! dimenzionisanje niza
        read(1, *) d           ! krupnoce zrna [mm]
        d = d * 1.e-3          ! krupnoce zrna [m]
      END SUBROUTINE Ulaz1



     SUBROUTINE Ulaz2
        print "(5x,A)", "Temperatura vode (0C)?  "
        read *, T
        print "(5x,A)", "Gustina cestice (t/m3)? "
        read *, ros
        nd = 1
        print "(5x,A)", "Srednji precnik (mm)?   "
        read *, d(nd)
        d(nd) = d(nd) * 1.e-03
     END SUBROUTINE Ulaz2



  END MODULE UlazniPodaci
 !----------------------------------------------------------------------------------



 
 !----------------------------------------------------------------------------------
  MODULE RacunskiAlgoritmi
    use UlazniPodaci

	 real, parameter :: ro = 1. ! gustina vode
	 real            :: vis     ! kinematicki koeficijent viskoznosti vode
	 real            :: W       ! brzina tonjenja cestice
     real            :: Re      ! Reynoldsov broj
	 real            :: Cd      ! koeficijent otpora


  contains


     SUBROUTINE fwi( ds, W )
        real,    parameter :: eps = 0.0001
        integer, parameter :: maxit = 50
        integer            :: it
        real :: ds, Cd1, dro, c, f

        dro = ros - 1.
        c = (4./3.) * ds * 9.81 * dro / ro

        ! Oblast turbulentnog tonjenja
        Cd = 0.4;   W  = SQRT( c / Cd );   Re = W * ds / vis
        if(Re > 3000.) return

        ! Prelazna oblast - pretraga zone resenja
        Cd =   1.;  W  = SQRT( c / Cd );   Re = W * ds / vis
        if(Re > 100.) goto 1
        Cd =  10.;  W  = SQRT( c / Cd );   Re = W * ds / vis
        if(Re >   2.) goto 1
        Cd = 200.;  W  = SQRT( c / Cd );   Re = W * ds / vis
        if(Re >   .1) goto 1
        goto 2        

        ! Iterativno odredjivanje Cd Newton-ovom metodom:
        ! Cd1 = Cd - f(Cd) / f'(Cd)

1       continue

        do it = 1, maxit
           if(Re < 0.1) goto 2
           f = Cd - (24./Re) - ( 3. / SQRT(Re) ) - 0.34
           if( ABS(f) < eps ) return
           Cd1 = Cd - f
           if(Cd1 < 0.34) Cd = 0.34
           Cd = Cd1
           W  = SQRT( c / Cd )
           Re = W * ds / vis
        end do        
        stop 'fWi: ne konvergira!'

        ! Oblast Stokes-ovog zakona
2       W  = 9.81 * dro * ds * ds / vis / ro / 18.
        Re = W * ds / vis
     END SUBROUTINE fwi   
 
        

     real FUNCTION fw1( ds )
        real :: ds, Gn, Rw
        Gn = 9.81 * ( ros - ro ) * ds**3 / vis**2
        Rw =.0556 * Gn
        if(Gn > 10.  .and. Gn <= 300.  )  Rw =.0784 * Gn**.850
        if(Gn > 300. .and. Gn <= 67000.)  Rw =.1711 * Gn**.713
        if(Gn > 67000.                 )  Rw =1.826 * Gn**.5
        fw1 = Rw * vis / ds
     END FUNCTION fw1



     real FUNCTION fw2( ds )
        real :: ds, s, s1
        s  = ros / ro;  s1 = ( s - 1.) * 9.81
        if(    ds >=   1.e-06 .and. ds <  100.e-06  ) then
           fw2 = s1 * ds * ds / 18. / ro / vis
        elseif( ds >= 100.e-06 .and. ds < 1000.e-06 ) then
           fw2 = 10.* vis * ( SQRT( 1. + 0.01 * s1 * ds**3 / vis**2 ) - 1. ) / ds
        elseif( ds >= 1000.e-06 ) then
           fw2 = 1.1 * SQRT( s1 * ds )
        else
           stop 'fw2: d van datih opsega!'
        endif
     END FUNCTION fw2
 

  END MODULE RacunskiAlgoritmi
 !----------------------------------------------------------------------------------





     PROGRAM WW
        use RacunskiAlgoritmi

        integer        :: kod1, kod2, i 
        real           :: Wi, W1, W2

9       print "(3(/5x,A))", "Podaci se ucitavaju:",                &
                            "1-iz fajla; 2-interaktivno",          &
                            "Izbor? "  
        read *, kod1
        if(kod1 /= 1 .and. kod1 /= 2) goto 9
        if(kod1 == 1) call Ulaz1 ! ucitavanje iz fajla
1       if(kod1 == 2) call Ulaz2 ! interaktivno ucitavanje
     
        vis = (1.14 -.031 * (T-15.) +.00068 * (T-15.)**2) * 1.e-06

        if( kod1 == 1) then
           write(2,100) T, vis
           write(2,101)
        endif
        print 100, T, vis
        print 101

        do i = 1, nd
           call fwi( d(i), Wi ) ! iterativni postupak
           W1 = fw1( d(i) )     ! Valembois
           W2 = fw2( d(i) )     ! Van Rijn
           if(kod1 == 1)                                           &
              write( 2, 102 ) i, d(i)*1000., W1, W2, Wi, Re, Cd
              print     102,  i, d(i)*1000., W1, W2, Wi, Re, Cd
        end do

        if(kod1 == 1) then
            close(1); close(2)
        else
           print "(/5x,A)", "Ponoviti proracun (0/1)? "
           read *, kod2
           if(kod2 == 1) goto 1
        endif

        100 format(/5x,'T =',f6.0,' oC;   vis =', e10.2,' m2/s')
        101 format(/5x,'No.',4x,'d',7x,'W-f1',8x,'W-f2',8x,'W-it', &
            8x,'Re',10x,'Cd'/10x,'(mm)',6x,'(m/s)',7x,'(m/s)',7x,  &
            '(m/s)',7x,'(-)',9x,'(-)'/)
        102 format( 5x,i2,2x,f6.3,3(2x,f10.6),2(2x,e10.4))
     END PROGRAM WW